# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.service_client import ServiceClient
from msrest import Configuration, Serializer, Deserializer
from .version import VERSION
from msrest.pipeline import ClientRawResponse
from msrest.exceptions import HttpOperationError
from . import models


class EnergyCapApiConfiguration(Configuration):
    """Configuration for EnergyCapApi
    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param str base_url: Service URL
    """

    def __init__(
            self, base_url=None):

        if not base_url:
            base_url = 'http://localhost'

        super(EnergyCapApiConfiguration, self).__init__(base_url)

        self.add_user_agent('EnergyCapSdk/{}'.format(VERSION))


class EnergyCapApi(object):
    """EnergyCAP API

    :ivar config: Configuration for client.
    :vartype config: EnergyCapApiConfiguration

    :param str base_url: Service URL
    """

    def __init__(
            self, base_url=None):

        self.config = EnergyCapApiConfiguration(base_url)
        self._client = ServiceClient(None, self.config)

        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self.api_version = 'v3'
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)


    def get_account(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one account.

        :param account_id: The account identifier for the account being
         retrieved
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountsAccountResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.AccountsAccountResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account.metadata['url']
        path_format_arguments = {
            'accountID': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountsAccountResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account.metadata = {'url': '/api/v3/account/{accountID}'}

    def edit_account(
            self, account_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits an account.

        :param account_id: The identifier for the account being edited
        :type account_id: int
        :param dto: Data representing the account being edited
        :type dto: ~energycap.sdk.models.AccountsAccountEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountsAccountResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.AccountsAccountResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_account.metadata['url']
        path_format_arguments = {
            'accountID': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'AccountsAccountEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountsAccountResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_account.metadata = {'url': '/api/v3/account/{accountID}'}

    def delete_account(
            self, account_id, delete_bills=None, custom_headers=None, raw=False, **operation_config):
        """Deletes an account. Currently NO confirmation is required.

        :param account_id: The identifier for the account being deleted
        :type account_id: int
        :param delete_bills: Should this cascade to delete any bills
         associated with this meter
        :type delete_bills: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_account.metadata['url']
        path_format_arguments = {
            'accountID': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if delete_bills is not None:
            query_parameters['deleteBills'] = self._serialize.query("delete_bills", delete_bills, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_account.metadata = {'url': '/api/v3/account/{accountID}'}

    def get_accounts(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all accounts.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.AccountsAccountResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_accounts.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountsAccountResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_accounts.metadata = {'url': '/api/v3/account'}

    def create_account(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates an account.

        :param dto: Data representing the account being created
        :type dto: ~energycap.sdk.models.AccountsAccountCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountsAccountResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.AccountsAccountResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_account.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'AccountsAccountCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountsAccountResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_account.metadata = {'url': '/api/v3/account'}

    def search_accounts(
            self, query=None, include_inactive_accounts=None, custom_headers=None, raw=False, **operation_config):
        """Search for accounts
        Search for accounts whose account code, address, old account code,
        meter serial number, or meter address contains "query".

        :param query: The string to search for
        :type query: str
        :param include_inactive_accounts: Flag to determine if inactive
         accounts should be included in the search results
        :type include_inactive_accounts: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.SearchSearchAccountMeterBillEntryDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.search_accounts.metadata['url']

        # Construct parameters
        query_parameters = {}
        if query is not None:
            query_parameters['query'] = self._serialize.query("query", query, 'str')
        if include_inactive_accounts is not None:
            query_parameters['includeInactiveAccounts'] = self._serialize.query("include_inactive_accounts", include_inactive_accounts, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[SearchSearchAccountMeterBillEntryDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    search_accounts.metadata = {'url': '/api/v3/account/search'}

    def get_account_udfs(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for a particular account.

        :param account_id: The account identifier for which UDF values are
         being retrieved
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_udfs.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_udfs.metadata = {'url': '/api/v3/account/{accountId}/udf'}

    def edit_account_ud_fs(
            self, account_id, udfs=None, custom_headers=None, raw=False, **operation_config):
        """Edits a account udf values.

        :param account_id: The identifier for the account being edited
        :type account_id: int
        :param udfs: Data representing the account udfs being edited
        :type udfs: list[~energycap.sdk.models.UDFUDFValueDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_account_ud_fs.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if udfs is not None:
            body_content = self._serialize.body(udfs, '[UDFUDFValueDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_account_ud_fs.metadata = {'url': '/api/v3/account/{accountId}/udf'}

    def get_accounts_udfs(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for all accounts.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFAccountUDFResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_accounts_udfs.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFAccountUDFResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_accounts_udfs.metadata = {'url': '/api/v3/account/udf'}

    def get_account_rates(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all rates for a particular account.

        :param account_id: The account identifier for which rate values are
         being retrieved
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.AccountsAccountRateResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_rates.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountsAccountRateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_rates.metadata = {'url': '/api/v3/account/{accountId}/rate'}

    def get_account_templates(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all templates for a particular account.

        :param account_id: The account identifier for which template values
         are being retrieved
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.AccountsAccountTemplateResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_templates.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountsAccountTemplateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_templates.metadata = {'url': '/api/v3/account/{accountId}/template'}

    def get_account_alerts(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all alerts for an account.

        :param account_id: The identifier for the account
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.AccountsAccountAlertResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_alerts.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountsAccountAlertResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_alerts.metadata = {'url': '/api/v3/account/{accountId}/alert'}

    def create_account_alert(
            self, account_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates an account alert.

        :param account_id: The identifier for the account to create alert is
         on
        :type account_id: int
        :param dto: Data representing the account alert being created
        :type dto: ~energycap.sdk.models.AccountsAccountAlertRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountsAccountAlertResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.AccountsAccountAlertResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_account_alert.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'AccountsAccountAlertRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountsAccountAlertResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_account_alert.metadata = {'url': '/api/v3/account/{accountId}/alert'}

    def get_account_alert(
            self, account_id, alert_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves an alert for an account.

        :param account_id: The identifier for the account the alert is on
        :type account_id: int
        :param alert_id: The identifier for the alert to return
        :type alert_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountsAccountAlertResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.AccountsAccountAlertResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_alert.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'alertId': self._serialize.url("alert_id", alert_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountsAccountAlertResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_alert.metadata = {'url': '/api/v3/account/{accountId}/alert/{alertId}'}

    def update_account_alert(
            self, account_id, alert_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Updates an account alert.

        :param account_id: The identifier for the account the alert to update
         is on
        :type account_id: int
        :param alert_id: The identifier for the account alert to update
        :type alert_id: int
        :param dto: Data representing the account alert being created
        :type dto: ~energycap.sdk.models.AccountsAccountAlertRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountsAccountAlertResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.AccountsAccountAlertResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_account_alert.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'alertId': self._serialize.url("alert_id", alert_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'AccountsAccountAlertRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountsAccountAlertResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_account_alert.metadata = {'url': '/api/v3/account/{accountId}/alert/{alertId}'}

    def delete_account_alert(
            self, account_id, alert_id, custom_headers=None, raw=False, **operation_config):
        """Deletes an account alert.

        :param account_id: The identifier for the account the alert is on
        :type account_id: int
        :param alert_id: The identifier for the alert to delete
        :type alert_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_account_alert.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'alertId': self._serialize.url("alert_id", alert_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_account_alert.metadata = {'url': '/api/v3/account/{accountId}/alert/{alertId}'}

    def get_default_billing_period(
            self, account_id, start_date=None, end_date=None, custom_headers=None, raw=False, **operation_config):
        """For a given accountId, startDate and endDates for a bill get a billing
        period recommendation
        The billing period will be in YYYYMM format YYYY represents the 4 digit
        year and MM is the two digit month 01 for Jan to 12 for Dec.

        :param account_id: The identifier for the account for which bill is
         being created
        :type account_id: int
        :param start_date: The start date of the bill
        :type start_date: datetime
        :param end_date: The end date of the bill
        :type end_date: datetime
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonSingleValueDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonSingleValueDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_default_billing_period.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if start_date is not None:
            query_parameters['startDate'] = self._serialize.query("start_date", start_date, 'iso-8601')
        if end_date is not None:
            query_parameters['endDate'] = self._serialize.query("end_date", end_date, 'iso-8601')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonSingleValueDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_default_billing_period.metadata = {'url': '/api/v3/account/{accountId}/billingPeriod'}

    def move_account(
            self, parent_cost_center_id, account_ids=None, custom_headers=None, raw=False, **operation_config):
        """Move list of accounts to be under parentCostCenterId.

        :param parent_cost_center_id: The identifier for the new parent
         costCenter
        :type parent_cost_center_id: int
        :param account_ids: List of accountIds to move under the
         parentCostCenterId
        :type account_ids: list[int]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.move_account.metadata['url']
        path_format_arguments = {
            'parentCostCenterId': self._serialize.url("parent_cost_center_id", parent_cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if account_ids is not None:
            body_content = self._serialize.body(account_ids, '[int]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    move_account.metadata = {'url': '/api/v3/parentCostCenter/{parentCostCenterId}/account'}

    def get_account_actual_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by account for a given number
        of years- Result includes cost, use, unit cost on an yearly basis. It
        also includes commodity level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.AccountDigestAccountDigestActualYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_actual_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountDigestAccountDigestActualYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_actual_yearly_list.metadata = {'url': '/api/v3/account/digest/actual/yearly'}

    def get_account_actual_yearly(
            self, account_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by account for a given number
        of years- Result includes cost, use, unit cost on an yearly basis. It
        also includes commodity level break up.

        :param account_id: The account to retrieve data for
        :type account_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountDigestAccountDigestActualYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.AccountDigestAccountDigestActualYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_actual_yearly.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountDigestAccountDigestActualYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_actual_yearly.metadata = {'url': '/api/v3/account/{accountId}/digest/actual/yearly'}

    def get_account_actual_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by account for a given number
        of years- Result includes cost, use, unit cost on a monthly basis. It
        also includes commodity level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 24 months
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.AccountDigestAccountDigestActualMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_actual_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountDigestAccountDigestActualMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_actual_monthly_list.metadata = {'url': '/api/v3/account/digest/actual/monthly'}

    def get_account_actual_monthly(
            self, account_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by account for a given number
        of years- Result includes cost, use, unit cost on a monthly basis. It
        also includes commodity level break up.

        :param account_id: The account to retrieve data for
        :type account_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 24 months
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountDigestAccountDigestActualMonthlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.AccountDigestAccountDigestActualMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_actual_monthly.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountDigestAccountDigestActualMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_actual_monthly.metadata = {'url': '/api/v3/account/{accountId}/digest/actual/monthly'}

    def create_account_meter(
            self, account_meter_dto=None, custom_headers=None, raw=False, **operation_config):
        """Links an account and meter together.  Meters must be linked to an
        active account to be usable in EnergyCAP.  Once an account and meter
        are created individually,
        they must be linked with this API.

        :param account_meter_dto: Data representing the account and meter
         being linked
        :type account_meter_dto:
         list[~energycap.sdk.models.AccountMeterAccountMeterRequestDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.AccountMeterAccountMeterResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_account_meter.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if account_meter_dto is not None:
            body_content = self._serialize.body(account_meter_dto, '[AccountMeterAccountMeterRequestDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountMeterAccountMeterResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_account_meter.metadata = {'url': '/api/v3/accountmeter'}

    def edit_account_meter(
            self, account_meter_id, account_meter_edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit an account meter relationship.

        :param account_meter_id: The id of the account meter record to edit
        :type account_meter_id: int
        :param account_meter_edit_dto: Data used to update the account and
         meter
        :type account_meter_edit_dto:
         ~energycap.sdk.models.AccountMeterAccountMeterEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountMeterAccountMeterResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.AccountMeterAccountMeterResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_account_meter.metadata['url']
        path_format_arguments = {
            'accountMeterId': self._serialize.url("account_meter_id", account_meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if account_meter_edit_dto is not None:
            body_content = self._serialize.body(account_meter_edit_dto, 'AccountMeterAccountMeterEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountMeterAccountMeterResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_account_meter.metadata = {'url': '/api/v3/accountmeter/{accountMeterId}'}

    def edit_account_rate_list(
            self, account_meter_id, account_rate_dto_list=None, custom_headers=None, raw=False, **operation_config):
        """Edit all AccountRate relationships.

        :param account_meter_id:
        :type account_meter_id: int
        :param account_rate_dto_list:
        :type account_rate_dto_list:
         list[~energycap.sdk.models.AccountMeterAccountRateRequestDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.AccountMeterAccountMeterRateResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_account_rate_list.metadata['url']
        path_format_arguments = {
            'accountMeterId': self._serialize.url("account_meter_id", account_meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if account_rate_dto_list is not None:
            body_content = self._serialize.body(account_rate_dto_list, '[AccountMeterAccountRateRequestDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountMeterAccountMeterRateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_account_rate_list.metadata = {'url': '/api/v3/accountmeter/{accountMeterId}/rate'}

    def create_account_rate(
            self, account_meter_id, account_rate_dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates an AccountRate Relationship.  Linking AccountMeter and Rate.

        :param account_meter_id:
        :type account_meter_id: int
        :param account_rate_dto:
        :type account_rate_dto:
         ~energycap.sdk.models.AccountMeterAccountRateRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountMeterAccountMeterRateResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.AccountMeterAccountMeterRateResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_account_rate.metadata['url']
        path_format_arguments = {
            'accountMeterId': self._serialize.url("account_meter_id", account_meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if account_rate_dto is not None:
            body_content = self._serialize.body(account_rate_dto, 'AccountMeterAccountRateRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountMeterAccountMeterRateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_account_rate.metadata = {'url': '/api/v3/accountmeter/{accountMeterId}/rate'}

    def edit_form_template_list(
            self, account_meter_id, form_template_request_dto_list=None, custom_headers=None, raw=False, **operation_config):
        """Edit all FormTemplate relationships.

        :param account_meter_id:
        :type account_meter_id: int
        :param form_template_request_dto_list:
        :type form_template_request_dto_list:
         list[~energycap.sdk.models.AccountMeterFormTemplateRequestDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.AccountMeterFormTemplateResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_form_template_list.metadata['url']
        path_format_arguments = {
            'accountMeterId': self._serialize.url("account_meter_id", account_meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if form_template_request_dto_list is not None:
            body_content = self._serialize.body(form_template_request_dto_list, '[AccountMeterFormTemplateRequestDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AccountMeterFormTemplateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_form_template_list.metadata = {'url': '/api/v3/accountmeter/{accountMeterId}/template'}

    def create_form_template(
            self, account_meter_id, form_template_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Assigns a new template to an account meter.
        The new template's begin date must be after the begin date of all
        existing template assignments.

        :param account_meter_id:
        :type account_meter_id: int
        :param form_template_request_dto:
        :type form_template_request_dto:
         ~energycap.sdk.models.AccountMeterFormTemplateRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AccountMeterFormTemplateResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.AccountMeterFormTemplateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_form_template.metadata['url']
        path_format_arguments = {
            'accountMeterId': self._serialize.url("account_meter_id", account_meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if form_template_request_dto is not None:
            body_content = self._serialize.body(form_template_request_dto, 'AccountMeterFormTemplateRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AccountMeterFormTemplateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_form_template.metadata = {'url': '/api/v3/accountmeter/{accountMeterId}/template'}

    def delete_template_assignment(
            self, account_meter_id, form_template_id, custom_headers=None, raw=False, **operation_config):
        """Delete an template assignment from an account meter
        If we are deleting a template assignment in the middle of two other
        template assignments, then we will move back the next template's start
        date
        If we are deleting the last template(current template), the extend
        second last template's end date to match the account meter's end date
        If we are deleting the first template pull the second template's begin
        date to match account meter's begin date.

        :param account_meter_id:
        :type account_meter_id: int
        :param form_template_id:
        :type form_template_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_template_assignment.metadata['url']
        path_format_arguments = {
            'accountMeterId': self._serialize.url("account_meter_id", account_meter_id, 'int'),
            'formTemplateId': self._serialize.url("form_template_id", form_template_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_template_assignment.metadata = {'url': '/api/v3/accountmeter/{accountMeterId}/template/{formTemplateId}'}

    def get_bill_activity(
            self, date_parameter=None, custom_headers=None, raw=False, **operation_config):
        """List of a bill IDs that were created or modified since the given date.

        :param date_parameter: Created or modified datetime; Defaults to
         yesterday at the current time
        :type date_parameter: datetime
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ActivityActivityDto] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_activity.metadata['url']

        # Construct parameters
        query_parameters = {}
        if date_parameter is not None:
            query_parameters['date'] = self._serialize.query("date_parameter", date_parameter, 'iso-8601')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ActivityActivityDto]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_activity.metadata = {'url': '/api/v3/activity/bill'}

    def get_system_settings(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.AdminAdminSettingsResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_system_settings.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AdminAdminSettingsResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_system_settings.metadata = {'url': '/api/v3/admin/systemsetting'}

    def update_system_settings(
            self, admin_settings_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param admin_settings_dto:
        :type admin_settings_dto:
         list[~energycap.sdk.models.AdminAdminSettingsRequestDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.AdminAdminSettingsResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_system_settings.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if admin_settings_dto is not None:
            body_content = self._serialize.body(admin_settings_dto, '[AdminAdminSettingsRequestDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AdminAdminSettingsResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_system_settings.metadata = {'url': '/api/v3/admin/systemsetting'}

    def get_account_periods(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves the list of Account Periods.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.AdminAccountPeriodResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_periods.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[AdminAccountPeriodResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_periods.metadata = {'url': '/api/v3/admin/accountperiod'}

    def login(
            self, login_request=None, custom_headers=None, raw=False, **operation_config):
        """

        :param login_request:
        :type login_request: ~energycap.sdk.models.AuthenticationLoginRequest
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: AuthenticationLoginResponse or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.AuthenticationLoginResponse or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.login.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if login_request is not None:
            body_content = self._serialize.body(login_request, 'AuthenticationLoginRequest')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('AuthenticationLoginResponse', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    login.metadata = {'url': '/api/v3/login'}

    def beta(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: dict or ClientRawResponse if raw=true
        :rtype: dict[str, bool] or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.beta.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('{bool}', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    beta.metadata = {'url': '/api/internal/beta'}

    def logout(
            self, custom_headers=None, raw=False, **operation_config):
        """Logout user- if they were logged in using a cookie.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.logout.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    logout.metadata = {'url': '/api/v3/logout'}

    def create_token_for_partition(
            self, partition=None, custom_headers=None, raw=False, **operation_config):
        """My.EnergyCap.Com needs a way to get a token from a partition name. Pass
        in a partition, create and return a token for provided partition.

        :param partition:
        :type partition: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonSingleValueDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonSingleValueDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_token_for_partition.metadata['url']

        # Construct parameters
        query_parameters = {}
        if partition is not None:
            query_parameters['partition'] = self._serialize.query("partition", partition, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonSingleValueDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_token_for_partition.metadata = {'url': '/api/v3/exchange/eco'}

    def get_batches(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BatchBatchResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_batches.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BatchBatchResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_batches.metadata = {'url': '/api/v3/batch'}

    def open_batch(
            self, batch_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param batch_dto:
        :type batch_dto: ~energycap.sdk.models.BatchBatchCreateDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BatchBatchResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BatchBatchResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.open_batch.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if batch_dto is not None:
            body_content = self._serialize.body(batch_dto, 'BatchBatchCreateDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BatchBatchResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    open_batch.metadata = {'url': '/api/v3/batch'}

    def get_batch(
            self, batch_id, custom_headers=None, raw=False, **operation_config):
        """

        :param batch_id:
        :type batch_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BatchBatchResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BatchBatchResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_batch.metadata['url']
        path_format_arguments = {
            'batchId': self._serialize.url("batch_id", batch_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BatchBatchResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_batch.metadata = {'url': '/api/v3/batch/{batchId}'}

    def edit_batch(
            self, batch_id, batch_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param batch_id:
        :type batch_id: int
        :param batch_dto:
        :type batch_dto: ~energycap.sdk.models.BatchBatchEditDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BatchBatchResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BatchBatchResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_batch.metadata['url']
        path_format_arguments = {
            'batchId': self._serialize.url("batch_id", batch_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if batch_dto is not None:
            body_content = self._serialize.body(batch_dto, 'BatchBatchEditDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BatchBatchResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_batch.metadata = {'url': '/api/v3/batch/{batchId}'}

    def delete_batch(
            self, batch_id, custom_headers=None, raw=False, **operation_config):
        """

        :param batch_id:
        :type batch_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_batch.metadata['url']
        path_format_arguments = {
            'batchId': self._serialize.url("batch_id", batch_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_batch.metadata = {'url': '/api/v3/batch/{batchId}'}

    def close_batch(
            self, batch_id, custom_headers=None, raw=False, **operation_config):
        """

        :param batch_id:
        :type batch_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BatchBatchResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BatchBatchResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.close_batch.metadata['url']
        path_format_arguments = {
            'batchId': self._serialize.url("batch_id", batch_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BatchBatchResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    close_batch.metadata = {'url': '/api/v3/batch/close/{batchId}'}

    def get_bills(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves an array of bills. The array of bills can be filtered by a
        number of criteria. See the filter property for more details.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBillResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bills.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBillResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bills.metadata = {'url': '/api/v3/bill'}

    def create_bill(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a new bill.

        :param dto:
        :type dto: ~energycap.sdk.models.BillsBillCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillIdResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsBillIdResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_bill.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'BillsBillCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillIdResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_bill.metadata = {'url': '/api/v3/bill'}

    def get_bill(
            self, bill_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one bill.

        :param bill_id: The bill identifier
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsBillResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill.metadata = {'url': '/api/v3/bill/{billId}'}

    def edit_bill(
            self, bill_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit an existing bill.

        :param bill_id:
        :type bill_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.BillsBillEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillIdResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsBillIdResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_bill.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'BillsBillEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillIdResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_bill.metadata = {'url': '/api/v3/bill/{billId}'}

    def get_bill_bodylines(
            self, bill_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one bill's bodylines.

        :param bill_id: The bill identifier
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBodylineResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_bodylines.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBodylineResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_bodylines.metadata = {'url': '/api/v3/bill/{billId}/bodyline'}

    def get_all_bills_bodylines(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all bill's bodylines.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBodylineResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_all_bills_bodylines.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBodylineResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_all_bills_bodylines.metadata = {'url': '/api/v3/bill/bodyline'}

    def bill_action_void(
            self, action=None, custom_headers=None, raw=False, **operation_config):
        """Update void flag for the given billIDs.

        :param action: Void action
        :type action: ~energycap.sdk.models.BillsBillActionVoidDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.bill_action_void.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'BillsBillActionVoidDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    bill_action_void.metadata = {'url': '/api/v3/bill/void'}

    def bill_action_approve(
            self, action=None, custom_headers=None, raw=False, **operation_config):
        """Update approved flag for the given billIDs.

        :param action: approve action
        :type action: ~energycap.sdk.models.BillsBillActionApproveDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.bill_action_approve.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'BillsBillActionApproveDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    bill_action_approve.metadata = {'url': '/api/v3/bill/approve'}

    def bill_action_export_hold(
            self, action=None, custom_headers=None, raw=False, **operation_config):
        """Update export hold flag for the given billIDs.

        :param action: export hold action
        :type action: ~energycap.sdk.models.BillsBillActionExportHoldDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.bill_action_export_hold.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'BillsBillActionExportHoldDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    bill_action_export_hold.metadata = {'url': '/api/v3/bill/exporthold'}

    def bill_action_delete(
            self, bill_ids=None, custom_headers=None, raw=False, **operation_config):
        """Delete given billIDs.

        :param bill_ids:
        :type bill_ids: ~energycap.sdk.models.BillsBillActionDeleteDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillDeleteActionResultDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillsBillDeleteActionResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.bill_action_delete.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_ids is not None:
            body_content = self._serialize.body(bill_ids, 'BillsBillActionDeleteDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillDeleteActionResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    bill_action_delete.metadata = {'url': '/api/v3/bill/delete'}

    def get_possible_destination_accounts(
            self, bill_ids=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves a list of accounts that contain meters of ALL of the
        commodities represented by the given Bills.

        :param bill_ids: List of Bill Ids to be used to determine the
         commodities to be matched on the destination accounts.
        :type bill_ids: ~energycap.sdk.models.CommonListIdsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsMoveBillsDestinationDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsMoveBillsDestinationDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_possible_destination_accounts.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_ids is not None:
            body_content = self._serialize.body(bill_ids, 'CommonListIdsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsMoveBillsDestinationDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_possible_destination_accounts.metadata = {'url': '/api/v3/bill/possibleDestinationAccounts'}

    def move_bills_to_account(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Moves the list of given bills to the specified destination account and
        meters. Each source Meter will be matched with
        a corresponding destination Meter based on commodity.
        <param name="dto">
        DTO containing a list of Bill Ids, the destination Account Id, the
        destination AcountMeter Ids, a flag
        indicating whether the Account GL information should be updated, and a
        flag indicating whether the Meter GL information
        should be updated.
        </param>.

        :param dto:
        :type dto: ~energycap.sdk.models.BillsBillActionMoveDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.move_bills_to_account.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'BillsBillActionMoveDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    move_bills_to_account.metadata = {'url': '/api/v3/bill/moveBills'}

    def get_bill_account_meters_actual(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves bill account meter records.

        :param filter: Number of years of data to return includes the current
         year - Defaults to 4
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBillAccountMeterResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_account_meters_actual.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBillAccountMeterResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_account_meters_actual.metadata = {'url': '/api/v3/bill/billaccountmeter/actual'}

    def get_bill_account_meters_calendarized(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves bill account meter calendarized records.

        :param filter: Number of years of data to return includes the current
         year - Defaults to 4
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBillAccountMeterResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_account_meters_calendarized.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBillAccountMeterResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_account_meters_calendarized.metadata = {'url': '/api/v3/bill/billaccountmeter/calendarized'}

    def get_bill_account_meters_normalized(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves bill account meter normalized records.

        :param filter: Number of years of data to return includes the current
         year - Defaults to 4
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBillAccountMeterResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_account_meters_normalized.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBillAccountMeterResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_account_meters_normalized.metadata = {'url': '/api/v3/bill/billaccountmeter/normalized'}

    def get_bill_entry_data(
            self, account_id, begin_date=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all data necessary to load the bill entry screen.

        :param account_id:
        :type account_id: int
        :param begin_date:
        :type begin_date: datetime
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillEntryResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsBillEntryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_entry_data.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if begin_date is not None:
            query_parameters['beginDate'] = self._serialize.query("begin_date", begin_date, 'iso-8601')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillEntryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_entry_data.metadata = {'url': '/api/v3/account/{accountId}/billEntry'}

    def get_account_bill_history(
            self, account_id, billing_period=None, number_of_years=None, include_void=None, include_accrual=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves bill history for an account.

        :param account_id: accountId to get bill history for
        :type account_id: int
        :param billing_period: required: billingPeriod in yyyymm format to
         return history from
        :type billing_period: int
        :param number_of_years: optional: Number of years of history to
         return.  Defaults to 5
        :type number_of_years: int
        :param include_void: optional: Should voided bills be included.
         Defaults to false
        :type include_void: bool
        :param include_accrual: optional: Should accrual bills be included.
         Defaults to false
        :type include_accrual: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBillHistoryResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_bill_history.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if billing_period is not None:
            query_parameters['billingPeriod'] = self._serialize.query("billing_period", billing_period, 'int')
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')
        if include_void is not None:
            query_parameters['includeVoid'] = self._serialize.query("include_void", include_void, 'bool')
        if include_accrual is not None:
            query_parameters['includeAccrual'] = self._serialize.query("include_accrual", include_accrual, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBillHistoryResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_bill_history.metadata = {'url': '/api/v3/account/{accountId}/billHistory'}

    def get_bill_entry_data_by_bill_id(
            self, bill_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all data necessary to load the bill entry screen.

        :param bill_id:
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillEntryResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsBillEntryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_entry_data_by_bill_id.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillEntryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_entry_data_by_bill_id.metadata = {'url': '/api/v3/bill/{billId}/billEntry'}

    def get_bill_history(
            self, bill_id, number_of_years=None, include_void=None, include_accrual=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves bill history from starting at the billId billing period.

        :param bill_id: billId to get bill history for
        :type bill_id: int
        :param number_of_years: optional: Number of years of history to
         return.  Defaults to 5
        :type number_of_years: int
        :param include_void: optional: Should voided bills be included.
         Defaults to false
        :type include_void: bool
        :param include_accrual: optional: Should accrual bills be included.
         Defaults to false
        :type include_accrual: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillsBillHistoryResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_history.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')
        if include_void is not None:
            query_parameters['includeVoid'] = self._serialize.query("include_void", include_void, 'bool')
        if include_accrual is not None:
            query_parameters['includeAccrual'] = self._serialize.query("include_accrual", include_accrual, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillsBillHistoryResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_history.metadata = {'url': '/api/v3/bill/{billId}/billHistory'}

    def export_bills(
            self, bill_export_dto=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves text of a standard EnergyCAP bill export file for AP and GL
        purposes.
        Only bills with the following conditions are included in the export
        file:
        1) approved bills (if the approval system is enabled in workflow
        settings)
        2) bill is in a closed batch (if after batch or after batch approval is
        set in the workflow settings)
        3) bill is not held for export
        4) bill is not void
        5) bill is within the user's topmost setting.
        6) bill in not already exported.

        :param bill_export_dto:
        :type bill_export_dto: ~energycap.sdk.models.BillsBillExportDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.export_bills.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_export_dto is not None:
            body_content = self._serialize.body(bill_export_dto, 'BillsBillExportDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    export_bills.metadata = {'url': '/api/v3/bill/export'}

    def unexport_bills(
            self, bill_unexport_dto=None, custom_headers=None, raw=False, **operation_config):
        """Marks one or more bills as unexported.

        :param bill_unexport_dto: An array of bill ids to be marked as
         unexported and export mode
        :type bill_unexport_dto: ~energycap.sdk.models.BillsBillUnexportDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.unexport_bills.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_unexport_dto is not None:
            body_content = self._serialize.body(bill_unexport_dto, 'BillsBillUnexportDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    unexport_bills.metadata = {'url': '/api/v3/bill/unexport'}

    def unexport_bills_by_guid(
            self, bill_export_guid, bill_export_guid1, custom_headers=None, raw=False, **operation_config):
        """Marks one or more bills as unexported. The bills to be unexported are
        selected based on the guid passed in.
        The guid is generated when bills are exported via the bill export end
        point, and the guid is returned in the header of the response in a
        field named Ec-Guid.

        :param bill_export_guid: The identifier for the bills to be unexported
        :type bill_export_guid: str
        :param bill_export_guid1: The identifier for the bills to be
         unexported
        :type bill_export_guid1: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.unexport_bills_by_guid.metadata['url']
        path_format_arguments = {
            'billExportGuid': self._serialize.url("bill_export_guid1", bill_export_guid1, 'str')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        query_parameters['billExportGuid'] = self._serialize.query("bill_export_guid", bill_export_guid, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    unexport_bills_by_guid.metadata = {'url': '/api/v3/bill/unexport/{billExportGuid}'}

    def export_bill_profile(
            self, export_mode=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves the text of a standard EnergyCAP bill export ini file for AP
        and GL purposes.

        :param export_mode: Specifies whether the profile should be for AP or
         GL. Pass in "AP" for AP Profile, and pass in "GL" for GL Profile. If
         this is not included in the request, it defaults to AP.
        :type export_mode: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.export_bill_profile.metadata['url']

        # Construct parameters
        query_parameters = {}
        if export_mode is not None:
            query_parameters['exportMode'] = self._serialize.query("export_mode", export_mode, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    export_bill_profile.metadata = {'url': '/api/v3/bill/exportProfile'}

    def bill_action_split(
            self, bill_ids=None, custom_headers=None, raw=False, **operation_config):
        """Run the bill split processor on the given list of bills.

        :param bill_ids:
        :type bill_ids: ~energycap.sdk.models.BillsBillActionSplitDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackTaskResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.bill_action_split.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_ids is not None:
            body_content = self._serialize.body(bill_ids, 'BillsBillActionSplitDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    bill_action_split.metadata = {'url': '/api/v3/bill/split'}

    def get_bill_alert_statuses(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all possible bill alert status options.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillAlertAlertStatusResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_alert_statuses.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillAlertAlertStatusResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_alert_statuses.metadata = {'url': '/api/v3/bill/alertStatus'}

    def get_bill_alert_severities(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all possible bill alert severity options.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillAlertAlertSeverityResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_alert_severities.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillAlertAlertSeverityResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_alert_severities.metadata = {'url': '/api/v3/bill/alertSeverity'}

    def get_bill_alerts(
            self, bill_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all alerts on a bill.

        :param bill_id:
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillAlertBillAlertResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_alerts.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillAlertBillAlertResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_alerts.metadata = {'url': '/api/v3/bill/{billId}/alert'}

    def update_bill_alerts(
            self, bill_id, bill_alert_bulk_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Updates all alerts on a bill.

        :param bill_id: The identifier for the bill the alert is on
        :type bill_id: int
        :param bill_alert_bulk_request_dto: The values to update the alerts
         with.  Null value won't update this property
        :type bill_alert_bulk_request_dto:
         ~energycap.sdk.models.BillAlertBillAlertBulkRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdatedOnlyResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdatedOnlyResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_bill_alerts.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_alert_bulk_request_dto is not None:
            body_content = self._serialize.body(bill_alert_bulk_request_dto, 'BillAlertBillAlertBulkRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdatedOnlyResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_bill_alerts.metadata = {'url': '/api/v3/bill/{billId}/alert'}

    def create_bill_alert(
            self, bill_id, bill_alert_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create an alert on a bill.

        :param bill_id: The identifier for the bill to create alert on
        :type bill_id: int
        :param bill_alert_request_dto: The values to create the alert with
        :type bill_alert_request_dto:
         ~energycap.sdk.models.BillAlertBillAlertRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillAlertBillAlertResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillAlertBillAlertResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_bill_alert.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_alert_request_dto is not None:
            body_content = self._serialize.body(bill_alert_request_dto, 'BillAlertBillAlertRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillAlertBillAlertResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_bill_alert.metadata = {'url': '/api/v3/bill/{billId}/alert'}

    def delete_bill_alerts(
            self, bill_id, custom_headers=None, raw=False, **operation_config):
        """Deletes an alert on a bill.

        :param bill_id: The identifier for the bill the alert is on
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_bill_alerts.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_bill_alerts.metadata = {'url': '/api/v3/bill/{billId}/alert'}

    def get_bill_alert(
            self, bill_id, alert_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves an alert on a bill.

        :param bill_id:
        :type bill_id: int
        :param alert_id:
        :type alert_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillAlertBillAlertResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillAlertBillAlertResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_alert.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int'),
            'alertId': self._serialize.url("alert_id", alert_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillAlertBillAlertResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_alert.metadata = {'url': '/api/v3/bill/{billId}/alert/{alertId}'}

    def update_bill_alert(
            self, bill_id, alert_id, bill_alert_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Updates an alert on a bill.

        :param bill_id: The identifier for the bill the alert is on
        :type bill_id: int
        :param alert_id: The identifier for the alert to update
        :type alert_id: int
        :param bill_alert_request_dto: The values to update the alert with
        :type bill_alert_request_dto:
         ~energycap.sdk.models.BillAlertBillAlertRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillAlertBillAlertResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillAlertBillAlertResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_bill_alert.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int'),
            'alertId': self._serialize.url("alert_id", alert_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_alert_request_dto is not None:
            body_content = self._serialize.body(bill_alert_request_dto, 'BillAlertBillAlertRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillAlertBillAlertResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_bill_alert.metadata = {'url': '/api/v3/bill/{billId}/alert/{alertId}'}

    def delete_bill_alert(
            self, bill_id, alert_id, custom_headers=None, raw=False, **operation_config):
        """Deletes an alert on a bill.

        :param bill_id: The identifier for the bill the alert is on
        :type bill_id: int
        :param alert_id: The identifier for the alert to delete
        :type alert_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_bill_alert.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int'),
            'alertId': self._serialize.url("alert_id", alert_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_bill_alert.metadata = {'url': '/api/v3/bill/{billId}/alert/{alertId}'}

    def update_list_bill_ids_bill_alerts(
            self, bill_ids_bill_alert_bulk_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Update all alerts on a list of billIds.

        :param bill_ids_bill_alert_bulk_request_dto: List of billIds and
         values to update the alerts with.  Null value won't update this
         property
        :type bill_ids_bill_alert_bulk_request_dto:
         ~energycap.sdk.models.BillAlertBillIdsBillAlertBulkRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdatedOnlyResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdatedOnlyResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_list_bill_ids_bill_alerts.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_ids_bill_alert_bulk_request_dto is not None:
            body_content = self._serialize.body(bill_ids_bill_alert_bulk_request_dto, 'BillAlertBillIdsBillAlertBulkRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdatedOnlyResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_list_bill_ids_bill_alerts.metadata = {'url': '/api/v3/bill/alert'}

    def update_specific_list_bill_ids_bill_alerts(
            self, specific_list_id, bill_alert_bulk_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Update all alerts on a list of bills.

        :param specific_list_id: The bill list id for the list of bills to
         update
        :type specific_list_id: int
        :param bill_alert_bulk_request_dto: The values to update the alerts
         with.  Null value won't update this property
        :type bill_alert_bulk_request_dto:
         ~energycap.sdk.models.BillAlertBillAlertBulkRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdatedOnlyResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdatedOnlyResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_specific_list_bill_ids_bill_alerts.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_alert_bulk_request_dto is not None:
            body_content = self._serialize.body(bill_alert_bulk_request_dto, 'BillAlertBillAlertBulkRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdatedOnlyResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_specific_list_bill_ids_bill_alerts.metadata = {'url': '/api/v3/list/{specificListId}/alert'}

    def update_account_list_bill_ids_bill_alerts(
            self, account_id, bill_alert_bulk_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Update all alerts on an account list of bills.

        :param account_id: The accountId of bills to update
        :type account_id: int
        :param bill_alert_bulk_request_dto: The values to update the alerts
         with.  Null value won't update this property
        :type bill_alert_bulk_request_dto:
         ~energycap.sdk.models.BillAlertBillAlertBulkRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdatedOnlyResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdatedOnlyResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_account_list_bill_ids_bill_alerts.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_alert_bulk_request_dto is not None:
            body_content = self._serialize.body(bill_alert_bulk_request_dto, 'BillAlertBillAlertBulkRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdatedOnlyResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_account_list_bill_ids_bill_alerts.metadata = {'url': '/api/v3/account/{accountId}/list/bill/alert'}

    def update_meter_list_bill_ids_bill_alerts(
            self, meter_id, bill_alert_bulk_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Update all alerts on a meter list of bills.

        :param meter_id: The meterId of bills to update
        :type meter_id: int
        :param bill_alert_bulk_request_dto: The values to update the alerts
         with.  Null value won't update this property
        :type bill_alert_bulk_request_dto:
         ~energycap.sdk.models.BillAlertBillAlertBulkRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdatedOnlyResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdatedOnlyResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_meter_list_bill_ids_bill_alerts.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_alert_bulk_request_dto is not None:
            body_content = self._serialize.body(bill_alert_bulk_request_dto, 'BillAlertBillAlertBulkRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdatedOnlyResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_meter_list_bill_ids_bill_alerts.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/alert'}

    def get_bill_audit_groups(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all bill audit groups.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillAuditBillAuditGroupResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_audit_groups.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillAuditBillAuditGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_audit_groups.metadata = {'url': '/api/v3/billauditgroup'}

    def create_bill_audit_group(
            self, bill_audit_group_create_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a bill audit group.

        :param bill_audit_group_create_dto:
        :type bill_audit_group_create_dto:
         ~energycap.sdk.models.BillAuditBillAuditGroupCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillAuditBillAuditGroupResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillAuditBillAuditGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_bill_audit_group.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_audit_group_create_dto is not None:
            body_content = self._serialize.body(bill_audit_group_create_dto, 'BillAuditBillAuditGroupCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillAuditBillAuditGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_bill_audit_group.metadata = {'url': '/api/v3/billauditgroup'}

    def get_bill_audit_group(
            self, bill_audit_group_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a bill audit group.

        :param bill_audit_group_id:
        :type bill_audit_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillAuditBillAuditGroupResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillAuditBillAuditGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_audit_group.metadata['url']
        path_format_arguments = {
            'billAuditGroupId': self._serialize.url("bill_audit_group_id", bill_audit_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillAuditBillAuditGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_audit_group.metadata = {'url': '/api/v3/billauditgroup/{billAuditGroupId}'}

    def edit_bill_audit_group(
            self, bill_audit_group_id, bill_audit_group_edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a bill audit group.

        :param bill_audit_group_id:
        :type bill_audit_group_id: int
        :param bill_audit_group_edit_dto:
        :type bill_audit_group_edit_dto:
         ~energycap.sdk.models.BillAuditBillAuditGroupEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillAuditBillAuditGroupResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillAuditBillAuditGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_bill_audit_group.metadata['url']
        path_format_arguments = {
            'billAuditGroupId': self._serialize.url("bill_audit_group_id", bill_audit_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_audit_group_edit_dto is not None:
            body_content = self._serialize.body(bill_audit_group_edit_dto, 'BillAuditBillAuditGroupEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillAuditBillAuditGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_bill_audit_group.metadata = {'url': '/api/v3/billauditgroup/{billAuditGroupId}'}

    def delete_bill_audit_group(
            self, bill_audit_group_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a bill audit group.

        :param bill_audit_group_id:
        :type bill_audit_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_bill_audit_group.metadata['url']
        path_format_arguments = {
            'billAuditGroupId': self._serialize.url("bill_audit_group_id", bill_audit_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_bill_audit_group.metadata = {'url': '/api/v3/billauditgroup/{billAuditGroupId}'}

    def execute_audit_processor(
            self, process_audits_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param process_audits_dto:
        :type process_audits_dto:
         ~energycap.sdk.models.BillAuditProcessAuditsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.execute_audit_processor.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if process_audits_dto is not None:
            body_content = self._serialize.body(process_audits_dto, 'BillAuditProcessAuditsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    execute_audit_processor.metadata = {'url': '/api/v3/billauditgroup/exec'}

    def get_bill_audit_group_filters(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all possible filters for a bill audit group.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CommonFilterResponse] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_audit_group_filters.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CommonFilterResponse]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_audit_group_filters.metadata = {'url': '/api/v3/billauditgroup/filter'}

    def get_bill_audits(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all bill audits.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.BillAuditBillAuditResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_audits.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillAuditBillAuditResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_audits.metadata = {'url': '/api/v3/billaudit'}

    def edit_specific_audit(
            self, specific_audit_id, specific_audit_property_edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a specific audit.

        :param specific_audit_id:
        :type specific_audit_id: int
        :param specific_audit_property_edit_dto:
        :type specific_audit_property_edit_dto:
         ~energycap.sdk.models.BillAuditSpecificAuditPropertyEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillAuditSpecificAuditResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillAuditSpecificAuditResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_specific_audit.metadata['url']
        path_format_arguments = {
            'specificAuditId': self._serialize.url("specific_audit_id", specific_audit_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if specific_audit_property_edit_dto is not None:
            body_content = self._serialize.body(specific_audit_property_edit_dto, 'BillAuditSpecificAuditPropertyEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillAuditSpecificAuditResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_specific_audit.metadata = {'url': '/api/v3/specificaudit/{specificAuditId}'}

    def get_bill_splits(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Get a list of bill split versions on an account.

        :param account_id: ID of the account to get all assigned bill split
         versions
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_splits.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewDistributionVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_splits.metadata = {'url': '/api/v3/account/{accountId}/billSplit'}

    def get_bill_split_version_history(
            self, account_id, meter_id, custom_headers=None, raw=False, **operation_config):
        """Get a list of bill split versions on this account and meter
        relationship.

        :param account_id: ID of the account to get all assigned bill split
         versions
        :type account_id: int
        :param meter_id: ID of the meter to get all assigned bill split
         versions
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_split_version_history.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewDistributionVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_split_version_history.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/billSplit'}

    def manage_bill_split_version_history(
            self, account_id, meter_id, bill_split_version_dtos=None, custom_headers=None, raw=False, **operation_config):
        """Manage a bill split's version history
        All bill split versions to save on this account and meter relationship
        should be passed
        If versionId is passed, this version will be updated to the request
        body values
        If copyVersionId is passed, a new version will be created and all
        instructions and inputs on the copyVersionId will be copied
        If versionId and copyVersionId are null a new version will be created
        Versions cannot overlap
        If a bill split version currently exists in the database, but is not
        passed in it will be deleted
        Only versions that are not currently associated with any bills can be
        deleted
        If any part of the create, update, or delete process fails, the entire
        process is rolled back.

        :param account_id: ID of the account whose bill split versions will be
         updated
        :type account_id: int
        :param meter_id: ID of the meter whose bill split versions will be
         updated
        :type meter_id: int
        :param bill_split_version_dtos: List of bill split distribution
         versions to add or edit
        :type bill_split_version_dtos:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionManageDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manage_bill_split_version_history.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_split_version_dtos is not None:
            body_content = self._serialize.body(bill_split_version_dtos, '[ChargebackNewDistributionVersionManageDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewDistributionVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    manage_bill_split_version_history.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/billSplit'}

    def get_bill_split_details(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Get details for a bill split distribution.

        :param account_id: ID of the account
        :type account_id: int
        :param meter_id: ID of the meter
        :type meter_id: int
        :param version_id: ID of the distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewBillSplitBillSplitDetailsResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewBillSplitBillSplitDetailsResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_split_details.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewBillSplitBillSplitDetailsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_split_details.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/billSplit/{versionId}'}

    def manage_bill_split_details(
            self, account_id, meter_id, version_id, bill_split_details_dto=None, custom_headers=None, raw=False, **operation_config):
        """Manage details for a bill split distribution.

        :param account_id: ID of the master account
        :type account_id: int
        :param meter_id: ID of the master meter
        :type meter_id: int
        :param version_id: ID of the version to update bill split details
        :type version_id: int
        :param bill_split_details_dto: Definition of details for bill split
        :type bill_split_details_dto:
         ~energycap.sdk.models.ChargebackNewBillSplitBillSplitDetailsRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewBillSplitBillSplitDetailsResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewBillSplitBillSplitDetailsResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manage_bill_split_details.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_split_details_dto is not None:
            body_content = self._serialize.body(bill_split_details_dto, 'ChargebackNewBillSplitBillSplitDetailsRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewBillSplitBillSplitDetailsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    manage_bill_split_details.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/billSplit/{versionId}'}

    def get_chargeback_task_by_version(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Get all chargeback tasks with details for a specific version on an
        account and meter.

        :param account_id: ID of the account
        :type account_id: int
        :param meter_id: ID of the meter
        :type meter_id: int
        :param version_id: ID of the distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewChargebackTaskVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_task_by_version.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewChargebackTaskVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_task_by_version.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/billSplit/{versionId}/chargebackTask'}

    def get_bill_view(
            self, bill_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves properties for bill view.

        :param bill_id: The bill identifier
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillViewBillViewResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillViewBillViewResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_view.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillViewBillViewResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_view.metadata = {'url': '/api/v3/billView/{billId}'}

    def get_calculated_bills(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Get a list of calculated bill versions on a meter.

        :param meter_id: ID of the meter to get all assigned calculated bill
         versions
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_calculated_bills.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewDistributionVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_calculated_bills.metadata = {'url': '/api/v3/meter/{meterId}/calculatedBill'}

    def get_calculated_bill_version_history(
            self, account_id, meter_id, custom_headers=None, raw=False, **operation_config):
        """Get a list of calculated bill versions on this account and meter
        relationship.

        :param account_id: ID of the account to get all assigned calculated
         bill versions
        :type account_id: int
        :param meter_id: ID of the meter to get all assigned calculated bill
         versions
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_calculated_bill_version_history.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewDistributionVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_calculated_bill_version_history.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill'}

    def manage_calculated_bill_version_history(
            self, account_id, meter_id, calculated_bill_version_dtos=None, custom_headers=None, raw=False, **operation_config):
        """Manage a calculated bill's version history
        All calculated bill versions to save on this account and meter
        relationship should be passed
        If versionId is passed, this version will be updated to the request
        body values
        If copyVersionId is passed, a new version will be created and all
        instructions and inputs on the copyVersionId will be copied
        If versionId and copyVersionId are null a new version will be created
        Versions cannot overlap
        If a calculated bill version currently exists in the database, but is
        not passed in it will be deleted
        Only versions that are not currently associated with any bills can be
        deleted
        If any part of the create, update, or delete process fails, the entire
        process is rolled back.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param calculated_bill_version_dtos: List of calculated bill
         distribution versions to add or edit
        :type calculated_bill_version_dtos:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionManageDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewDistributionVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manage_calculated_bill_version_history.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if calculated_bill_version_dtos is not None:
            body_content = self._serialize.body(calculated_bill_version_dtos, '[ChargebackNewDistributionVersionManageDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewDistributionVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    manage_calculated_bill_version_history.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill'}

    def get_calculate_bill_details(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """All details related to calculated bill setup including use, cost, meter
        line items, and account line items.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewCalculatedBillCalculatedBillDetailsResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewCalculatedBillCalculatedBillDetailsResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_calculate_bill_details.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewCalculatedBillCalculatedBillDetailsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_calculate_bill_details.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}'}

    def get_calculate_bill_use(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Get calculated use details.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewCalculatedBillCalculatedBillUseResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewCalculatedBillCalculatedBillUseResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_calculate_bill_use.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewCalculatedBillCalculatedBillUseResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_calculate_bill_use.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/use'}

    def manage_calculate_bill_use(
            self, account_id, meter_id, version_id, calculated_bill_use_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Manage calculated bill's use
        * If use does not exist on the distribution it will be added
        * If use already exists it will be saved as the option sent in and any
        existing inputs will be deleted
        * Since use is required for the calculated bill distribution to work,
        one option is always required and cannot be deleted.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param calculated_bill_use_request_dto: Definition of how the meter
         will calculate its use
        :type calculated_bill_use_request_dto:
         ~energycap.sdk.models.ChargebackNewCalculatedBillCalculatedBillUseRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewCalculatedBillCalculatedBillUseResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewCalculatedBillCalculatedBillUseResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manage_calculate_bill_use.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if calculated_bill_use_request_dto is not None:
            body_content = self._serialize.body(calculated_bill_use_request_dto, 'ChargebackNewCalculatedBillCalculatedBillUseRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewCalculatedBillCalculatedBillUseResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    manage_calculate_bill_use.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/use'}

    def get_calculate_bill_cost(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Get calculated cost details.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewCalculatedBillCalculatedBillCostResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewCalculatedBillCalculatedBillCostResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_calculate_bill_cost.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewCalculatedBillCalculatedBillCostResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_calculate_bill_cost.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/cost'}

    def manage_calculate_bill_cost(
            self, account_id, meter_id, version_id, calculated_bill_cost_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Manage calculated bill's cost
        If cost does not exist on the distribtion it will be added
        If one exists it will be saved as the option sent in
        Since cost is required for the calculated bill distribtion to work.
        One option is always required and cannot be deleted.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param calculated_bill_cost_request_dto: Definition of how the meter
         will calculate its cost
        :type calculated_bill_cost_request_dto:
         ~energycap.sdk.models.ChargebackNewCalculatedBillCalculatedBillCostRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewCalculatedBillCalculatedBillCostResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewCalculatedBillCalculatedBillCostResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manage_calculate_bill_cost.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if calculated_bill_cost_request_dto is not None:
            body_content = self._serialize.body(calculated_bill_cost_request_dto, 'ChargebackNewCalculatedBillCalculatedBillCostRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewCalculatedBillCalculatedBillCostResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    manage_calculate_bill_cost.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/cost'}

    def get_meter_line_items(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Get calculated bill meter line items.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewCalculatedBillGenericBodylineResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_line_items.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewCalculatedBillGenericBodylineResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_line_items.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/meterLineItem'}

    def manage_meter_line_items(
            self, account_id, meter_id, version_id, list_meter_line_item_dto=None, custom_headers=None, raw=False, **operation_config):
        """Manage calculated bill meter line items
        The order of items in the list defines the calculation order
        All line itmes to save should be passed
        If a line currently exists and is not passed in it will be deleted
        If any part of the create, update, or delete process fails, the entire
        process is rolled back.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param list_meter_line_item_dto: Definition of the meter line items
        :type list_meter_line_item_dto:
         list[~energycap.sdk.models.ChargebackNewCalculatedBillGenericBodylineRequestDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewCalculatedBillGenericBodylineResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manage_meter_line_items.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if list_meter_line_item_dto is not None:
            body_content = self._serialize.body(list_meter_line_item_dto, '[ChargebackNewCalculatedBillGenericBodylineRequestDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewCalculatedBillGenericBodylineResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    manage_meter_line_items.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/meterLineItem'}

    def get_account_line_items(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Get calculated bill account line items.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewCalculatedBillGenericBodylineResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_account_line_items.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewCalculatedBillGenericBodylineResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_account_line_items.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/accountLineItem'}

    def manage_account_line_items(
            self, account_id, meter_id, version_id, list_account_line_item_dto=None, custom_headers=None, raw=False, **operation_config):
        """Manage calculated bill account line items
        The order of items in the list defines the calculation order
        All line itmes to save should be passed
        If a line currently exists and is not passed in it will be deleted
        If any part of the create, update, or delete process fails, the entire
        process is rolled back.

        :param account_id: ID of the account to receive calculated bill
         details
        :type account_id: int
        :param meter_id: ID of the meter to receive calculated bill details
        :type meter_id: int
        :param version_id: ID of the calculated bill distribution version
        :type version_id: int
        :param list_account_line_item_dto: Definition of the account line
         items.
        :type list_account_line_item_dto:
         list[~energycap.sdk.models.ChargebackNewCalculatedBillGenericBodylineRequestDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewCalculatedBillGenericBodylineResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manage_account_line_items.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if list_account_line_item_dto is not None:
            body_content = self._serialize.body(list_account_line_item_dto, '[ChargebackNewCalculatedBillGenericBodylineRequestDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewCalculatedBillGenericBodylineResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    manage_account_line_items.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/accountLineItem'}

    def get_chargeback_task_by_version1(
            self, account_id, meter_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Get all chargeback tasks with details for a specific version on an
        account and meter.

        :param account_id: ID of the account
        :type account_id: int
        :param meter_id: ID of the meter
        :type meter_id: int
        :param version_id: ID of the distribution version
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewChargebackTaskVersionResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_task_by_version1.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewChargebackTaskVersionResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_task_by_version1.metadata = {'url': '/api/v3/account/{accountId}/meter/{meterId}/calculatedBill/{versionId}/chargebackTask'}

    def get_channel(
            self, channel_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves details about a single channel.

        :param channel_id: The channel identifier
        :type channel_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChannelChannelResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ChannelChannelResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_channel.metadata['url']
        path_format_arguments = {
            'channelID': self._serialize.url("channel_id", channel_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChannelChannelResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_channel.metadata = {'url': '/api/v3/channel/{channelID}'}

    def edit_channel(
            self, channel_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits an existing channel.

        :param channel_id: The channel identifier
        :type channel_id: int
        :param dto: Data representing the channel to be edited
        :type dto: ~energycap.sdk.models.ChannelChannelEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChannelChannelResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ChannelChannelResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_channel.metadata['url']
        path_format_arguments = {
            'channelID': self._serialize.url("channel_id", channel_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'ChannelChannelEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChannelChannelResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_channel.metadata = {'url': '/api/v3/channel/{channelID}'}

    def delete_channel(
            self, channel_id, custom_headers=None, raw=False, **operation_config):
        """Deletes and existing channel.

        :param channel_id: The channel identifier
        :type channel_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_channel.metadata['url']
        path_format_arguments = {
            'channelID': self._serialize.url("channel_id", channel_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_channel.metadata = {'url': '/api/v3/channel/{channelID}'}

    def create_channel(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a channel.

        :param dto: Data representing the channel to be created
        :type dto: ~energycap.sdk.models.ChannelChannelCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChannelChannelResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ChannelChannelResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_channel.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'ChannelChannelCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChannelChannelResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_channel.metadata = {'url': '/api/v3/channel'}

    def execute_bill_split_processor(
            self, process_split_dto=None, custom_headers=None, raw=False, **operation_config):
        """Process one or more Bill Split distributions for a specific billing
        period.

        :param process_split_dto:
        :type process_split_dto:
         ~energycap.sdk.models.ChargebackNewChargebackProcessorChargebackProcessorSettingsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackTaskResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.execute_bill_split_processor.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if process_split_dto is not None:
            body_content = self._serialize.body(process_split_dto, 'ChargebackNewChargebackProcessorChargebackProcessorSettingsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    execute_bill_split_processor.metadata = {'url': '/api/v3/billSplit/exec'}

    def execute_calculate_bill_processor(
            self, process_calculated_bill_dto=None, custom_headers=None, raw=False, **operation_config):
        """Process one or more Calculated Bill distributions for a specific
        billing period.

        :param process_calculated_bill_dto:
        :type process_calculated_bill_dto:
         ~energycap.sdk.models.ChargebackNewChargebackProcessorChargebackProcessorSettingsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackTaskResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.execute_calculate_bill_processor.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if process_calculated_bill_dto is not None:
            body_content = self._serialize.body(process_calculated_bill_dto, 'ChargebackNewChargebackProcessorChargebackProcessorSettingsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    execute_calculate_bill_processor.metadata = {'url': '/api/v3/calculatedBill/exec'}

    def execute_processor_by_workflow_step(
            self, chargeback_workflow_step_id, settings=None, custom_headers=None, raw=False, **operation_config):
        """Process distributions assigned to a chargeback workflow step for a
        specific billing period.

        :param chargeback_workflow_step_id: The chargeback workflow step id.
        :type chargeback_workflow_step_id: int
        :param settings: The other processor settings.
        :type settings:
         ~energycap.sdk.models.ChargebackNewChargebackProcessorChargebackProcessorSettingsWithoutFiltersDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackTaskResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.execute_processor_by_workflow_step.metadata['url']
        path_format_arguments = {
            'chargebackWorkflowStepId': self._serialize.url("chargeback_workflow_step_id", chargeback_workflow_step_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if settings is not None:
            body_content = self._serialize.body(settings, 'ChargebackNewChargebackProcessorChargebackProcessorSettingsWithoutFiltersDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    execute_processor_by_workflow_step.metadata = {'url': '/api/v3/chargebackWorkflowStep/{chargebackWorkflowStepId}/exec'}

    def void_chargeback_task(
            self, task_id, custom_headers=None, raw=False, **operation_config):
        """Void all bills created by the chargeback task
        If this is a split task, any source bills will be reset to has not been
        split
        If any bills have already been exported they will still be voided.

        :param task_id: Identifier for the chargeback task
        :type task_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.void_chargeback_task.metadata['url']
        path_format_arguments = {
            'taskId': self._serialize.url("task_id", task_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    void_chargeback_task.metadata = {'url': '/api/v3/chargebackTask/{taskId}/void'}

    def delete_chargeback_task(
            self, task_id, custom_headers=None, raw=False, **operation_config):
        """Delete all bills created by the chargeback task
        If this is a split task, any source bills will be reset to has not been
        split
        If any bills have already been exported they will still be deleted.

        :param task_id: Identifier for the chargeback task
        :type task_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_chargeback_task.metadata['url']
        path_format_arguments = {
            'taskId': self._serialize.url("task_id", task_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_chargeback_task.metadata = {'url': '/api/v3/chargebackTask/{taskId}/delete'}

    def get_chargeback_tasks(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Get all chargeback tasks with details.

        :param filter: Filters to apply
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_tasks.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewChargebackTaskResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_tasks.metadata = {'url': '/api/v3/chargebackTask'}

    def get_chargeback_task(
            self, task_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a chargeback task with details.

        :param task_id: Identifier for the chargeback task
        :type task_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackTaskResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_task.metadata['url']
        path_format_arguments = {
            'taskId': self._serialize.url("task_id", task_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_task.metadata = {'url': '/api/v3/chargebackTask/{taskId}'}

    def update_chargeback_task(
            self, task_id, chargeback_task_edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Update a chargeback task's comment.

        :param task_id: Identifier for the chargeback task to update
        :type task_id: int
        :param chargeback_task_edit_dto: Data representing the comment
        :type chargeback_task_edit_dto:
         ~energycap.sdk.models.ChargebackNewChargebackTaskRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackTaskResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_chargeback_task.metadata['url']
        path_format_arguments = {
            'taskId': self._serialize.url("task_id", task_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if chargeback_task_edit_dto is not None:
            body_content = self._serialize.body(chargeback_task_edit_dto, 'ChargebackNewChargebackTaskRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_chargeback_task.metadata = {'url': '/api/v3/chargebackTask/{taskId}'}

    def get_chargeback_task_error_file(
            self, task_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a chargeback task's error log as an EXCEL file.

        :param task_id: Identifier for the chargeback task
        :type task_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_task_error_file.metadata['url']
        path_format_arguments = {
            'taskId': self._serialize.url("task_id", task_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    get_chargeback_task_error_file.metadata = {'url': '/api/v3/chargebackTask/{taskId}/errorOutput'}

    def get_chargeback_workflows(
            self, custom_headers=None, raw=False, **operation_config):
        """Get a list of chargeback workflows.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_workflows.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_workflows.metadata = {'url': '/api/v3/chargebackWorkflow'}

    def create_chargeback_workflow(
            self, chargeback_workflow_create_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a chargeback workflow.

        :param chargeback_workflow_create_dto: Definition for chargeback
         workflow
        :type chargeback_workflow_create_dto:
         ~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_chargeback_workflow.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if chargeback_workflow_create_dto is not None:
            body_content = self._serialize.body(chargeback_workflow_create_dto, 'ChargebackNewChargebackWorkflowChargebackWorkflowCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_chargeback_workflow.metadata = {'url': '/api/v3/chargebackWorkflow'}

    def get_chargeback_workflow(
            self, chargeback_workflow_id, custom_headers=None, raw=False, **operation_config):
        """Get a chargeback workflow.

        :param chargeback_workflow_id: Chargeback workflow id
        :type chargeback_workflow_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_workflow.metadata['url']
        path_format_arguments = {
            'chargebackWorkflowId': self._serialize.url("chargeback_workflow_id", chargeback_workflow_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_workflow.metadata = {'url': '/api/v3/chargebackWorkflow/{chargebackWorkflowId}'}

    def edit_chargeback_workflow(
            self, chargeback_workflow_id, chargeback_workflow_edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a chargeback workflow.

        :param chargeback_workflow_id: Chargeback workflow id
        :type chargeback_workflow_id: int
        :param chargeback_workflow_edit_dto: Definition for chargeback
         workflow
        :type chargeback_workflow_edit_dto:
         ~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_chargeback_workflow.metadata['url']
        path_format_arguments = {
            'chargebackWorkflowId': self._serialize.url("chargeback_workflow_id", chargeback_workflow_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if chargeback_workflow_edit_dto is not None:
            body_content = self._serialize.body(chargeback_workflow_edit_dto, 'ChargebackNewChargebackWorkflowChargebackWorkflowEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackWorkflowChargebackWorkflowResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_chargeback_workflow.metadata = {'url': '/api/v3/chargebackWorkflow/{chargebackWorkflowId}'}

    def delete_chargeback_workflow(
            self, chargeback_workflow_id, custom_headers=None, raw=False, **operation_config):
        """Delete a chargeback workflow.

        :param chargeback_workflow_id: Chargeback workflow id
        :type chargeback_workflow_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_chargeback_workflow.metadata['url']
        path_format_arguments = {
            'chargebackWorkflowId': self._serialize.url("chargeback_workflow_id", chargeback_workflow_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_chargeback_workflow.metadata = {'url': '/api/v3/chargebackWorkflow/{chargebackWorkflowId}'}

    def get_chargeback_workflow_steps(
            self, chargeback_workflow_type=None, custom_headers=None, raw=False, **operation_config):
        """Get a list of chargeback workflow steps.

        :param chargeback_workflow_type: Pass either "split" or "calculation"
         to filter workflow steps
        :type chargeback_workflow_type: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowStepChildDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_workflow_steps.metadata['url']

        # Construct parameters
        query_parameters = {}
        if chargeback_workflow_type is not None:
            query_parameters['chargebackWorkflowType'] = self._serialize.query("chargeback_workflow_type", chargeback_workflow_type, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChargebackNewChargebackWorkflowChargebackWorkflowStepChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_workflow_steps.metadata = {'url': '/api/v3/chargebackWorkflowStep'}

    def void_chargeback_workflow_steps(
            self, chargeback_workflow_step_id, billing_period=None, custom_headers=None, raw=False, **operation_config):
        """Void all bills created by a chargeback workflow step for a particular
        billing period
        If this workflow step was run multiple times, all bills created across
        all runs for the billing period will be voided
        If this is a split step, any source bills will be reset to has not been
        split
        If any bills have already been exported they will still be voided.

        :param chargeback_workflow_step_id: Chargeback Workflow Step ID
        :type chargeback_workflow_step_id: int
        :param billing_period: Billing period to undo
        :type billing_period: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.void_chargeback_workflow_steps.metadata['url']
        path_format_arguments = {
            'chargebackWorkflowStepId': self._serialize.url("chargeback_workflow_step_id", chargeback_workflow_step_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if billing_period is not None:
            query_parameters['billingPeriod'] = self._serialize.query("billing_period", billing_period, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    void_chargeback_workflow_steps.metadata = {'url': '/api/v3/chargebackWorkflowStep/{chargebackWorkflowStepId}/void'}

    def delete_chargeback_workflow_steps(
            self, chargeback_workflow_step_id, billing_period=None, custom_headers=None, raw=False, **operation_config):
        """Delete all bills created by a chargeback workflow step
        If this workflow step was run multiple times, all bills created across
        all runs for the billing period will be voided
        If this is a split step, any source bills will be reset to has not been
        split
        If any bills have already been exported they will still be deleted.

        :param chargeback_workflow_step_id: Chargeback Workflow Step ID
        :type chargeback_workflow_step_id: int
        :param billing_period: Billing period to undo
        :type billing_period: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_chargeback_workflow_steps.metadata['url']
        path_format_arguments = {
            'chargebackWorkflowStepId': self._serialize.url("chargeback_workflow_step_id", chargeback_workflow_step_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if billing_period is not None:
            query_parameters['billingPeriod'] = self._serialize.query("billing_period", billing_period, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_chargeback_workflow_steps.metadata = {'url': '/api/v3/chargebackWorkflowStep/{chargebackWorkflowStepId}/delete'}

    def get_chargeback_workflow_digest(
            self, chargeback_workflow_step_id, billing_period=None, custom_headers=None, raw=False, **operation_config):
        """Get a digest of all bills by meter for all meters involved in a
        chargeback workflow step for a given billing period.

        :param chargeback_workflow_step_id: The id of the chargeback workflow
         step.
        :type chargeback_workflow_step_id: int
        :param billing_period: The billing period of the digest.
        :type billing_period: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return:
         ChargebackNewChargebackWorkflowChargebackWorkflowDigestResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.ChargebackNewChargebackWorkflowChargebackWorkflowDigestResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_chargeback_workflow_digest.metadata['url']
        path_format_arguments = {
            'chargebackWorkflowStepId': self._serialize.url("chargeback_workflow_step_id", chargeback_workflow_step_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if billing_period is not None:
            query_parameters['billingPeriod'] = self._serialize.query("billing_period", billing_period, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackWorkflowChargebackWorkflowDigestResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_chargeback_workflow_digest.metadata = {'url': '/api/v3/chargebackWorkflow/{chargebackWorkflowStepId}/digest'}

    def get_units_by_commodity(
            self, commodity_id, class_code=None, noun_code=None, unit_type_code=None, custom_headers=None, raw=False, **operation_config):
        """Returns a list of units by a specific commodity.

        :param commodity_id: The specific commodity ID
        :type commodity_id: int
        :param class_code: (Optional) A grouping of classes for the units.
         Valid values are 'use', 'cost', 'demand'.
        :type class_code: str
        :param noun_code: (Optional) A specific class for units
        :type noun_code: str
        :param unit_type_code: (Optional) A specific unity type
        :type unit_type_code: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UnitUnitChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_units_by_commodity.metadata['url']
        path_format_arguments = {
            'commodityId': self._serialize.url("commodity_id", commodity_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if class_code is not None:
            query_parameters['classCode'] = self._serialize.query("class_code", class_code, 'str')
        if noun_code is not None:
            query_parameters['nounCode'] = self._serialize.query("noun_code", noun_code, 'str')
        if unit_type_code is not None:
            query_parameters['unitTypeCode'] = self._serialize.query("unit_type_code", unit_type_code, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UnitUnitChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_units_by_commodity.metadata = {'url': '/api/v3/commodity/{commodityId}/unit'}

    def get_commodities(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CommoditiesCommodityDto] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_commodities.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CommoditiesCommodityDto]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_commodities.metadata = {'url': '/api/v3/commodity'}

    def get_commodity(
            self, commodity_id, custom_headers=None, raw=False, **operation_config):
        """

        :param commodity_id:
        :type commodity_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommoditiesCommodityDto or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommoditiesCommodityDto or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_commodity.metadata['url']
        path_format_arguments = {
            'commodityId': self._serialize.url("commodity_id", commodity_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommoditiesCommodityDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_commodity.metadata = {'url': '/api/v3/commodity/{commodityId}'}

    def get_commodity_actual_yearly(
            self, commodity_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by commodities for a given
        number of years- Result includes cost, use, unit cost on an yearly
        basis.

        :param commodity_id: The commodity to retrieve data for
        :type commodity_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommodityDigestCommodityDigestActualYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.CommodityDigestCommodityDigestActualYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_commodity_actual_yearly.metadata['url']
        path_format_arguments = {
            'commodityId': self._serialize.url("commodity_id", commodity_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommodityDigestCommodityDigestActualYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_commodity_actual_yearly.metadata = {'url': '/api/v3/commodity/{commodityId}/digest/actual/yearly'}

    def get_cost_centers(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all the cost centers.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CostCentersCostCenterResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_cost_centers.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CostCentersCostCenterResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_cost_centers.metadata = {'url': '/api/v3/costcenter'}

    def create_cost_center(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a Cost Center.

        :param dto: Date representing the cost center being created
        :type dto: ~energycap.sdk.models.CostCentersCostCenterCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CostCentersCostCenterResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.CostCentersCostCenterResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_cost_center.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'CostCentersCostCenterCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CostCentersCostCenterResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_cost_center.metadata = {'url': '/api/v3/costcenter'}

    def get_cost_center(
            self, cost_center_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves the cost center by ID.

        :param cost_center_id: The identifier for the cost center being
         retrieved
        :type cost_center_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CostCentersCostCenterResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.CostCentersCostCenterResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_cost_center.metadata['url']
        path_format_arguments = {
            'costCenterID': self._serialize.url("cost_center_id", cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CostCentersCostCenterResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_cost_center.metadata = {'url': '/api/v3/costcenter/{costCenterID}'}

    def edit_cost_center(
            self, cost_center_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a cost center.

        :param cost_center_id: The identifier for the cost center being edited
        :type cost_center_id: int
        :param dto: Data representing the cost center being edited
        :type dto: ~energycap.sdk.models.CostCentersCostCenterCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CostCentersCostCenterResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.CostCentersCostCenterResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_cost_center.metadata['url']
        path_format_arguments = {
            'costCenterID': self._serialize.url("cost_center_id", cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'CostCentersCostCenterCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CostCentersCostCenterResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_cost_center.metadata = {'url': '/api/v3/costcenter/{costCenterID}'}

    def delete_cost_center(
            self, cost_center_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a cost center.

        :param cost_center_id: The identifier for the cost center being
         deleted
        :type cost_center_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_cost_center.metadata['url']
        path_format_arguments = {
            'costCenterID': self._serialize.url("cost_center_id", cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_cost_center.metadata = {'url': '/api/v3/costcenter/{costCenterID}'}

    def move_cost_center(
            self, parent_cost_center_id, cost_center_ids=None, custom_headers=None, raw=False, **operation_config):
        """Move list of costCenters to be under parentCostCenterId.

        :param parent_cost_center_id: The identifier for the new parent
         costCenter
        :type parent_cost_center_id: int
        :param cost_center_ids: List of costCenterIds to move under the
         parentCostCenterId
        :type cost_center_ids: list[int]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.move_cost_center.metadata['url']
        path_format_arguments = {
            'parentCostCenterId': self._serialize.url("parent_cost_center_id", parent_cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if cost_center_ids is not None:
            body_content = self._serialize.body(cost_center_ids, '[int]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    move_cost_center.metadata = {'url': '/api/v3/parentCostCenter/{parentCostCenterId}/costCenter'}

    def get_cost_center_actual_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by costCenter for a given
        number of years- Result includes cost, use, unit cost on an yearly
        basis. It also includes commodity level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.CostCenterDigestCostCenterDigestActualYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_cost_center_actual_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CostCenterDigestCostCenterDigestActualYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_cost_center_actual_yearly_list.metadata = {'url': '/api/v3/costCenter/digest/actual/yearly'}

    def get_cost_center_actual_yearly(
            self, cost_center_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by costCenter for a given
        number of years- Result includes cost, use, unit cost on an yearly
        basis. It also includes commodity level break up.

        :param cost_center_id: The costCenter to retrieve data for
        :type cost_center_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CostCenterDigestCostCenterDigestActualYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.CostCenterDigestCostCenterDigestActualYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_cost_center_actual_yearly.metadata['url']
        path_format_arguments = {
            'costCenterId': self._serialize.url("cost_center_id", cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CostCenterDigestCostCenterDigestActualYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_cost_center_actual_yearly.metadata = {'url': '/api/v3/costCenter/{costCenterId}/digest/actual/yearly'}

    def get_cost_center_actual_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by costCenter for a given
        number of years- Result includes cost, use, unit cost on a monthly
        basis. It also includes commodity level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 24 months
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.CostCenterDigestCostCenterDigestActualMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_cost_center_actual_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CostCenterDigestCostCenterDigestActualMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_cost_center_actual_monthly_list.metadata = {'url': '/api/v3/costCenter/digest/actual/monthly'}

    def get_cost_center_actual_monthly(
            self, cost_center_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by costCenter for a given
        number of years- Result includes cost, use, unit cost on a monthly
        basis. It also includes commodity level break up.

        :param cost_center_id: The costCenter to retrieve data for
        :type cost_center_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 24 months
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CostCenterDigestCostCenterDigestActualMonthlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.CostCenterDigestCostCenterDigestActualMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_cost_center_actual_monthly.metadata['url']
        path_format_arguments = {
            'costCenterId': self._serialize.url("cost_center_id", cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CostCenterDigestCostCenterDigestActualMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_cost_center_actual_monthly.metadata = {'url': '/api/v3/costCenter/{costCenterId}/digest/actual/monthly'}

    def get_customers(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves an array of customers.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CustomersCustomerResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_customers.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CustomersCustomerResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_customers.metadata = {'url': '/api/v3/customer'}

    def get_dashboards(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves an array of dashboards. The array of dashboards can be
        filtered by a number of criteria. See the filter property for more
        details.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.DashboardDashboardResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_dashboards.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[DashboardDashboardResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_dashboards.metadata = {'url': '/api/v3/dashboard'}

    def create_dashboard(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a dashboard.

        :param dto:
        :type dto: ~energycap.sdk.models.DashboardDashboardCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: DashboardDashboardResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.DashboardDashboardResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_dashboard.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'DashboardDashboardCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('DashboardDashboardResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_dashboard.metadata = {'url': '/api/v3/dashboard'}

    def dashboard_action_visible(
            self, dashboard_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Update the visibility of a specific dashboard, updating its lastViewed
        and displayOrder.

        :param dashboard_id: Specific dashboard
        :type dashboard_id: int
        :param action: Visibility action
        :type action: ~energycap.sdk.models.DashboardDashboardActionVisibleDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: DashboardDashboardResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.DashboardDashboardResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.dashboard_action_visible.metadata['url']
        path_format_arguments = {
            'dashboardId': self._serialize.url("dashboard_id", dashboard_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'DashboardDashboardActionVisibleDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('DashboardDashboardResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    dashboard_action_visible.metadata = {'url': '/api/v3/dashboard/{dashboardId}/visible'}

    def edit_dashboard(
            self, dashboard_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a dashboard.

        :param dashboard_id:
        :type dashboard_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.DashboardDashboardEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: DashboardDashboardResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.DashboardDashboardResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_dashboard.metadata['url']
        path_format_arguments = {
            'dashboardId': self._serialize.url("dashboard_id", dashboard_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'DashboardDashboardEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('DashboardDashboardResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_dashboard.metadata = {'url': '/api/v3/dashboard/{dashboardId}'}

    def delete_dashboard(
            self, dashboard_id, custom_headers=None, raw=False, **operation_config):
        """Delete a dashboard.

        :param dashboard_id: Specific dashboard
        :type dashboard_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_dashboard.metadata['url']
        path_format_arguments = {
            'dashboardId': self._serialize.url("dashboard_id", dashboard_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_dashboard.metadata = {'url': '/api/v3/dashboard/{dashboardId}'}

    def copy_dashboard(
            self, dashboard_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Copy a dashboard.

        :param dashboard_id: Specific dashboard
        :type dashboard_id: int
        :param dto: Request object
        :type dto: ~energycap.sdk.models.DashboardDashboardCopyDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: DashboardDashboardResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.DashboardDashboardResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.copy_dashboard.metadata['url']
        path_format_arguments = {
            'dashboardId': self._serialize.url("dashboard_id", dashboard_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'DashboardDashboardCopyDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('DashboardDashboardResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    copy_dashboard.metadata = {'url': '/api/v3/dashboard/{dashboardId}/copy'}

    def get_dashboard_widgets(
            self, dashboard_id, custom_headers=None, raw=False, **operation_config):
        """Get all widgets on a dashboard.

        :param dashboard_id: Dashboard Identifier
        :type dashboard_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.DashboardSpecificWidgetResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_dashboard_widgets.metadata['url']
        path_format_arguments = {
            'dashboardId': self._serialize.url("dashboard_id", dashboard_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[DashboardSpecificWidgetResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_dashboard_widgets.metadata = {'url': '/api/v3/dashboard/{dashboardId}/widget'}

    def edit_dashboard_widgets(
            self, dashboard_id, dtos=None, custom_headers=None, raw=False, **operation_config):
        """Reorder, edit, and delete specific widgets on a dashboard.

        :param dashboard_id: dashboard
        :type dashboard_id: int
        :param dtos: Request object
        :type dtos: list[~energycap.sdk.models.DashboardWidgetEditDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.DashboardSpecificWidgetResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_dashboard_widgets.metadata['url']
        path_format_arguments = {
            'dashboardId': self._serialize.url("dashboard_id", dashboard_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dtos is not None:
            body_content = self._serialize.body(dtos, '[DashboardWidgetEditDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[DashboardSpecificWidgetResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_dashboard_widgets.metadata = {'url': '/api/v3/dashboard/{dashboardId}/widget'}

    def add_widget(
            self, dashboard_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Add a widget to a dashboard.

        :param dashboard_id: dashboard
        :type dashboard_id: int
        :param dto: Request object
        :type dto: ~energycap.sdk.models.DashboardWidgetCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: DashboardSpecificWidgetResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.DashboardSpecificWidgetResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.add_widget.metadata['url']
        path_format_arguments = {
            'dashboardId': self._serialize.url("dashboard_id", dashboard_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'DashboardWidgetCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('DashboardSpecificWidgetResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    add_widget.metadata = {'url': '/api/v3/dashboard/{dashboardId}/widget'}

    def get_energy_project(
            self, project_id, custom_headers=None, raw=False, **operation_config):
        """Get an energy project.

        :param project_id: The identifier for the energy project to retrieve
        :type project_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: EnergyProjectEnergyProjectResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.EnergyProjectEnergyProjectResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_energy_project.metadata['url']
        path_format_arguments = {
            'projectId': self._serialize.url("project_id", project_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('EnergyProjectEnergyProjectResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_energy_project.metadata = {'url': '/api/v3/energyproject/{projectId}'}

    def edit_energy_project(
            self, project_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits an Energy Project.

        :param project_id: The identifier for the energy project being edited
        :type project_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.EnergyProjectEnergyProjectEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: EnergyProjectEnergyProjectResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.EnergyProjectEnergyProjectResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_energy_project.metadata['url']
        path_format_arguments = {
            'projectId': self._serialize.url("project_id", project_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'EnergyProjectEnergyProjectEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('EnergyProjectEnergyProjectResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_energy_project.metadata = {'url': '/api/v3/energyproject/{projectId}'}

    def delete_energy_project(
            self, project_id, custom_headers=None, raw=False, **operation_config):
        """Deletes an Energy Project.

        :param project_id: The identifier for the energy project being deleted
        :type project_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_energy_project.metadata['url']
        path_format_arguments = {
            'projectId': self._serialize.url("project_id", project_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_energy_project.metadata = {'url': '/api/v3/energyproject/{projectId}'}

    def create_energy_project(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates an Energy Project.

        :param dto: Data representing the Energy project being created
        :type dto: ~energycap.sdk.models.EnergyProjectEnergyProjectCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: EnergyProjectEnergyProjectResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.EnergyProjectEnergyProjectResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_energy_project.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'EnergyProjectEnergyProjectCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('EnergyProjectEnergyProjectResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_energy_project.metadata = {'url': '/api/v3/energyproject'}

    def get_energy_project_types(
            self, custom_headers=None, raw=False, **operation_config):
        """Get all the energy project types.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.EnergyProjectEnergyProjectTypeDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_energy_project_types.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[EnergyProjectEnergyProjectTypeDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_energy_project_types.metadata = {'url': '/api/v3/energyproject/energyprojecttype'}

    def create_energy_project_type(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a energy project type.

        :param dto:
        :type dto:
         ~energycap.sdk.models.EnergyProjectEnergyProjectTypeCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: EnergyProjectEnergyProjectTypeDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.EnergyProjectEnergyProjectTypeDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_energy_project_type.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'EnergyProjectEnergyProjectTypeCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('EnergyProjectEnergyProjectTypeDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_energy_project_type.metadata = {'url': '/api/v3/energyproject/energyprojecttype'}

    def get_energy_project_manager(
            self, query=None, custom_headers=None, raw=False, **operation_config):
        """Get the energy project managers.

        :param query:
        :type query: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.EnergyProjectProjectManagerResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_energy_project_manager.metadata['url']

        # Construct parameters
        query_parameters = {}
        if query is not None:
            query_parameters['query'] = self._serialize.query("query", query, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[EnergyProjectProjectManagerResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_energy_project_manager.metadata = {'url': '/api/v3/energyproject/projectmanager'}

    def empty(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.empty.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    empty.metadata = {'url': '/api/v3/empty'}

    def environment(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: EnvironmentEnvironmentDto or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.EnvironmentEnvironmentDto or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.environment.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('EnvironmentEnvironmentDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    environment.metadata = {'url': '/api/v3/environment'}

    def export_historical_bills(
            self, dto=None, export_file_name=None, custom_headers=None, raw=False, **operation_config):
        """

        :param dto:
        :type dto: ~energycap.sdk.models.BillsBillActionHistoricalExportDto
        :param export_file_name:
        :type export_file_name: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MicrosoftAspNetCoreMvcFileContentResult or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.MicrosoftAspNetCoreMvcFileContentResult
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.export_historical_bills.metadata['url']

        # Construct parameters
        query_parameters = {}
        if export_file_name is not None:
            query_parameters['exportFileName'] = self._serialize.query("export_file_name", export_file_name, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'BillsBillActionHistoricalExportDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MicrosoftAspNetCoreMvcFileContentResult', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    export_historical_bills.metadata = {'url': '/api/v3/generatebillimport'}

    def get_filter_data(
            self, data_field_id, data_field_id1, search=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves list of values for data filter.

        :param data_field_id:
        :type data_field_id: int
        :param data_field_id1:
        :type data_field_id1: str
        :param search:
        :type search: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.FilterFilterDataResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_filter_data.metadata['url']
        path_format_arguments = {
            'dataFieldId': self._serialize.url("data_field_id1", data_field_id1, 'str')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        query_parameters['dataFieldId'] = self._serialize.query("data_field_id", data_field_id, 'int')
        if search is not None:
            query_parameters['search'] = self._serialize.query("search", search, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[FilterFilterDataResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_filter_data.metadata = {'url': '/api/v3/filterData/{dataFieldId}'}

    def get_gl_sub_codes(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all General Ledger SubCode definitions.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GeneralLedgerGLSubcodeResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_gl_sub_codes.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GeneralLedgerGLSubcodeResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_gl_sub_codes.metadata = {'url': '/api/v3/glsubcode'}

    def edit_gl_sub_codes(
            self, subcode_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit General Ledger Subcode definitions - update name, type and if
        "list" type edit (add/delete/edit) pick list values.
        One or more subcodes can be edited.

        :param subcode_request_dto: Provide the subcode definition
        :type subcode_request_dto:
         list[~energycap.sdk.models.GeneralLedgerGLSubcodeRequestDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GeneralLedgerGLSubcodeResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_gl_sub_codes.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if subcode_request_dto is not None:
            body_content = self._serialize.body(subcode_request_dto, '[GeneralLedgerGLSubcodeRequestDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GeneralLedgerGLSubcodeResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_gl_sub_codes.metadata = {'url': '/api/v3/glsubcode'}

    def get_gl_sub_code(
            self, sub_code_index, custom_headers=None, raw=False, **operation_config):
        """Retrieves specific General Ledger Subcode definition.

        :param sub_code_index: The index of the subCode being retrieved
        :type sub_code_index: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GeneralLedgerGLSubcodeResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GeneralLedgerGLSubcodeResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_gl_sub_code.metadata['url']
        path_format_arguments = {
            'subCodeIndex': self._serialize.url("sub_code_index", sub_code_index, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GeneralLedgerGLSubcodeResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_gl_sub_code.metadata = {'url': '/api/v3/glsubcode/{subCodeIndex}'}

    def get_gl_sub_code_values(
            self, sub_code_index, custom_headers=None, raw=False, **operation_config):
        """Retrieves unique General Ledger Subcode values.

        :param sub_code_index: The index of the subCode whose values are being
         retrieved
        :type sub_code_index: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[str] or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_gl_sub_code_values.metadata['url']
        path_format_arguments = {
            'subCodeIndex': self._serialize.url("sub_code_index", sub_code_index, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[str]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_gl_sub_code_values.metadata = {'url': '/api/v3/glsubcode/{subCodeIndex}/savedValues'}

    def get_general_ledgers(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all General Ledgers.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.GeneralLedgerGeneralLedgerResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_general_ledgers.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GeneralLedgerGeneralLedgerResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_general_ledgers.metadata = {'url': '/api/v3/generalledger'}

    def create_general_ledger(
            self, general_ledger_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a General Ledger.

        :param general_ledger_request_dto: Data representing the general
         ledger being created
        :type general_ledger_request_dto:
         ~energycap.sdk.models.GeneralLedgerGeneralLedgerRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GeneralLedgerGeneralLedgerResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GeneralLedgerGeneralLedgerResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_general_ledger.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if general_ledger_request_dto is not None:
            body_content = self._serialize.body(general_ledger_request_dto, 'GeneralLedgerGeneralLedgerRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GeneralLedgerGeneralLedgerResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_general_ledger.metadata = {'url': '/api/v3/generalledger'}

    def get_general_ledger(
            self, general_ledger_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a specific General Ledger.

        :param general_ledger_id: General Ledger ID to return
        :type general_ledger_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GeneralLedgerGeneralLedgerResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GeneralLedgerGeneralLedgerResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_general_ledger.metadata['url']
        path_format_arguments = {
            'generalLedgerId': self._serialize.url("general_ledger_id", general_ledger_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GeneralLedgerGeneralLedgerResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_general_ledger.metadata = {'url': '/api/v3/generalledger/{generalLedgerId}'}

    def edit_general_ledger(
            self, general_ledger_id, general_ledger_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Updates a General Ledger.

        :param general_ledger_id: General Ledger ID to update
        :type general_ledger_id: int
        :param general_ledger_request_dto: Data representing the general
         ledger to update
        :type general_ledger_request_dto:
         ~energycap.sdk.models.GeneralLedgerGeneralLedgerRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GeneralLedgerGeneralLedgerResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GeneralLedgerGeneralLedgerResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_general_ledger.metadata['url']
        path_format_arguments = {
            'generalLedgerId': self._serialize.url("general_ledger_id", general_ledger_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if general_ledger_request_dto is not None:
            body_content = self._serialize.body(general_ledger_request_dto, 'GeneralLedgerGeneralLedgerRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GeneralLedgerGeneralLedgerResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_general_ledger.metadata = {'url': '/api/v3/generalledger/{generalLedgerId}'}

    def hierarchical_ancestry_places(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Returns place ancestry information, useful for populating a tree of
        places.

        :param place_id: Place ID
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: HierarchicalHierarchicalPlacesDto or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.HierarchicalHierarchicalPlacesDto or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.hierarchical_ancestry_places.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('HierarchicalHierarchicalPlacesDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    hierarchical_ancestry_places.metadata = {'url': '/api/v3/hierarchy/ancestry/place/{placeId}'}

    def hierarchical_place(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Returns place descendant information, useful for expanding a place
        node.

        :param place_id: Place ID
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: HierarchicalHierarchicalPlacesDto or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.HierarchicalHierarchicalPlacesDto or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.hierarchical_place.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('HierarchicalHierarchicalPlacesDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    hierarchical_place.metadata = {'url': '/api/v3/hierarchy/place/{placeId}'}

    def hierarchical_ancestry_cost_centers(
            self, cost_center_id, custom_headers=None, raw=False, **operation_config):
        """Returns cost center ancestry information, useful for populating a tree
        of cost centers.

        :param cost_center_id: CostCenter ID
        :type cost_center_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: HierarchicalHierarchicalCostCentersDto or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.HierarchicalHierarchicalCostCentersDto
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.hierarchical_ancestry_cost_centers.metadata['url']
        path_format_arguments = {
            'costCenterId': self._serialize.url("cost_center_id", cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('HierarchicalHierarchicalCostCentersDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    hierarchical_ancestry_cost_centers.metadata = {'url': '/api/v3/hierarchy/ancestry/costCenter/{costCenterId}'}

    def hierarchical_cost_center(
            self, cost_center_id, custom_headers=None, raw=False, **operation_config):
        """Returns costCenter descendant information, useful for expanding a
        costCenter node.

        :param cost_center_id: CostCenter ID
        :type cost_center_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: HierarchicalHierarchicalCostCentersDto or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.HierarchicalHierarchicalCostCentersDto
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.hierarchical_cost_center.metadata['url']
        path_format_arguments = {
            'costCenterId': self._serialize.url("cost_center_id", cost_center_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('HierarchicalHierarchicalCostCentersDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    hierarchical_cost_center.metadata = {'url': '/api/v3/hierarchy/costCenter/{costCenterId}'}

    def bill_import(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """This endpoint allows the import of an EnergyCAP Advanced or a profile
        mapped file. The file can be comma separated file (CSV/DAT/TXT) or
        EXCEL (XLSX) or a collection of files (ZIP). The file should be base64
        encoded with a UTF8 character set.

        :param dto: DTO containing the data to import, filename with
         extension, optional profileCode to map input file to advanced format,
         optional batch settings and optional import note/comment
        :type dto: ~energycap.sdk.models.ImportBillImportRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: TaskGuidResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.TaskGuidResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.bill_import.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'ImportBillImportRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('TaskGuidResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    bill_import.metadata = {'url': '/api/v3/import/bill'}

    def reading_import(
            self, request_dto=None, custom_headers=None, raw=False, **operation_config):
        """This endpoint allows the import of a channel reading ("interval") data
        import file. The file should be a base64-encoded delimited text file
        with a UTF8 character set.

        :param request_dto: DTO containing the data to export and the profile
         that defines the structure.
        :type request_dto: ~energycap.sdk.models.ImportReadingImportRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: TaskGuidResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.TaskGuidResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.reading_import.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if request_dto is not None:
            body_content = self._serialize.body(request_dto, 'ImportReadingImportRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('TaskGuidResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    reading_import.metadata = {'url': '/api/v3/import/reading'}

    def get_reading_import_profiles(
            self, custom_headers=None, raw=False, **operation_config):
        """Returns all Reading Import profiles.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.ImportReadingImportProfileResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_reading_import_profiles.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ImportReadingImportProfileResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_reading_import_profiles.metadata = {'url': '/api/v3/import/reading/profile'}

    def create_reading_import_profile(
            self, request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates an interval data import profile.

        :param request_dto: The details about the profile being created
        :type request_dto:
         ~energycap.sdk.models.ImportReadingImportProfileRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ImportReadingImportProfileResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ImportReadingImportProfileResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_reading_import_profile.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if request_dto is not None:
            body_content = self._serialize.body(request_dto, 'ImportReadingImportProfileRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ImportReadingImportProfileResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_reading_import_profile.metadata = {'url': '/api/v3/import/reading/profile'}

    def get_reading_import_profile(
            self, profile_id, custom_headers=None, raw=False, **operation_config):
        """Returns one reading import profile.

        :param profile_id:
        :type profile_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ImportReadingImportProfileResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ImportReadingImportProfileResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_reading_import_profile.metadata['url']
        path_format_arguments = {
            'profileId': self._serialize.url("profile_id", profile_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ImportReadingImportProfileResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_reading_import_profile.metadata = {'url': '/api/v3/import/reading/profile/{profileId}'}

    def edit_reading_import_profile(
            self, profile_id, request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits an interval data import profile.

        :param profile_id: The identifier for the profile being edited
        :type profile_id: int
        :param request_dto: The details about the profile being edited
        :type request_dto:
         ~energycap.sdk.models.ImportReadingImportProfileRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ImportReadingImportProfileResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ImportReadingImportProfileResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_reading_import_profile.metadata['url']
        path_format_arguments = {
            'profileId': self._serialize.url("profile_id", profile_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if request_dto is not None:
            body_content = self._serialize.body(request_dto, 'ImportReadingImportProfileRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ImportReadingImportProfileResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_reading_import_profile.metadata = {'url': '/api/v3/import/reading/profile/{profileId}'}

    def delete_reading_import_profile(
            self, profile_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a reading import profile.

        :param profile_id: The identifier for the profile being deleted
        :type profile_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_reading_import_profile.metadata['url']
        path_format_arguments = {
            'profileId': self._serialize.url("profile_id", profile_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_reading_import_profile.metadata = {'url': '/api/v3/import/reading/profile/{profileId}'}

    def get_bill_import_profiles(
            self, custom_headers=None, raw=False, **operation_config):
        """Get all bill import profiles.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.BillImportProfileBillImportProfileResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_import_profiles.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[BillImportProfileBillImportProfileResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_import_profiles.metadata = {'url': '/api/v3/import/bill/profile'}

    def write_bill_import_profile(
            self, request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a bill import profile.

        :param request_dto: The details about the profile being created
        :type request_dto:
         ~energycap.sdk.models.BillImportProfileBillImportProfileRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillImportProfileBillImportProfileResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.BillImportProfileBillImportProfileResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.write_bill_import_profile.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if request_dto is not None:
            body_content = self._serialize.body(request_dto, 'BillImportProfileBillImportProfileRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillImportProfileBillImportProfileResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    write_bill_import_profile.metadata = {'url': '/api/v3/import/bill/profile'}

    def get_bill_import_profile(
            self, profile_id, custom_headers=None, raw=False, **operation_config):
        """Get a bill import profile.

        :param profile_id: The identifier for the profile to get
        :type profile_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillImportProfileBillImportProfileResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.BillImportProfileBillImportProfileResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_import_profile.metadata['url']
        path_format_arguments = {
            'profileId': self._serialize.url("profile_id", profile_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillImportProfileBillImportProfileResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_import_profile.metadata = {'url': '/api/v3/import/bill/profile/{profileId}'}

    def edit_bill_import_profile(
            self, profile_id, request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a bill import profile.

        :param profile_id: The identifier for the profile being edited
        :type profile_id: int
        :param request_dto: The details about the profile being edited
        :type request_dto:
         ~energycap.sdk.models.BillImportProfileBillImportProfileRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillImportProfileBillImportProfileResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.BillImportProfileBillImportProfileResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_bill_import_profile.metadata['url']
        path_format_arguments = {
            'profileId': self._serialize.url("profile_id", profile_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if request_dto is not None:
            body_content = self._serialize.body(request_dto, 'BillImportProfileBillImportProfileRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillImportProfileBillImportProfileResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_bill_import_profile.metadata = {'url': '/api/v3/import/bill/profile/{profileId}'}

    def delete_bill_import_profile(
            self, profile_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a bill import profile.

        :param profile_id: The identifier for the profile being deleted
        :type profile_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_bill_import_profile.metadata['url']
        path_format_arguments = {
            'profileId': self._serialize.url("profile_id", profile_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_bill_import_profile.metadata = {'url': '/api/v3/import/bill/profile/{profileId}'}

    def account_list(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Get the account bills list.

        :param account_id: The account identifier to retrieve bills list for
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ListListResponse or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ListListResponse or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_list.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ListListResponse', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    account_list.metadata = {'url': '/api/v3/account/{accountId}/list/bill'}

    def account_bill_list_export(
            self, account_id, bill_export_options_dto=None, custom_headers=None, raw=False, **operation_config):
        """Export all account bills to ap or gl.

        :param account_id: The account to export all bills
        :type account_id: int
        :param bill_export_options_dto: Bill Export Options
        :type bill_export_options_dto:
         ~energycap.sdk.models.BillsBillExportOptionsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_bill_list_export.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_export_options_dto is not None:
            body_content = self._serialize.body(bill_export_options_dto, 'BillsBillExportOptionsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    account_bill_list_export.metadata = {'url': '/api/v3/account/{accountId}/list/bill/export'}

    def account_bill_list_unexport(
            self, account_id, bill_unexport_options_dto=None, custom_headers=None, raw=False, **operation_config):
        """Unexport account bills.

        :param account_id: The account for which to unexport all bills
        :type account_id: int
        :param bill_unexport_options_dto: Value of "ap" for A/P unexport, or
         "gl" for GL unexport
        :type bill_unexport_options_dto:
         ~energycap.sdk.models.BillsBillUnexportOptionsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_bill_list_unexport.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_unexport_options_dto is not None:
            body_content = self._serialize.body(bill_unexport_options_dto, 'BillsBillUnexportOptionsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    account_bill_list_unexport.metadata = {'url': '/api/v3/account/{accountId}/list/bill/unexport'}

    def account_bill_list_approve(
            self, account_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Approve/unapprove account bills.

        :param account_id: The account to approve/unapprove all bills
        :type account_id: int
        :param action: Boolean representing approve flag
        :type action: ~energycap.sdk.models.ListListActionApproveDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_bill_list_approve.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionApproveDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    account_bill_list_approve.metadata = {'url': '/api/v3/account/{accountId}/list/bill/approve'}

    def account_bill_list_delete_bills(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Delete all bills for account.

        :param account_id: The account to delete all bills from
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillDeleteActionResultDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillsBillDeleteActionResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_bill_list_delete_bills.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillDeleteActionResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    account_bill_list_delete_bills.metadata = {'url': '/api/v3/account/{accountId}/list/bill/data'}

    def account_bill_list_export_hold(
            self, account_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Hold/release account bills for export.

        :param account_id: The account to set the export hold flag on all
         bills
        :type account_id: int
        :param action: Boolean representing exportHold flag
        :type action: ~energycap.sdk.models.ListListActionExportHoldDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_bill_list_export_hold.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionExportHoldDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    account_bill_list_export_hold.metadata = {'url': '/api/v3/account/{accountId}/list/bill/exporthold'}

    def account_bill_list_void(
            self, account_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Void/unvoid account bills.

        :param account_id: The account to void/unvoid all bills
        :type account_id: int
        :param action: Boolean representing void flag
        :type action: ~energycap.sdk.models.ListListActionVoidDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_bill_list_void.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionVoidDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    account_bill_list_void.metadata = {'url': '/api/v3/account/{accountId}/list/bill/void'}

    def account_bill_list_export1(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """

        :param account_id:
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_bill_list_export1.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    account_bill_list_export1.metadata = {'url': '/api/v3/account/{accountId}/list/bill/excelexport'}

    def get_possible_destination_accounts_for_account_list(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a list of Accounts that contain meters of ALL of the
        commodities represented by the given Account's Bill List.

        :param account_id: A specific Account that should be used for finding
         matching Accounts (uses all Bills on the Account Bill List).
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsMoveBillsDestinationDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsMoveBillsDestinationDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_possible_destination_accounts_for_account_list.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsMoveBillsDestinationDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_possible_destination_accounts_for_account_list.metadata = {'url': '/api/v3/account/{accountId}/list/bill/possibleDestinationAccounts'}

    def move_bills_to_account_from_account_list(
            self, account_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Moves the bills from the given Account's Bill List to the specified
        destination Account and Meters. Each source Meter
        will be matched with a corresponding destination Meter based on
        commodity.
        <param name="accountId">
        Account for the Bill List containing a list of bills that should be
        moved. All bills from this Account's Bill List will be moved.
        </param><param name="dto">
        DTO containing the destination Account Id, the destination AcountMeter
        Ids, a flag indicating whether the Account GL
        information should be updated, and a flag indicating whether the Meter
        GL information should be updated.
        </param>.

        :param account_id:
        :type account_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.BillsBillActionMoveListDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.move_bills_to_account_from_account_list.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'BillsBillActionMoveListDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    move_bills_to_account_from_account_list.metadata = {'url': '/api/v3/account/{accountId}/list/bill/moveBills'}

    def meter_bill_list(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """

        :param meter_id:
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ListListResponse or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ListListResponse or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ListListResponse', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    meter_bill_list.metadata = {'url': '/api/v3/meter/{meterId}/list/bill'}

    def meter_bill_list_export(
            self, meter_id, bill_export_options_dto=None, custom_headers=None, raw=False, **operation_config):
        """Export all meter bills to ap or gl.

        :param meter_id: The meter to export all bills
        :type meter_id: int
        :param bill_export_options_dto: Bill Export Options
        :type bill_export_options_dto:
         ~energycap.sdk.models.BillsBillExportOptionsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list_export.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_export_options_dto is not None:
            body_content = self._serialize.body(bill_export_options_dto, 'BillsBillExportOptionsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    meter_bill_list_export.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/export'}

    def meter_bill_list_unexport(
            self, meter_id, bill_unexport_options_dto=None, custom_headers=None, raw=False, **operation_config):
        """Unexport meter bills.

        :param meter_id: The meter for which to unexport all bills
        :type meter_id: int
        :param bill_unexport_options_dto: Value of "ap" for A/P unexport, or
         "gl" for GL unexport
        :type bill_unexport_options_dto:
         ~energycap.sdk.models.BillsBillUnexportOptionsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list_unexport.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_unexport_options_dto is not None:
            body_content = self._serialize.body(bill_unexport_options_dto, 'BillsBillUnexportOptionsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    meter_bill_list_unexport.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/unexport'}

    def meter_bill_list_approve(
            self, meter_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Approve/unapprove meter bills.

        :param meter_id: The meter to approve/unapprove all bills
        :type meter_id: int
        :param action: Boolean representing approve flag
        :type action: ~energycap.sdk.models.ListListActionApproveDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list_approve.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionApproveDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    meter_bill_list_approve.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/approve'}

    def meter_bill_list_delete_bills(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Delete all bills for meter.

        :param meter_id: The meter to delete all bills from
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillDeleteActionResultDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillsBillDeleteActionResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list_delete_bills.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillDeleteActionResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    meter_bill_list_delete_bills.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/data'}

    def meter_bill_list_export_hold(
            self, meter_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Hold/release meter bills for export.

        :param meter_id: The meter to set the export hold flag on all bills
        :type meter_id: int
        :param action: Boolean representing exportHold flag
        :type action: ~energycap.sdk.models.ListListActionExportHoldDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list_export_hold.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionExportHoldDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    meter_bill_list_export_hold.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/exporthold'}

    def meter_bill_list_void(
            self, meter_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Void/unvoid meter bills.

        :param meter_id: The meter to void/unvoid all bills
        :type meter_id: int
        :param action: Boolean representing void flag
        :type action: ~energycap.sdk.models.ListListActionVoidDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list_void.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionVoidDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    meter_bill_list_void.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/void'}

    def meter_bill_list_export1(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """

        :param meter_id:
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_bill_list_export1.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    meter_bill_list_export1.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/excelexport'}

    def get_dynamic_list_with_data(
            self, list_type_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Get the definition and data for a list type.

        :param list_type_id: The list type for which the definition and data
         are to be retrieved
        :type list_type_id: int
        :param dto: DTO containing the list of initial filters to apply and a
         flag indicating whether to reset the current columns.
         Only one specific list of the specified list type exists, so columns
         and filters are shared for every dynamic list of the list type.
        :type dto: ~energycap.sdk.models.ListDynamicListRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ListListResponse or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ListListResponse or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_dynamic_list_with_data.metadata['url']
        path_format_arguments = {
            'listTypeId': self._serialize.url("list_type_id", list_type_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'ListDynamicListRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ListListResponse', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_dynamic_list_with_data.metadata = {'url': '/api/v3/dynamiclist/{listTypeId}'}

    def get_specific_list_filters(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Get the filters for a specific list.

        :param specific_list_id: The specific list for which the filters are
         to be retrieved
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CommonFilterResponse] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_specific_list_filters.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CommonFilterResponse]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_specific_list_filters.metadata = {'url': '/api/v3/list/{specificListId}/filter'}

    def edit_list_filters(
            self, specific_list_id, list_filters_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param specific_list_id:
        :type specific_list_id: int
        :param list_filters_dto:
        :type list_filters_dto:
         list[~energycap.sdk.models.CommonFilterEditDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CommonFilterResponse] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_list_filters.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if list_filters_dto is not None:
            body_content = self._serialize.body(list_filters_dto, '[CommonFilterEditDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CommonFilterResponse]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_list_filters.metadata = {'url': '/api/v3/list/{specificListId}/filter'}

    def edit_list_columns(
            self, specific_list_id, list_columns_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param specific_list_id:
        :type specific_list_id: int
        :param list_columns_dto:
        :type list_columns_dto:
         list[~energycap.sdk.models.ListListColumnEditDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ListListResponseListField] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_list_columns.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if list_columns_dto is not None:
            body_content = self._serialize.body(list_columns_dto, '[ListListColumnEditDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ListListResponseListField]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_list_columns.metadata = {'url': '/api/v3/list/{specificListId}/column'}

    def reset_specific_list(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Reset a specific list to its default columns and filters.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.reset_specific_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    reset_specific_list.metadata = {'url': '/api/v3/list/{specificListId}/reset'}

    def create_specific_list(
            self, specific_list_id, create_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a new specific list as a copy of an existing specific list.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param create_dto: The specific list create properties
        :type create_dto: ~energycap.sdk.models.ListListCopyDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ListListCreateResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ListListCreateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_specific_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if create_dto is not None:
            body_content = self._serialize.body(create_dto, 'ListListCopyDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ListListCreateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_specific_list.metadata = {'url': '/api/v3/list/{specificListId}'}

    def delete_specific_list(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Delete a specific list.

        :param specific_list_id: The id of the specific list to delete
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_specific_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_specific_list.metadata = {'url': '/api/v3/list/{specificListId}'}

    def get_lists(
            self, list_type_id=None, custom_headers=None, raw=False, **operation_config):
        """

        :param list_type_id:
        :type list_type_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ListListCreateResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_lists.metadata['url']

        # Construct parameters
        query_parameters = {}
        if list_type_id is not None:
            query_parameters['listTypeId'] = self._serialize.query("list_type_id", list_type_id, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ListListCreateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_lists.metadata = {'url': '/api/v3/list'}

    def create_specific_list1(
            self, create_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a new specific list with default list settings.

        :param create_dto: The specific list create properties
        :type create_dto: ~energycap.sdk.models.ListListCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ListListCreateResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ListListCreateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_specific_list1.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if create_dto is not None:
            body_content = self._serialize.body(create_dto, 'ListListCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ListListCreateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_specific_list1.metadata = {'url': '/api/v3/list'}

    def get_specific_list_with_data(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Get the definition and data for a specific list.

        :param specific_list_id: The specific list for which the definition
         and data are to be retrieved
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ListListResponse or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ListListResponse or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_specific_list_with_data.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ListListResponse', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_specific_list_with_data.metadata = {'url': '/api/v3/list/{specificListId}/data'}

    def delete_bills_in_bill_list(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Delete all bills in a bill list.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsBillDeleteActionResultDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.BillsBillDeleteActionResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_bills_in_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsBillDeleteActionResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    delete_bills_in_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/data'}

    def export_bill_list(
            self, specific_list_id, bill_export_options_dto=None, custom_headers=None, raw=False, **operation_config):
        """Export all bills in a list to ap or gl.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param bill_export_options_dto: Bill Export Options
        :type bill_export_options_dto:
         ~energycap.sdk.models.BillsBillExportOptionsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.export_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_export_options_dto is not None:
            body_content = self._serialize.body(bill_export_options_dto, 'BillsBillExportOptionsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    export_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/export'}

    def approve_bill_list(
            self, specific_list_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Approve/unapprove bill list.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param action: Boolean representing approve flag
        :type action: ~energycap.sdk.models.ListListActionApproveDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.approve_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionApproveDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    approve_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/approve'}

    def export_hold_bill_list(
            self, specific_list_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Hold/release bill list for export.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param action: Boolean representing exportHold flag
        :type action: ~energycap.sdk.models.ListListActionExportHoldDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.export_hold_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionExportHoldDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    export_hold_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/exporthold'}

    def void_bill_list(
            self, specific_list_id, action=None, custom_headers=None, raw=False, **operation_config):
        """Void/unvoid bill list.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param action: Boolean representing void flag
        :type action: ~energycap.sdk.models.ListListActionVoidDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.void_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if action is not None:
            body_content = self._serialize.body(action, 'ListListActionVoidDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    void_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/void'}

    def list_export(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Export list data.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.list_export.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    list_export.metadata = {'url': '/api/v3/list/{specificListId}/data/excelexport'}

    def get_possible_destination_accounts_for_list(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a list of accounts that contain meters of ALL of the
        commodities represented by the given Bill List.

        :param specific_list_id: A specific Bill List that should be used for
         finding matching Accounts.
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: BillsMoveBillsDestinationDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.BillsMoveBillsDestinationDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_possible_destination_accounts_for_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('BillsMoveBillsDestinationDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_possible_destination_accounts_for_list.metadata = {'url': '/api/v3/list/{specificListId}/possibleDestinationAccounts'}

    def move_bills_to_account_from_list(
            self, specific_list_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Moves the bills from the given Bill List to the specified destination
        Account and Meters. Each source Meter will be matched with
        a corresponding destination Meter based on commodity.
        <param name="specificListId">
        Bill List containing a list of bills that should be moved. All bills
        from this list will be moved.
        </param><param name="dto">
        DTO containing the destination Account Id, the destination AcountMeter
        Ids, a flag indicating whether the Account GL
        information should be updated, and a flag indicating whether the Meter
        GL information should be updated.
        </param>.

        :param specific_list_id:
        :type specific_list_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.BillsBillActionMoveListDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.move_bills_to_account_from_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'BillsBillActionMoveListDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    move_bills_to_account_from_list.metadata = {'url': '/api/v3/list/{specificListId}/moveBills'}

    def export_historical_bill_list(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Generate a bill import file from bills in a specific list.

        :param specific_list_id: The id of the specific list to get bills from
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MicrosoftAspNetCoreMvcFileContentResult or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.MicrosoftAspNetCoreMvcFileContentResult
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.export_historical_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MicrosoftAspNetCoreMvcFileContentResult', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    export_historical_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/bill/generateimport'}

    def account_export_historical_bill_list(
            self, account_id, custom_headers=None, raw=False, **operation_config):
        """Generate a bill import file from all bills in an account bill list.

        :param account_id: The account from which to get all bills
        :type account_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MicrosoftAspNetCoreMvcFileContentResult or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.MicrosoftAspNetCoreMvcFileContentResult
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.account_export_historical_bill_list.metadata['url']
        path_format_arguments = {
            'accountId': self._serialize.url("account_id", account_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MicrosoftAspNetCoreMvcFileContentResult', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    account_export_historical_bill_list.metadata = {'url': '/api/v3/account/{accountId}/list/bill/generateimport'}

    def meter_export_historical_bill_list(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Generate a bill import file from all bills in a meter bill list.

        :param meter_id: The meter from which to get all bills
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MicrosoftAspNetCoreMvcFileContentResult or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.MicrosoftAspNetCoreMvcFileContentResult
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.meter_export_historical_bill_list.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MicrosoftAspNetCoreMvcFileContentResult', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    meter_export_historical_bill_list.metadata = {'url': '/api/v3/meter/{meterId}/list/bill/generateimport'}

    def unexport_bill_list(
            self, specific_list_id, bill_unexport_options_dto=None, custom_headers=None, raw=False, **operation_config):
        """Unexport bill list.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param bill_unexport_options_dto: Value of "ap" for A/P unexport, or
         "gl" for GL unexport
        :type bill_unexport_options_dto:
         ~energycap.sdk.models.BillsBillUnexportOptionsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.unexport_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if bill_unexport_options_dto is not None:
            body_content = self._serialize.body(bill_unexport_options_dto, 'BillsBillUnexportOptionsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    unexport_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/unexport'}

    def split_bill_list(
            self, specific_list_id, custom_headers=None, raw=False, **operation_config):
        """Split all bills eligible to be split in a bill list.

        :param specific_list_id: The specific list identifier
        :type specific_list_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ChargebackNewChargebackTaskResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.ChargebackNewChargebackTaskResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.split_bill_list.metadata['url']
        path_format_arguments = {
            'specificListId': self._serialize.url("specific_list_id", specific_list_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ChargebackNewChargebackTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    split_bill_list.metadata = {'url': '/api/v3/list/{specificListId}/split'}

    def upgrade(
            self, custom_headers=None, raw=False, **operation_config):
        """Runs all applicable application upgrades.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.upgrade.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    upgrade.metadata = {'url': '/api/v3/me/upgrade'}

    def me(
            self, custom_headers=None, raw=False, **operation_config):
        """Intital session starting point. Support authorization via a bearer
        token or session cookie.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeMeDto or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.MeMeDto or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.me.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeMeDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    me.metadata = {'url': '/api/v3/me'}

    def update_me(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param dto:
        :type dto: ~energycap.sdk.models.MeMeRequestDto
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeMeDto or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.MeMeDto or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_me.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'MeMeRequestDto')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeMeDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_me.metadata = {'url': '/api/v3/me'}

    def get_meter_channels(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves the channels for a meter.

        :param meter_id: The identifier for the meter to retrieve the channels
         from
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ChannelChannelResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_channels.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ChannelChannelResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_channels.metadata = {'url': '/api/v3/meter/{meterId}/channel'}

    def get_meters(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all meters.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.LogicalDeviceMeterResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meters.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceMeterResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meters.metadata = {'url': '/api/v3/meter'}

    def create_meter(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a meter.

        :param dto: Data representing the meter being created
        :type dto: ~energycap.sdk.models.LogicalDeviceMeterCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: LogicalDeviceMeterResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.LogicalDeviceMeterResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_meter.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'LogicalDeviceMeterCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('LogicalDeviceMeterResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_meter.metadata = {'url': '/api/v3/meter'}

    def get_meter(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one meter.

        :param meter_id: The meter identifier for the meter being retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: LogicalDeviceMeterResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.LogicalDeviceMeterResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter.metadata['url']
        path_format_arguments = {
            'meterID': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('LogicalDeviceMeterResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter.metadata = {'url': '/api/v3/meter/{meterID}'}

    def edit_meter(
            self, meter_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a meter.

        :param meter_id: The meter identifier for the meter being edited
        :type meter_id: int
        :param dto: Data representing the meter being edited
        :type dto: ~energycap.sdk.models.LogicalDeviceMeterEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: LogicalDeviceMeterResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.LogicalDeviceMeterResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_meter.metadata['url']
        path_format_arguments = {
            'meterID': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'LogicalDeviceMeterEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('LogicalDeviceMeterResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_meter.metadata = {'url': '/api/v3/meter/{meterID}'}

    def delete_meter(
            self, meter_id, delete_bills=None, custom_headers=None, raw=False, **operation_config):
        """Deletes a meter.

        :param meter_id: The identifier for the meter being deleted
        :type meter_id: int
        :param delete_bills: Should this cascade to delete any bills
         associated with this meter
        :type delete_bills: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_meter.metadata['url']
        path_format_arguments = {
            'meterID': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if delete_bills is not None:
            query_parameters['deleteBills'] = self._serialize.query("delete_bills", delete_bills, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_meter.metadata = {'url': '/api/v3/meter/{meterID}'}

    def get_meter_udfs(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for a particular meter.

        :param meter_id: The meter identifier for which UDF values are being
         retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_udfs.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_udfs.metadata = {'url': '/api/v3/meter/{meterId}/udf'}

    def edit_meter_ud_fs(
            self, meter_id, udfs=None, custom_headers=None, raw=False, **operation_config):
        """Edits a meter UDF values.

        :param meter_id: The identifier for the meter being edited
        :type meter_id: int
        :param udfs: Data representing the meter UDFs being edited
        :type udfs: list[~energycap.sdk.models.UDFUDFValueDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_meter_ud_fs.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if udfs is not None:
            body_content = self._serialize.body(udfs, '[UDFUDFValueDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_meter_ud_fs.metadata = {'url': '/api/v3/meter/{meterId}/udf'}

    def get_meters_udfs(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for all meters.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFMeterUDFResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meters_udfs.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFMeterUDFResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meters_udfs.metadata = {'url': '/api/v3/meter/udf'}

    def get_meter_groups(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all Meter Groups for a particular meter.

        :param meter_id: The meter identifier for which group values are being
         retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.LogicalDeviceMeterGroupResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_groups.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceMeterGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_groups.metadata = {'url': '/api/v3/meter/{meterId}/group'}

    def edit_group_membership(
            self, meter_id, groups=None, custom_headers=None, raw=False, **operation_config):
        """Edits a meter's group memberships.
        Note that all groups a meter should belong to must be passed in.
        Passing in one group means that the meter will belong to one group
        only,
        even if it previously belonged to several.
        Only manual groups assigments can be changed; auto groups are
        unaffected.

        :param meter_id: The meter identifier whose group memberships are
         being edited
        :type meter_id: int
        :param groups: The groups the meter will belong to
        :type groups:
         list[~energycap.sdk.models.LogicalDeviceMeterGroupMemberDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.LogicalDeviceMeterGroupResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_group_membership.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if groups is not None:
            body_content = self._serialize.body(groups, '[LogicalDeviceMeterGroupMemberDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceMeterGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_group_membership.metadata = {'url': '/api/v3/meter/{meterId}/group'}

    def get_meter_rates(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all rates for a particular meter.

        :param meter_id: The meter identifier for which rate values are being
         retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.LogicalDeviceMeterRateResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_rates.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceMeterRateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_rates.metadata = {'url': '/api/v3/meter/{meterId}/rate'}

    def get_meter_templates(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all templates for a particular meter.

        :param meter_id: The meter identifier for which template values are
         being retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.LogicalDeviceMeterTemplateResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_templates.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceMeterTemplateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_templates.metadata = {'url': '/api/v3/meter/{meterId}/template'}

    def get_meter_bills(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all bills for a particular meter.

        :param meter_id: The meter identifier for which bill values are being
         retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.LogicalDeviceMeterBillResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_bills.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceMeterBillResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_bills.metadata = {'url': '/api/v3/meter/{meterId}/bill'}

    def get_ghg_meter_factors(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Get the ghg meter factors.

        :param meter_id: The meter identifier for ghg factors are being
         retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.LogicalDeviceGHGFactorsResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_ghg_meter_factors.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceGHGFactorsResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_ghg_meter_factors.metadata = {'url': '/api/v3/meter/{meterId}/ghg/factor'}

    def get_calendarized_use_vs_weather_data(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Get calendarized Use vs Weather data.

        :param meter_id: The meter identifier
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.LogicalDeviceMeterCalendarizedUseVsWeatherResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_calendarized_use_vs_weather_data.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceMeterCalendarizedUseVsWeatherResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_calendarized_use_vs_weather_data.metadata = {'url': '/api/v3/meter/{meterId}/usevsweather/calendarized'}

    def move_meter(
            self, parent_place_id, meter_ids=None, custom_headers=None, raw=False, **operation_config):
        """Move list of meters to be under parentPlaceId.

        :param parent_place_id: The identifier for the new parent place
        :type parent_place_id: int
        :param meter_ids: List of meterIds to move under the parentPlaceId
        :type meter_ids: list[int]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.move_meter.metadata['url']
        path_format_arguments = {
            'parentPlaceId': self._serialize.url("parent_place_id", parent_place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if meter_ids is not None:
            body_content = self._serialize.body(meter_ids, '[int]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    move_meter.metadata = {'url': '/api/v3/parentPlace/{parentPlaceId}/meter'}

    def get_meter_actual_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up billing data by meter for a given range of years-
        Result includes cost, use, unit cost on an yearly basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_actual_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_actual_yearly_list.metadata = {'url': '/api/v3/meter/digest/actual/yearly'}

    def get_meter_actual_yearly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data for a given meter for a given
        range of years- Result includes cost, use, unit cost on an yearly
        basis.

        :param meter_id: The identifier for the meter being retrieved
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_actual_yearly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_actual_yearly.metadata = {'url': '/api/v3/meter/{meterId}/digest/actual/yearly'}

    def get_meter_calendarized_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized data by meter for a given range of
        years- Result includes cost, use, unit cost on an yearly basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_calendarized_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_calendarized_yearly_list.metadata = {'url': '/api/v3/meter/digest/calendarized/yearly'}

    def get_meter_calendarized_yearly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized billing data for a given meter for a
        given range of years- Result includes cost, use, unit cost on an yearly
        basis.

        :param meter_id: The identifier for the meter being retrieved
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_calendarized_yearly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestActualAndCalendarizedYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_calendarized_yearly.metadata = {'url': '/api/v3/meter/{meterId}/digest/calendarized/yearly'}

    def get_meter_normalized_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized data by meter for a given range of
        years- Result includes cost, use, unit cost on an yearly basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestNormalizedYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_normalized_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestNormalizedYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_normalized_yearly_list.metadata = {'url': '/api/v3/meter/digest/normalized/yearly'}

    def get_meter_normalized_yearly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized billing data for a given meter for a
        given range of years- Result includes cost, use, unit cost on an yearly
        basis.

        :param meter_id: The identifier for the meter being retrieved
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestNormalizedYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestNormalizedYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_normalized_yearly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestNormalizedYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_normalized_yearly.metadata = {'url': '/api/v3/meter/{meterId}/digest/normalized/yearly'}

    def get_meter_savings_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves savings data by meter for a given number of years.

        :param number_of_years: Number of years of data to return including
         the current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestSavingsYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_savings_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestSavingsYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_savings_yearly_list.metadata = {'url': '/api/v3/meter/digest/savings/yearly'}

    def get_meter_savings_yearly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves savings data for a single meter for a given number of years.

        :param meter_id: The meter for which to return data
        :type meter_id: int
        :param number_of_years: Number of years of data to return including
         the current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestSavingsYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestSavingsYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_savings_yearly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestSavingsYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_savings_yearly.metadata = {'url': '/api/v3/meter/{meterId}/digest/savings/yearly'}

    def get_meter_ghg_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves Greenhouse Gas data by meter for a given number of years.

        :param number_of_years: Number of years of data to return including
         the current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestGHGYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_ghg_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestGHGYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_ghg_yearly_list.metadata = {'url': '/api/v3/meter/digest/ghg/yearly'}

    def get_meter_ghg_yearly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves Greenhouse Gas data for a single meter for a given number of
        years.

        :param meter_id: The meter for which to return data
        :type meter_id: int
        :param number_of_years: Number of years of data to return including
         the current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestGHGYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestGHGYearlyResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_ghg_yearly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestGHGYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_ghg_yearly.metadata = {'url': '/api/v3/meter/{meterId}/digest/ghg/yearly'}

    def get_meter_actual_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by meter for a given range of
        months- Result includes cost, use, unit cost on a monthly basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_actual_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_actual_monthly_list.metadata = {'url': '/api/v3/meter/digest/actual/monthly'}

    def get_meter_actual_monthly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data for a given meter for a given
        range of months- Result includes cost, use, unit cost on a monthly
        basis.

        :param meter_id: The identifier for the meter being retrieved
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_actual_monthly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_actual_monthly.metadata = {'url': '/api/v3/meter/{meterId}/digest/actual/monthly'}

    def get_meter_calendarized_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized billing data by meter for a given
        range of months- Result includes cost, use, unit cost on a monthly
        basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_calendarized_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_calendarized_monthly_list.metadata = {'url': '/api/v3/meter/digest/calendarized/monthly'}

    def get_meter_calendarized_monthly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized billing data for a given meter for a
        given range of months- Result includes cost, use, unit cost on a
        monthly basis.

        :param meter_id: The identifier for the meter being retrieved
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_calendarized_monthly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestActualAndCalendarizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_calendarized_monthly.metadata = {'url': '/api/v3/meter/{meterId}/digest/calendarized/monthly'}

    def get_meter_normalized_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized billing data by meter for a given range
        of months- Result includes cost, use, unit cost on a monthly basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.MeterDigestMeterDigestNormalizedMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_normalized_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[MeterDigestMeterDigestNormalizedMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_normalized_monthly_list.metadata = {'url': '/api/v3/meter/digest/normalized/monthly'}

    def get_meter_normalized_monthly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized billing data for a given meter for a
        given range of months- Result includes cost, use, unit cost on a
        monthly basis.

        :param meter_id: The identifier for the meter being retrieved
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterDigestMeterDigestNormalizedMonthlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterDigestMeterDigestNormalizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_normalized_monthly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterDigestMeterDigestNormalizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_normalized_monthly.metadata = {'url': '/api/v3/meter/{meterId}/digest/normalized/monthly'}

    def get_meter_savings_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up savings billing data by meter for a given number of
        years- Result includes cost and use savings on a monthly basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years/24 months
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.RollupChartMeterDigestSavingsMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_savings_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[RollupChartMeterDigestSavingsMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_savings_monthly_list.metadata = {'url': '/api/v3/meter/digest/savings/monthly'}

    def get_meter_savings_monthly(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up savings billing data for a given meter for a given
        number of years- Result includes cost and use savings on a monthly
        basis.

        :param meter_id: The identifier for the meter being retrieved
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years/24 months
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RollupChartMeterDigestSavingsMonthlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.RollupChartMeterDigestSavingsMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_savings_monthly.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RollupChartMeterDigestSavingsMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_savings_monthly.metadata = {'url': '/api/v3/meter/{meterId}/digest/savings/monthly'}

    def get_meter_groups1(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Gets a list of meter groups.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GroupMeterGroupResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_groups1.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GroupMeterGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_groups1.metadata = {'url': '/api/v3/meterGroup'}

    def create_meter_group(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a meter group.

        :param dto: Data representing the meter group being created
        :type dto: ~energycap.sdk.models.GroupMeterGroupRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupMeterGroupResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.GroupMeterGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_meter_group.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupMeterGroupRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupMeterGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_meter_group.metadata = {'url': '/api/v3/meterGroup'}

    def get_meter_group(
            self, meter_group_id, custom_headers=None, raw=False, **operation_config):
        """Gets a meter group.

        :param meter_group_id:
        :type meter_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupMeterGroupResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.GroupMeterGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_group.metadata['url']
        path_format_arguments = {
            'meterGroupId': self._serialize.url("meter_group_id", meter_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupMeterGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_group.metadata = {'url': '/api/v3/meterGroup/{meterGroupId}'}

    def edit_meter_group(
            self, meter_group_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a meter group.

        :param meter_group_id: Id of the group to edit
        :type meter_group_id: int
        :param dto: Data representing the meter group being edited
        :type dto: ~energycap.sdk.models.GroupMeterGroupRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupMeterGroupResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.GroupMeterGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_meter_group.metadata['url']
        path_format_arguments = {
            'meterGroupId': self._serialize.url("meter_group_id", meter_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupMeterGroupRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupMeterGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_meter_group.metadata = {'url': '/api/v3/meterGroup/{meterGroupId}'}

    def delete_meter_group(
            self, meter_group_id, custom_headers=None, raw=False, **operation_config):
        """Delete a user-created meter group.

        :param meter_group_id:
        :type meter_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_meter_group.metadata['url']
        path_format_arguments = {
            'meterGroupId': self._serialize.url("meter_group_id", meter_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_meter_group.metadata = {'url': '/api/v3/meterGroup/{meterGroupId}'}

    def get_meter_categories(
            self, custom_headers=None, raw=False, **operation_config):
        """Gets a list of meter categories.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GroupMeterCategoryResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_categories.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GroupMeterCategoryResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_categories.metadata = {'url': '/api/v3/meterGroup/category'}

    def create_meter_category(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a meter group category.

        :param dto: Data representing the meter group category being created
        :type dto: ~energycap.sdk.models.GroupMeterCategoryRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupMeterCategoryResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GroupMeterCategoryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_meter_category.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupMeterCategoryRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupMeterCategoryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_meter_category.metadata = {'url': '/api/v3/meterGroup/category'}

    def get_meter_category(
            self, category_id, custom_headers=None, raw=False, **operation_config):
        """Retrieve a meter group category.

        :param category_id: The identifier for the meter group category being
         retrieved
        :type category_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupMeterCategoryResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GroupMeterCategoryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_category.metadata['url']
        path_format_arguments = {
            'categoryID': self._serialize.url("category_id", category_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupMeterCategoryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_category.metadata = {'url': '/api/v3/meterGroup/category/{categoryID}'}

    def get_meter_group_members(
            self, group_id, always_limit_by_topmost=None, custom_headers=None, raw=False, **operation_config):
        """Gets a list of meter group members.

        :param group_id:
        :type group_id: int
        :param always_limit_by_topmost:
        :type always_limit_by_topmost: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GroupMeterGroupMemberChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_group_members.metadata['url']
        path_format_arguments = {
            'groupId': self._serialize.url("group_id", group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if always_limit_by_topmost is not None:
            query_parameters['alwaysLimitByTopmost'] = self._serialize.query("always_limit_by_topmost", always_limit_by_topmost, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GroupMeterGroupMemberChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_group_members.metadata = {'url': '/api/v3/meterGroup/{groupId}/member'}

    def update_meter_group_member(
            self, meter_group_id, meter_id, group_member_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Update a group member's include in charts flag.

        :param meter_group_id:
        :type meter_group_id: int
        :param meter_id:
        :type meter_id: int
        :param group_member_request_dto:
        :type group_member_request_dto:
         ~energycap.sdk.models.GroupGroupMemberRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupMeterGroupMemberChildDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GroupMeterGroupMemberChildDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_meter_group_member.metadata['url']
        path_format_arguments = {
            'meterGroupId': self._serialize.url("meter_group_id", meter_group_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if group_member_request_dto is not None:
            body_content = self._serialize.body(group_member_request_dto, 'GroupGroupMemberRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupMeterGroupMemberChildDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_meter_group_member.metadata = {'url': '/api/v3/meterGroup/{meterGroupId}/member/{meterId}'}

    def include_all_meter_group_member_in_charts(
            self, meter_group_id, custom_headers=None, raw=False, **operation_config):
        """Include all group members in charts.

        :param meter_group_id:
        :type meter_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.include_all_meter_group_member_in_charts.metadata['url']
        path_format_arguments = {
            'meterGroupId': self._serialize.url("meter_group_id", meter_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    include_all_meter_group_member_in_charts.metadata = {'url': '/api/v3/meterGroup/{meterGroupId}/member/includeInCharts'}

    def edit_meter_category(
            self, category_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Update a meter group category.

        :param category_id: Id of the category to update
        :type category_id: int
        :param dto: Data to update the meter group category
        :type dto: ~energycap.sdk.models.GroupMeterCategoryRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupMeterCategoryResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GroupMeterCategoryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_meter_category.metadata['url']
        path_format_arguments = {
            'categoryId': self._serialize.url("category_id", category_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupMeterCategoryRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupMeterCategoryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_meter_category.metadata = {'url': '/api/v3/meterGroup/category/{categoryId}'}

    def delete_meter_category(
            self, category_id, custom_headers=None, raw=False, **operation_config):
        """Delete a meter group category.

        :param category_id: Id of the category to delete
        :type category_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_meter_category.metadata['url']
        path_format_arguments = {
            'categoryId': self._serialize.url("category_id", category_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_meter_category.metadata = {'url': '/api/v3/meterGroup/category/{categoryId}'}

    def get_meter_group_ranking(
            self, group_id, data_view, chart, billing_period_option=None, start_period=None, end_period=None, commodity_id=None, high_to_low_ranking=None, show_excluded_members=None, show_zero_averages=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves cost, use, unit cost or demand per day ranking data by meter
        group.
        Specify values for any query parameters whose database values are to be
        overridden.

        :param group_id: The meter group to retrieve data for
        :type group_id: int
        :param data_view: The desired data view - actual, calendarized,
         normalized
        :type data_view: int
        :param chart: The desired chart - cost, use, unitcost, demand
        :type chart: int
        :param billing_period_option: The billing period option -
         last12periods, lastyear, lastfiscalyear, custom (optional)
        :type billing_period_option: int
        :param start_period: The custom start period (optional)
        :type start_period: int
        :param end_period: The custom end period (optional)
        :type end_period: int
        :param commodity_id: Id of commodity to filter by (optional)
        :type commodity_id: int
        :param high_to_low_ranking: Ranking from high to low, or low to high?
         (optional)
        :type high_to_low_ranking: bool
        :param show_excluded_members: Show group members excluded from the
         group? (optional)
        :type show_excluded_members: bool
        :param show_zero_averages: Show group members whose ranking averages
         are zero? (optional)
        :type show_zero_averages: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: MeterGroupDigestMeterGroupDigestResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.MeterGroupDigestMeterGroupDigestResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_meter_group_ranking.metadata['url']
        path_format_arguments = {
            'groupId': self._serialize.url("group_id", group_id, 'int'),
            'dataView': self._serialize.url("data_view", data_view, 'int'),
            'chart': self._serialize.url("chart", chart, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if billing_period_option is not None:
            query_parameters['billingPeriodOption'] = self._serialize.query("billing_period_option", billing_period_option, 'int')
        if start_period is not None:
            query_parameters['startPeriod'] = self._serialize.query("start_period", start_period, 'int')
        if end_period is not None:
            query_parameters['endPeriod'] = self._serialize.query("end_period", end_period, 'int')
        if commodity_id is not None:
            query_parameters['commodityId'] = self._serialize.query("commodity_id", commodity_id, 'int')
        if high_to_low_ranking is not None:
            query_parameters['highToLowRanking'] = self._serialize.query("high_to_low_ranking", high_to_low_ranking, 'bool')
        if show_excluded_members is not None:
            query_parameters['showExcludedMembers'] = self._serialize.query("show_excluded_members", show_excluded_members, 'bool')
        if show_zero_averages is not None:
            query_parameters['showZeroAverages'] = self._serialize.query("show_zero_averages", show_zero_averages, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MeterGroupDigestMeterGroupDigestResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_meter_group_ranking.metadata = {'url': '/api/v3/meterGroup/{groupId}/digest/{dataView}/{chart}'}

    def create_readings(
            self, dtos=None, custom_headers=None, raw=False, **operation_config):
        """Creates new readings.

        :param dtos: Data representing the readings being imported
        :type dtos: list[~energycap.sdk.models.ReadingReadingCreateDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReadingReadingResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ReadingReadingResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_readings.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dtos is not None:
            body_content = self._serialize.body(dtos, '[ReadingReadingCreateDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReadingReadingResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_readings.metadata = {'url': '/api/v3/reading'}

    def delete_readings(
            self, delete_dt_os=None, custom_headers=None, raw=False, **operation_config):
        """Deletes one or more sets of meter readings.
        Each set of meter readings to be deleted is on one channel and covers a
        time range signified by a begin and end date and time.
        The time range includes all readings greater than the begin time and
        less than or equal to the end date and time.
        To delete only one reading, set the begin and end date and time to the
        same value.

        :param delete_dt_os: The channel id, begin date and time and end date
         and time of the set of meter readings to be deleted
        :type delete_dt_os:
         list[~energycap.sdk.models.ReadingReadingDeleteDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReadingDeleteReadingResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ReadingDeleteReadingResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_readings.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if delete_dt_os is not None:
            body_content = self._serialize.body(delete_dt_os, '[ReadingReadingDeleteDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReadingDeleteReadingResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    delete_readings.metadata = {'url': '/api/v3/reading'}

    def get_route_readings(
            self, route_id, year=None, month=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves the readings for a route for one month.

        :param route_id: The identifier for the route containing the readings
        :type route_id: int
        :param year: The year of the readings
        :type year: int
        :param month: The month of the readings
        :type month: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ReadingReadingRouteResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_route_readings.metadata['url']
        path_format_arguments = {
            'routeId': self._serialize.url("route_id", route_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if year is not None:
            query_parameters['year'] = self._serialize.query("year", year, 'int')
        if month is not None:
            query_parameters['month'] = self._serialize.query("month", month, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ReadingReadingRouteResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_route_readings.metadata = {'url': '/api/v3/route/{routeId}/reading'}

    def get_readings(
            self, channel_id, begin_date=None, end_date=None, original_value=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves an array of readings. The array of readings can be filtered
        by a number of criteria. See the filter property for more details.

        :param channel_id: Specific Channel ID
        :type channel_id: int
        :param begin_date: The starting date of readings to return
        :type begin_date: datetime
        :param end_date: The end date of readings to return
        :type end_date: datetime
        :param original_value: If 'true', returns the orginal observation
         value. If 'false' returns the computed observation value based on the
         channel type.
        :type original_value: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReadingReadingDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.ReadingReadingDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_readings.metadata['url']
        path_format_arguments = {
            'channelId': self._serialize.url("channel_id", channel_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if begin_date is not None:
            query_parameters['beginDate'] = self._serialize.query("begin_date", begin_date, 'iso-8601')
        if end_date is not None:
            query_parameters['endDate'] = self._serialize.query("end_date", end_date, 'iso-8601')
        if original_value is not None:
            query_parameters['originalValue'] = self._serialize.query("original_value", original_value, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReadingReadingDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_readings.metadata = {'url': '/api/v3/channel/{channelId}/reading'}

    def get_observation_types(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Get the observation types for a given noun class code and/or noun code.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.LogicalDeviceObservationTypeChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_observation_types.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[LogicalDeviceObservationTypeChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_observation_types.metadata = {'url': '/api/v3/observationtype'}

    def get_place(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one place.

        :param place_id: The identifier for the place being retrieved
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlacePlaceResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.PlacePlaceResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place.metadata['url']
        path_format_arguments = {
            'placeID': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlacePlaceResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place.metadata = {'url': '/api/v3/place/{placeID}'}

    def get_places(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all places.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.PlacePlaceResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_places.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlacePlaceResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_places.metadata = {'url': '/api/v3/place'}

    def create_place(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a place.

        :param dto: Data representing the place being created
        :type dto: ~energycap.sdk.models.PlacePlaceCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlacePlaceResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.PlacePlaceResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_place.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'PlacePlaceCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlacePlaceResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_place.metadata = {'url': '/api/v3/place'}

    def edit_place(
            self, place_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a place.

        :param place_id: The identifier for the place being edited
        :type place_id: int
        :param dto: Data representing the place being edited
        :type dto: ~energycap.sdk.models.PlacePlaceEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlacePlaceResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.PlacePlaceResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_place.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'PlacePlaceEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlacePlaceResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_place.metadata = {'url': '/api/v3/place/{placeId}'}

    def delete_place(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a place.

        :param place_id: The identifier for the place being deleted
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_place.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_place.metadata = {'url': '/api/v3/place/{placeId}'}

    def get_place_types(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all place types.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.PlacePlaceTypeResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_types.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlacePlaceTypeResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_types.metadata = {'url': '/api/v3/place/placetype'}

    def get_place_udfs(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for a particular place.

        :param place_id: The place identifier for which UDF values are being
         retrieved
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_udfs.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_udfs.metadata = {'url': '/api/v3/place/{placeId}/udf'}

    def edit_place_ud_fs(
            self, place_id, udfs=None, custom_headers=None, raw=False, **operation_config):
        """Edits a place UDF values.

        :param place_id: The identifier for the place being edited
        :type place_id: int
        :param udfs: Data representing the place UDFs being edited
        :type udfs: list[~energycap.sdk.models.UDFUDFValueDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_place_ud_fs.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if udfs is not None:
            body_content = self._serialize.body(udfs, '[UDFUDFValueDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_place_ud_fs.metadata = {'url': '/api/v3/place/{placeId}/udf'}

    def get_place_sizes(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all place size records for a particular place.

        :param place_id: The identifier for the place being retrieved
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.PlacePlaceSizeChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_sizes.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlacePlaceSizeChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_sizes.metadata = {'url': '/api/v3/place/{placeId}/size'}

    def edit_place_size_list(
            self, place_id, place_size_dto_list=None, custom_headers=None, raw=False, **operation_config):
        """API endpoint which takes care of Create, Edit and Delete of new place
        sizes in bulk.

        :param place_id: The identifier of the place
        :type place_id: int
        :param place_size_dto_list: Data representing the place size being
         created
        :type place_size_dto_list:
         list[~energycap.sdk.models.PlacePlaceSizeCreateDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.PlacePlaceSizeChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_place_size_list.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if place_size_dto_list is not None:
            body_content = self._serialize.body(place_size_dto_list, '[PlacePlaceSizeCreateDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlacePlaceSizeChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_place_size_list.metadata = {'url': '/api/v3/place/{placeId}/size'}

    def create_place_size(
            self, place_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a place size version
        If a preceding place size version exists, the preceding record's end
        date will be changed to match the created record's begin date
        If a succeeding place size version exists the created records end date
        will match the succeeding version's begin date.

        :param place_id: The identifier of the place to add a place size to
        :type place_id: int
        :param dto: Data representing the place size being created
        :type dto: ~energycap.sdk.models.PlacePlaceSizeCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlacePlaceSizeChildDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.PlacePlaceSizeChildDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_place_size.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'PlacePlaceSizeCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlacePlaceSizeChildDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_place_size.metadata = {'url': '/api/v3/place/{placeId}/size'}

    def get_place_size(
            self, place_id, place_size_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a place size record for a particular place.

        :param place_id: The identifier for the place being retrieved
        :type place_id: int
        :param place_size_id: The identifier for the place size being
         retrieved
        :type place_size_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlacePlaceSizeChildDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.PlacePlaceSizeChildDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_size.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int'),
            'placeSizeId': self._serialize.url("place_size_id", place_size_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlacePlaceSizeChildDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_size.metadata = {'url': '/api/v3/place/{placeId}/size/{placeSizeId}'}

    def delete_place_size(
            self, place_id, place_size_id, custom_headers=None, raw=False, **operation_config):
        """Delete a place size version
        The preceding version's end date will be shifted to the deleted
        record's end date to prevent any gaps.

        :param place_id: The identifier of the place
        :type place_id: int
        :param place_size_id: The identifier of the place size
        :type place_size_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_place_size.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int'),
            'placeSizeId': self._serialize.url("place_size_id", place_size_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_place_size.metadata = {'url': '/api/v3/place/{placeId}/size/{placeSizeId}'}

    def get_places_udfs(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for all places.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFPlaceUDFResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_places_udfs.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFPlaceUDFResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_places_udfs.metadata = {'url': '/api/v3/place/udf'}

    def get_place_energy_star_place_types(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrives all energystar place types for a place.

        :param place_id: The identifier for the place for which energy star
         place types are being retrieved
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlacePlaceEnergyStarPlaceTypeResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_energy_star_place_types.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlacePlaceEnergyStarPlaceTypeResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_energy_star_place_types.metadata = {'url': '/api/v3/place/{placeId}/energystarplacetype'}

    def create_energy_star_place_type(
            self, place_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a energy type place type.

        :param place_id: The identifier for the place for which energy star
         place type is being created
        :type place_id: int
        :param dto: Data representing the energy star place type being created
        :type dto:
         ~energycap.sdk.models.PlacePlaceEnergyStarPlaceTypeCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlacePlaceEnergyStarPlaceTypeResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.PlacePlaceEnergyStarPlaceTypeResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_energy_star_place_type.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'PlacePlaceEnergyStarPlaceTypeCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlacePlaceEnergyStarPlaceTypeResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_energy_star_place_type.metadata = {'url': '/api/v3/place/{placeId}/energystarplacetype'}

    def edit_energy_star_property(
            self, place_id, place_energy_star_place_type_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a energy type place type property.

        :param place_id: The identifier for the place for which energy star
         place type are being retrieved
        :type place_id: int
        :param place_energy_star_place_type_id: The identifier for energy star
         place type being retrieved
        :type place_energy_star_place_type_id: int
        :param dto: Data representing the energy star place type being edited
        :type dto: ~energycap.sdk.models.PlacePlaceEnergyStarPlaceTypeEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlacePlaceEnergyStarPlaceTypeResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.PlacePlaceEnergyStarPlaceTypeResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_energy_star_property.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int'),
            'placeEnergyStarPlaceTypeID': self._serialize.url("place_energy_star_place_type_id", place_energy_star_place_type_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'PlacePlaceEnergyStarPlaceTypeEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlacePlaceEnergyStarPlaceTypeResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_energy_star_property.metadata = {'url': '/api/v3/place/{placeId}/energystarplacetype/{placeEnergyStarPlaceTypeID}'}

    def delete_energy_star_place_type(
            self, place_id, place_energy_star_place_type_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a place energy type place type property.

        :param place_id: The identifier for the place for which energy star
         place types are being retrieved
        :type place_id: int
        :param place_energy_star_place_type_id:
        :type place_energy_star_place_type_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_energy_star_place_type.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int'),
            'placeEnergyStarPlaceTypeID': self._serialize.url("place_energy_star_place_type_id", place_energy_star_place_type_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_energy_star_place_type.metadata = {'url': '/api/v3/place/{placeId}/energystarplacetype/{placeEnergyStarPlaceTypeID}'}

    def get_place_groups(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all Place Groups for a particular place by placeId.

        :param place_id: The place identifier for which group values are being
         retrieved
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.PlacePlaceGroupResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_groups.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlacePlaceGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_groups.metadata = {'url': '/api/v3/place/{placeId}/group'}

    def edit_group_membership1(
            self, place_id, groups=None, custom_headers=None, raw=False, **operation_config):
        """Edits a place's group memberships.
        Note that all groups a place should belong to must be passed in.
        Passing in one group means that the place will belong to one group
        only,
        even if it previously belonged to several.
        Only manual groups assigments can be changed; auto groups are
        unaffected.

        :param place_id: The place identifier whose group memberships are
         being edited
        :type place_id: int
        :param groups: The groups the place will belong to
        :type groups: list[~energycap.sdk.models.PlacePlaceGroupMemberDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.PlacePlaceGroupResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_group_membership1.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if groups is not None:
            body_content = self._serialize.body(groups, '[PlacePlaceGroupMemberDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlacePlaceGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_group_membership1.metadata = {'url': '/api/v3/place/{placeId}/group'}

    def get_energy_projects_for_place(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all Energy Projects for a particular place by placeId. For
        organizations it will be the rollup of its children's energy projects.

        :param place_id: The place identifier for which energy projects are
         being retrieved
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.EnergyProjectEnergyProjectResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_energy_projects_for_place.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[EnergyProjectEnergyProjectResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_energy_projects_for_place.metadata = {'url': '/api/v3/place/{placeId}/project'}

    def move_place(
            self, parent_place_id, place_ids=None, custom_headers=None, raw=False, **operation_config):
        """Move list of places to be under parentPlaceId.

        :param parent_place_id: The identifier for the new parent place
        :type parent_place_id: int
        :param place_ids: List of placeIds to move under the parentPlaceId
        :type place_ids: list[int]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonUpdateResultDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonUpdateResultDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.move_place.metadata['url']
        path_format_arguments = {
            'parentPlaceId': self._serialize.url("parent_place_id", parent_place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if place_ids is not None:
            body_content = self._serialize.body(place_ids, '[int]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonUpdateResultDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    move_place.metadata = {'url': '/api/v3/parentPlace/{parentPlaceId}/place'}

    def get_place_actual_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by building/organization for a
        given number of years- Result includes cost, use, unit cost on an
        yearly basis. It also includes target comparison data and commodity
        level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_actual_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_actual_yearly_list.metadata = {'url': '/api/v3/place/digest/actual/yearly'}

    def get_place_actual_yearly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by building/organization for a
        given number of years- Result includes cost, use, unit cost on an
        yearly basis. It also includes target comparison data and commodity
        level break up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_actual_yearly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_actual_yearly.metadata = {'url': '/api/v3/place/{placeId}/digest/actual/yearly'}

    def get_place_calendarized_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized billing data by building/organization
        for a given number of years- Result includes cost, use, unit cost on an
        yearly basis. It also includes target comparison data and commodity
        level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_calendarized_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_calendarized_yearly_list.metadata = {'url': '/api/v3/place/digest/calendarized/yearly'}

    def get_place_calendarized_yearly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized billing data by building/organization
        for a given number of years- Result includes cost, use, unit cost on an
        yearly basis. It also includes target comparison data and commodity
        level break up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_calendarized_yearly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestActualAndCalendarizedYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_calendarized_yearly.metadata = {'url': '/api/v3/place/{placeId}/digest/calendarized/yearly'}

    def get_place_normalized_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized billing data by building/organization
        for a given number of years- Result include use on an yearly basis. It
        also includes target comparison data and commodity level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestNormalizedYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_normalized_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestNormalizedYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_normalized_yearly_list.metadata = {'url': '/api/v3/place/digest/normalized/yearly'}

    def get_place_normalized_yearly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized billing data by building/organization
        for a given number of years- Result includes cost, use, unit cost on an
        yearly basis. It also includes target comparison data and commodity
        level break up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestNormalizedYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestNormalizedYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_normalized_yearly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestNormalizedYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_normalized_yearly.metadata = {'url': '/api/v3/place/{placeId}/digest/normalized/yearly'}

    def get_place_savings_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up savings billing data by building/organization for a
        given number of years- Result includes cost, use, unit cost on an
        yearly basis. It also includes a commodity level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestSavingsYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_savings_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestSavingsYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_savings_yearly_list.metadata = {'url': '/api/v3/place/digest/savings/yearly'}

    def get_place_savings_yearly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """

        :param place_id:
        :type place_id: int
        :param number_of_years:
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestSavingsYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestSavingsYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_savings_yearly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestSavingsYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_savings_yearly.metadata = {'url': '/api/v3/place/{placeId}/digest/savings/yearly'}

    def get_place_actual_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by building/organization for a
        given number of years- Result includes cost, use, unit cost on a
        monthly basis. It also includes target comparison data and commodity
        level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_actual_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_actual_monthly_list.metadata = {'url': '/api/v3/place/digest/actual/monthly'}

    def get_place_actual_monthly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by building/organization for a
        given number of years- Result includes cost, use, unit cost on a
        monthly basis. It also includes target comparison data and commodity
        level break up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_actual_monthly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_actual_monthly.metadata = {'url': '/api/v3/place/{placeId}/digest/actual/monthly'}

    def get_place_calendarized_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized billing data by building/organization
        for a given number of years- Result includes cost, use, unit cost on a
        monthly basis. It also includes target comparison data and commodity
        level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for calendarized
         and savings, and 24 or 26 (if 13 accouting periods) for calendarized
         and normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_calendarized_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_calendarized_monthly_list.metadata = {'url': '/api/v3/place/digest/calendarized/monthly'}

    def get_place_calendarized_monthly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up calendarized billing data by building/organization
        for a given number of years- Result includes cost, use, unit cost on a
        monthly basis. It also includes target comparison data and commodity
        level break up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for calendarized
         and savings, and 24 or 26 (if 13 accouting periods) for calendarized
         and normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_calendarized_monthly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_calendarized_monthly.metadata = {'url': '/api/v3/place/{placeId}/digest/calendarized/monthly'}

    def get_place_nomalized_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized billing data by building/organization
        for a given number of years- Result includes use on a monthly basis. It
        also includes target comparison data and commodity level break up.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for normalized and
         savings, and 24 or 26 (if 13 accouting periods) for normalized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestNormalizedMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_nomalized_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestNormalizedMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_nomalized_monthly_list.metadata = {'url': '/api/v3/place/digest/normalized/monthly'}

    def get_place_nomalized_monthly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up normalized billing data by building/organization
        for a given number of years- Result includes cost, use, unit cost on a
        monthly basis. It also includes target comparison data and commodity
        level break up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for normalized and
         savings, and 24 or 26 (if 13 accouting periods) for normalized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestNormalizedMonthlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestNormalizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_nomalized_monthly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestNormalizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_nomalized_monthly.metadata = {'url': '/api/v3/place/{placeId}/digest/normalized/monthly'}

    def get_place_savings_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up savings data by building/organization for a given
        number of years- Result includes cost and use savings on a monthly
        basis.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years/24 months
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestSavingsMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_savings_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestSavingsMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_savings_monthly_list.metadata = {'url': '/api/v3/place/digest/savings/monthly'}

    def get_place_savings_monthly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up savings data by building/organization for a given
        number of years- Result includes cost, use, unit cost on a monthly
        basis. It also includes target comparison data and commodity level
        break up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to two years of months (24 for actual and
         savings, and 24 or 26 (if 13 accouting periods) for calendarized and
         normalized)
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestSavingsMonthlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestSavingsMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_savings_monthly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestSavingsMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_savings_monthly.metadata = {'url': '/api/v3/place/{placeId}/digest/savings/monthly'}

    def get_place_ghg_yearly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up ghg  billing data by building/organization for a
        given number of years- Result includes use broken down by type and
        scope.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestGHGYearlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_ghg_yearly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestGHGYearlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_ghg_yearly_list.metadata = {'url': '/api/v3/place/digest/ghg/yearly'}

    def get_place_ghg_yearly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up ghg  billing data by building/organization for a
        given number of years- Result includes use broken down by type and
        scope.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestGHGYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestGHGYearlyResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_ghg_yearly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestGHGYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_ghg_yearly.metadata = {'url': '/api/v3/place/{placeId}/digest/ghg/yearly'}

    def get_place_ghg_monthly_list(
            self, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up ghg  billing data by building/organization for a
        given number of years- Result includes use broken down by type and
        scope.

        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestGHGMonthlyResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_ghg_monthly_list.metadata['url']

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestGHGMonthlyResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_ghg_monthly_list.metadata = {'url': '/api/v3/place/digest/ghg/monthly'}

    def get_place_ghg_monthly(
            self, place_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up ghg  billing data by building/organization for a
        given number of years- Result includes use broken down by type and
        scope.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestGHGMonthlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestGHGMonthlyResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_ghg_monthly.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestGHGMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_ghg_monthly.metadata = {'url': '/api/v3/place/{placeId}/digest/ghg/monthly'}

    def get_eui_chart_data(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves energy use intensity data for a place.
        If data is requested for an organization, then all the places under the
        organination will be rolled up.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestEnergyUseIntensityDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_eui_chart_data.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestEnergyUseIntensityDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_eui_chart_data.metadata = {'url': '/api/v3/place/{placeId}/digest/energyUseIntensity'}

    def get_energy_star_rating_chart_data(
            self, place_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves energy star rating history for the place.

        :param place_id: The place to retrieve data for
        :type place_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestEnergyStarRatingDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_energy_star_rating_chart_data.metadata['url']
        path_format_arguments = {
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestEnergyStarRatingDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_energy_star_rating_chart_data.metadata = {'url': '/api/v3/place/{placeId}/digest/energystar/ratings'}

    def get_place_groups1(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Gets a list of place groups.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GroupPlaceGroupResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_groups1.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GroupPlaceGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_groups1.metadata = {'url': '/api/v3/placeGroup'}

    def create_place_group(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a place group.

        :param dto: Data representing the place group being created
        :type dto: ~energycap.sdk.models.GroupPlaceGroupRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupPlaceGroupResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.GroupPlaceGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_place_group.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupPlaceGroupRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupPlaceGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_place_group.metadata = {'url': '/api/v3/placeGroup'}

    def get_place_group(
            self, place_group_id, custom_headers=None, raw=False, **operation_config):
        """Gets a place group.

        :param place_group_id:
        :type place_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupPlaceGroupResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.GroupPlaceGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_group.metadata['url']
        path_format_arguments = {
            'placeGroupId': self._serialize.url("place_group_id", place_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupPlaceGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_group.metadata = {'url': '/api/v3/placeGroup/{placeGroupId}'}

    def edit_place_group(
            self, place_group_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a place group.

        :param place_group_id: Id of the group to edit
        :type place_group_id: int
        :param dto: Data representing the place group being edited
        :type dto: ~energycap.sdk.models.GroupPlaceGroupRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupPlaceGroupResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.GroupPlaceGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_place_group.metadata['url']
        path_format_arguments = {
            'placeGroupId': self._serialize.url("place_group_id", place_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupPlaceGroupRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupPlaceGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_place_group.metadata = {'url': '/api/v3/placeGroup/{placeGroupId}'}

    def delete_place_group(
            self, place_group_id, custom_headers=None, raw=False, **operation_config):
        """Delete a user-created place group.

        :param place_group_id:
        :type place_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_place_group.metadata['url']
        path_format_arguments = {
            'placeGroupId': self._serialize.url("place_group_id", place_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_place_group.metadata = {'url': '/api/v3/placeGroup/{placeGroupId}'}

    def get_place_categories(
            self, custom_headers=None, raw=False, **operation_config):
        """Gets a list of place categories.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GroupPlaceCategoryResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_categories.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GroupPlaceCategoryResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_categories.metadata = {'url': '/api/v3/placeGroup/category'}

    def create_place_category(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a place group category.

        :param dto: Data representing the place group category being created
        :type dto: ~energycap.sdk.models.GroupsPlaceCategoryRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupPlaceCategoryResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GroupPlaceCategoryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_place_category.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupsPlaceCategoryRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupPlaceCategoryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_place_category.metadata = {'url': '/api/v3/placeGroup/category'}

    def get_place_group_members(
            self, place_group_id, custom_headers=None, raw=False, **operation_config):
        """Gets a list of place group members.

        :param place_group_id:
        :type place_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.GroupPlaceGroupMemberDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_group_members.metadata['url']
        path_format_arguments = {
            'placeGroupId': self._serialize.url("place_group_id", place_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[GroupPlaceGroupMemberDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_group_members.metadata = {'url': '/api/v3/placeGroup/{placeGroupId}/member'}

    def update_place_group_member(
            self, place_group_id, place_id, group_member_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Update a group member's include in charts flag.

        :param place_group_id:
        :type place_group_id: int
        :param place_id:
        :type place_id: int
        :param group_member_request_dto:
        :type group_member_request_dto:
         ~energycap.sdk.models.GroupGroupMemberRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupPlaceGroupMemberDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.GroupPlaceGroupMemberDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.update_place_group_member.metadata['url']
        path_format_arguments = {
            'placeGroupId': self._serialize.url("place_group_id", place_group_id, 'int'),
            'placeId': self._serialize.url("place_id", place_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if group_member_request_dto is not None:
            body_content = self._serialize.body(group_member_request_dto, 'GroupGroupMemberRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupPlaceGroupMemberDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    update_place_group_member.metadata = {'url': '/api/v3/placeGroup/{placeGroupId}/member/{placeId}'}

    def include_all_place_group_member_in_charts(
            self, place_group_id, custom_headers=None, raw=False, **operation_config):
        """Include all group members in charts.

        :param place_group_id:
        :type place_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.include_all_place_group_member_in_charts.metadata['url']
        path_format_arguments = {
            'placeGroupId': self._serialize.url("place_group_id", place_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    include_all_place_group_member_in_charts.metadata = {'url': '/api/v3/placeGroup/{placeGroupId}/member/includeInCharts'}

    def get_place_category(
            self, category_id, custom_headers=None, raw=False, **operation_config):
        """Retrieve a Place Group Category.

        :param category_id: The identifier for the place group category being
         retrieved
        :type category_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupPlaceCategoryResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GroupPlaceCategoryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_category.metadata['url']
        path_format_arguments = {
            'categoryID': self._serialize.url("category_id", category_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupPlaceCategoryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_category.metadata = {'url': '/api/v3/placeGroup/category/{categoryID}'}

    def edit_place_category(
            self, category_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Update a place group category.

        :param category_id: Id of the category to update
        :type category_id: int
        :param dto: Data to update the place group category
        :type dto: ~energycap.sdk.models.GroupsPlaceCategoryRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: GroupPlaceCategoryResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.GroupPlaceCategoryResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_place_category.metadata['url']
        path_format_arguments = {
            'categoryId': self._serialize.url("category_id", category_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'GroupsPlaceCategoryRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('GroupPlaceCategoryResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_place_category.metadata = {'url': '/api/v3/placeGroup/category/{categoryId}'}

    def delete_place_category(
            self, category_id, custom_headers=None, raw=False, **operation_config):
        """Delete a place group category.

        :param category_id: Id of the category to delete
        :type category_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_place_category.metadata['url']
        path_format_arguments = {
            'categoryId': self._serialize.url("category_id", category_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_place_category.metadata = {'url': '/api/v3/placeGroup/category/{categoryId}'}

    def get_place_group_ranking(
            self, group_id, data_view, chart, billing_period_option=None, start_period=None, end_period=None, commodity_id=None, high_to_low_ranking=None, show_excluded_members=None, show_zero_averages=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves ranking data by place group.
        Depending on the "chart" parameter retrieved ranking data could be
        cost, use, or demand per area, cost or use per day, heating sensitivity
        or cooling sensitivity
        Specify values for any query parameters whose database values are to be
        overridden.
        For the heating sensitivity and cooling sensitivity charts we sort by
        weather sensitivity (slope) and exclude non-weather meters.

        :param group_id: The place group to retrieve data for
        :type group_id: int
        :param data_view: The desired data view - actual, calendarized,
         normalized
        :type data_view: int
        :param chart: The desired chart - areacost, areause, areademand, cost,
         use, heatingsensitivity, coolingsensitivity
        :type chart: int
        :param billing_period_option: The billing period option -
         last12periods, currentyear, lastfiscalyear, custom (optional)
        :type billing_period_option: int
        :param start_period: The custom start period (optional)
        :type start_period: int
        :param end_period: The custom end period (optional)
        :type end_period: int
        :param commodity_id: Id of commodity to filter by (optional)
        :type commodity_id: int
        :param high_to_low_ranking: Ranking from high to low, or low to high?
         (optional)
        :type high_to_low_ranking: bool
        :param show_excluded_members: Show group members excluded from the
         group? (optional)
        :type show_excluded_members: bool
        :param show_zero_averages: Show group members whose ranking averages
         are zero? (optional)
        :type show_zero_averages: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceGroupDigestPlaceGroupDigestResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceGroupDigestPlaceGroupDigestResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_place_group_ranking.metadata['url']
        path_format_arguments = {
            'groupId': self._serialize.url("group_id", group_id, 'int'),
            'dataView': self._serialize.url("data_view", data_view, 'int'),
            'chart': self._serialize.url("chart", chart, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if billing_period_option is not None:
            query_parameters['billingPeriodOption'] = self._serialize.query("billing_period_option", billing_period_option, 'int')
        if start_period is not None:
            query_parameters['startPeriod'] = self._serialize.query("start_period", start_period, 'int')
        if end_period is not None:
            query_parameters['endPeriod'] = self._serialize.query("end_period", end_period, 'int')
        if commodity_id is not None:
            query_parameters['commodityId'] = self._serialize.query("commodity_id", commodity_id, 'int')
        if high_to_low_ranking is not None:
            query_parameters['highToLowRanking'] = self._serialize.query("high_to_low_ranking", high_to_low_ranking, 'bool')
        if show_excluded_members is not None:
            query_parameters['showExcludedMembers'] = self._serialize.query("show_excluded_members", show_excluded_members, 'bool')
        if show_zero_averages is not None:
            query_parameters['showZeroAverages'] = self._serialize.query("show_zero_averages", show_zero_averages, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceGroupDigestPlaceGroupDigestResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_place_group_ranking.metadata = {'url': '/api/v3/placeGroup/{groupId}/digest/{dataView}/{chart}'}

    def get_postal_code(
            self, postal_code, custom_headers=None, raw=False, **operation_config):
        """Retrieves the city, state, country, and weather station for a postal
        code.

        :param postal_code:
        :type postal_code: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PostalCodePostalCodeResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.PostalCodePostalCodeResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_postal_code.metadata['url']
        path_format_arguments = {
            'postalCode': self._serialize.url("postal_code", postal_code, 'str')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PostalCodePostalCodeResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_postal_code.metadata = {'url': '/api/v3/postalcode/{postalCode}'}

    def get_primary_uses(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all Primary Uses. The array of Primary Uses can be filtered
        by Primary Use type. Possible values for this filter include: "meter",
        "place". See the filter property for more details.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.PrimaryUsePrimaryUseResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_primary_uses.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PrimaryUsePrimaryUseResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_primary_uses.metadata = {'url': '/api/v3/primaryuse'}

    def create_primary_use(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a Primary Use.

        :param dto: Data representing the Primary Use being created
        :type dto: ~energycap.sdk.models.PrimaryUsePrimaryUseCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PrimaryUsePrimaryUseResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.PrimaryUsePrimaryUseResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_primary_use.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'PrimaryUsePrimaryUseCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PrimaryUsePrimaryUseResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_primary_use.metadata = {'url': '/api/v3/primaryuse'}

    def delete_primary_use(
            self, primary_use_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a primary use.

        :param primary_use_id: The identifier for the primary use being
         deleted
        :type primary_use_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_primary_use.metadata['url']
        path_format_arguments = {
            'primaryUseID': self._serialize.url("primary_use_id", primary_use_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_primary_use.metadata = {'url': '/api/v3/primaryuse/{primaryUseID}'}

    def get_rate(
            self, rate_id, custom_headers=None, raw=False, **operation_config):
        """Gets a rate.

        :param rate_id:
        :type rate_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RateRateResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RateRateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_rate.metadata['url']
        path_format_arguments = {
            'rateID': self._serialize.url("rate_id", rate_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RateRateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_rate.metadata = {'url': '/api/v3/rate/{rateID}'}

    def get_rates(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Gets all rates.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.RateRateResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_rates.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[RateRateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_rates.metadata = {'url': '/api/v3/rate'}

    def create_rate(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a rate.

        :param dto:
        :type dto: ~energycap.sdk.models.RateRateCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RateRateResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RateRateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_rate.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'RateRateCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RateRateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_rate.metadata = {'url': '/api/v3/rate'}

    def get_rate_version(
            self, rate_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Gets a rate version.

        :param rate_id:
        :type rate_id: int
        :param version_id:
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RateRateVersionResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RateRateVersionResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_rate_version.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RateRateVersionResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_rate_version.metadata = {'url': '/api/v3/rate/{rateId}/version/{versionId}'}

    def edit_rate_version(
            self, rate_id, version_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a rate version.

        :param rate_id:
        :type rate_id: int
        :param version_id:
        :type version_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.RateRateVersionEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RateRateVersionResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RateRateVersionResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_rate_version.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'RateRateVersionEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RateRateVersionResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_rate_version.metadata = {'url': '/api/v3/rate/{rateId}/version/{versionId}'}

    def delete_rate_version(
            self, rate_id, version_id, custom_headers=None, raw=False, **operation_config):
        """

        :param rate_id:
        :type rate_id: int
        :param version_id:
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_rate_version.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_rate_version.metadata = {'url': '/api/v3/rate/{rateId}/version/{versionId}'}

    def create_rate_version(
            self, rate_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a rate version.

        :param rate_id:
        :type rate_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.RateRateVersionCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RateRateVersionResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RateRateVersionResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_rate_version.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'RateRateVersionCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RateRateVersionResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_rate_version.metadata = {'url': '/api/v3/rate/{rateId}/version'}

    def edit_rate(
            self, rate_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a rate.

        :param rate_id:
        :type rate_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.RateRateEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RateRateResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RateRateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_rate.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'RateRateEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RateRateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_rate.metadata = {'url': '/api/v3/rate/{rateId}'}

    def delete_rate(
            self, rate_id, custom_headers=None, raw=False, **operation_config):
        """

        :param rate_id:
        :type rate_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_rate.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_rate.metadata = {'url': '/api/v3/rate/{rateId}'}

    def get_rate_unit(
            self, rate_id, custom_headers=None, raw=False, **operation_config):
        """Gets the unit used by a rate. The unit is determined by the usage unit
        on the most recent template assigned to a meter with the given rate.

        :param rate_id:
        :type rate_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: UnitUnitResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.UnitUnitResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_rate_unit.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('UnitUnitResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_rate_unit.metadata = {'url': '/api/v3/rate/{rateId}/unit'}

    def get_rate_actual_yearly(
            self, rate_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by rate for a given number of
        years- Result includes cost, use, unit cost on an yearly basis.

        :param rate_id: The rate to retrieve data for
        :type rate_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RateDigestRateDigestActualYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.RateDigestRateDigestActualYearlyResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_rate_actual_yearly.metadata['url']
        path_format_arguments = {
            'rateId': self._serialize.url("rate_id", rate_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RateDigestRateDigestActualYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_rate_actual_yearly.metadata = {'url': '/api/v3/rate/{rateId}/digest/actual/yearly'}

    def get_report_groups(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ReportsReportGroupResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_report_groups.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ReportsReportGroupResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_report_groups.metadata = {'url': '/api/v3/report/reportgroup'}

    def create_report_group(
            self, report_group=None, custom_headers=None, raw=False, **operation_config):
        """

        :param report_group:
        :type report_group: ~energycap.sdk.models.ReportsReportGroupRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReportsReportGroupResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ReportsReportGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_report_group.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if report_group is not None:
            body_content = self._serialize.body(report_group, 'ReportsReportGroupRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReportsReportGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_report_group.metadata = {'url': '/api/v3/report/reportgroup'}

    def get_report_group(
            self, report_group_id, custom_headers=None, raw=False, **operation_config):
        """

        :param report_group_id:
        :type report_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReportsReportGroupResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ReportsReportGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_report_group.metadata['url']
        path_format_arguments = {
            'reportGroupId': self._serialize.url("report_group_id", report_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReportsReportGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_report_group.metadata = {'url': '/api/v3/report/reportgroup/{reportGroupId}'}

    def edit_report_group(
            self, report_group_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param report_group_id:
        :type report_group_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.ReportsReportGroupRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReportsReportGroupResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ReportsReportGroupResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_report_group.metadata['url']
        path_format_arguments = {
            'reportGroupId': self._serialize.url("report_group_id", report_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'ReportsReportGroupRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReportsReportGroupResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_report_group.metadata = {'url': '/api/v3/report/reportgroup/{reportGroupId}'}

    def delete_report_group(
            self, report_group_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a Report Group.

        :param report_group_id:
        :type report_group_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_report_group.metadata['url']
        path_format_arguments = {
            'reportGroupId': self._serialize.url("report_group_id", report_group_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_report_group.metadata = {'url': '/api/v3/report/reportgroup/{reportGroupId}'}

    def get_specific_reports(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ReportsSpecificReportResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_specific_reports.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ReportsSpecificReportResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_specific_reports.metadata = {'url': '/api/v3/report'}

    def get_specific_report(
            self, specific_report_id, custom_headers=None, raw=False, **operation_config):
        """Get a report by its specificReportID.

        :param specific_report_id:
        :type specific_report_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReportsSpecificReportResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ReportsSpecificReportResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_specific_report.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReportsSpecificReportResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_specific_report.metadata = {'url': '/api/v3/report/{specificReportId}'}

    def edit_specific_report(
            self, specific_report_id, edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a specific report.

        :param specific_report_id: The specific report to edit
        :type specific_report_id: int
        :param edit_dto: The specific report edit properties
        :type edit_dto: ~energycap.sdk.models.ReportsReportEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReportsSpecificReportResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ReportsSpecificReportResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_specific_report.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if edit_dto is not None:
            body_content = self._serialize.body(edit_dto, 'ReportsReportEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReportsSpecificReportResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_specific_report.metadata = {'url': '/api/v3/report/{specificReportId}'}

    def delete_specific_report(
            self, specific_report_id, custom_headers=None, raw=False, **operation_config):
        """

        :param specific_report_id:
        :type specific_report_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_specific_report.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_specific_report.metadata = {'url': '/api/v3/report/{specificReportId}'}

    def install_reports(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: object or ClientRawResponse if raw=true
        :rtype: object or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.install_reports.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200, 400]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('MicrosoftAspNetCoreMvcOkResult', response)
        if response.status_code == 400:
            deserialized = self._deserialize('object', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    install_reports.metadata = {'url': '/api/v3/report/install/latest'}

    def get_report_filters(
            self, specific_report_id, custom_headers=None, raw=False, **operation_config):
        """Get the filters for a specific report.

        :param specific_report_id: The specific report for which the filters
         are to be retrieved
        :type specific_report_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CommonFilterResponse] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_report_filters.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CommonFilterResponse]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_report_filters.metadata = {'url': '/api/v3/report/{specificReportId}/filter'}

    def edit_report_filters(
            self, specific_report_id, report_filters_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit the filters for a specific report.

        :param specific_report_id: The specific report for which the filters
         are to be edited
        :type specific_report_id: int
        :param report_filters_dto: The list of filters to apply
        :type report_filters_dto:
         list[~energycap.sdk.models.CommonFilterEditDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CommonFilterResponse] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_report_filters.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if report_filters_dto is not None:
            body_content = self._serialize.body(report_filters_dto, '[CommonFilterEditDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CommonFilterResponse]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_report_filters.metadata = {'url': '/api/v3/report/{specificReportId}/filter'}

    def run_report(
            self, specific_report_id, format=None, custom_headers=None, raw=False, **operation_config):
        """Run a report and download in the provided format.

        :param specific_report_id: The ID of the specific report to run
        :type specific_report_id: int
        :param format: The format in which the generated report should be
         downloaded. Supported options are ["excel", "csv", "pdf", "word"]
        :type format: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.run_report.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if format is not None:
            query_parameters['format'] = self._serialize.query("format", format, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    run_report.metadata = {'url': '/api/v3/report/exec/{specificReportId}'}

    def distribute_report(
            self, specific_report_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Execute a report, save it in a given format, and email it to a list of
        recipients.
        If distributing reports, returns a 204 No Content if onlySendIfData is
        set to True and the report generated no data;
        returns a 200 and the generated report for download or an empty body if
        onlySendIfData is set to True.

        :param specific_report_id: The ID of the specific report to run
        :type specific_report_id: int
        :param dto: Allows the user to run one report and optionally email it
         to one or more recipients. They can optionally specify whether or not
         the report(s) should be sent if they contain no data.
        :type dto: ~energycap.sdk.models.ReportsDistributedReportsSettingsDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.distribute_report.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'ReportsDistributedReportsSettingsDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    distribute_report.metadata = {'url': '/api/v3/report/exec/{specificReportId}'}

    def create_specific_report(
            self, specific_report_id, create_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a new specific report.

        :param specific_report_id: The specific report to create a copy of
        :type specific_report_id: int
        :param create_dto: The specific report create properties
        :type create_dto: ~energycap.sdk.models.ReportsReportCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: ReportsSpecificReportResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.ReportsSpecificReportResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_specific_report.metadata['url']
        path_format_arguments = {
            'specificReportId': self._serialize.url("specific_report_id", specific_report_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if create_dto is not None:
            body_content = self._serialize.body(create_dto, 'ReportsReportCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('ReportsSpecificReportResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_specific_report.metadata = {'url': '/api/v3/report/{specificReportId}/copy'}

    def run_quick_report(
            self, quick_report_dto=None, custom_headers=None, raw=False, **operation_config):
        """Generates a "quick report" in its preferred export format as identified
        by the provided report code and applying ONLY the filters supplied.

        :param quick_report_dto: Report code and filter set
        :type quick_report_dto: ~energycap.sdk.models.ReportsQuickReportDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.run_quick_report.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if quick_report_dto is not None:
            body_content = self._serialize.body(quick_report_dto, 'ReportsQuickReportDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    run_quick_report.metadata = {'url': '/api/v3/report/quick'}

    def get_routes(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves all routes.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.RouteRouteResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_routes.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[RouteRouteResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_routes.metadata = {'url': '/api/v3/route'}

    def create_route(
            self, route=None, custom_headers=None, raw=False, **operation_config):
        """Creates a new route.

        :param route: Data representing the route being created
        :type route: ~energycap.sdk.models.RouteRouteRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RouteRouteResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RouteRouteResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_route.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if route is not None:
            body_content = self._serialize.body(route, 'RouteRouteRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RouteRouteResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_route.metadata = {'url': '/api/v3/route'}

    def get_route(
            self, id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one route.

        :param id: The identifier for route being retrieved
        :type id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RouteRouteResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RouteRouteResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_route.metadata['url']
        path_format_arguments = {
            'id': self._serialize.url("id", id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RouteRouteResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_route.metadata = {'url': '/api/v3/route/{id}'}

    def edit_route(
            self, id, route_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a route.

        :param id: The identifier for the route being edited
        :type id: int
        :param route_dto: Data representing the route being edited
        :type route_dto: ~energycap.sdk.models.RouteRouteRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RouteRouteResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RouteRouteResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_route.metadata['url']
        path_format_arguments = {
            'id': self._serialize.url("id", id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if route_dto is not None:
            body_content = self._serialize.body(route_dto, 'RouteRouteRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RouteRouteResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_route.metadata = {'url': '/api/v3/route/{id}'}

    def delete_route(
            self, id, custom_headers=None, raw=False, **operation_config):
        """Deletes a route.

        :param id: The identifier for the route being deleted
        :type id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_route.metadata['url']
        path_format_arguments = {
            'id': self._serialize.url("id", id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_route.metadata = {'url': '/api/v3/route/{id}'}

    def add_meter(
            self, route_id, route_meter_dto=None, custom_headers=None, raw=False, **operation_config):
        """Adds a meter to a route.

        :param route_id: The identifier for the route to add the meters to
        :type route_id: int
        :param route_meter_dto: Object representing meter to add to the route
        :type route_meter_dto: ~energycap.sdk.models.RouteRouteMeterDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RouteRouteResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RouteRouteResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.add_meter.metadata['url']
        path_format_arguments = {
            'routeID': self._serialize.url("route_id", route_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if route_meter_dto is not None:
            body_content = self._serialize.body(route_meter_dto, 'RouteRouteMeterDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RouteRouteResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    add_meter.metadata = {'url': '/api/v3/route/{routeID}/meter'}

    def edit_meter1(
            self, route_id, meter_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Changes the display order of a route meter (routeId, meterId
        relationship).

        :param route_id: The identifier for the route the meter belongs to
        :type route_id: int
        :param meter_id: The identifier for the meter being edited
        :type meter_id: int
        :param dto: Data representing the route meter being edited
        :type dto: ~energycap.sdk.models.RouteRouteMeterEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_meter1.metadata['url']
        path_format_arguments = {
            'routeID': self._serialize.url("route_id", route_id, 'int'),
            'meterID': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'RouteRouteMeterEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    edit_meter1.metadata = {'url': '/api/v3/route/{routeID}/meter/{meterID}'}

    def remove_meter(
            self, route_id, meter_id, custom_headers=None, raw=False, **operation_config):
        """Removes a meter from a route.

        :param route_id: The identifier for the route the meter is being
         remove from
        :type route_id: int
        :param meter_id: The identifier for the meter being removed
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.remove_meter.metadata['url']
        path_format_arguments = {
            'routeID': self._serialize.url("route_id", route_id, 'int'),
            'meterID': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    remove_meter.metadata = {'url': '/api/v3/route/{routeID}/meter/{meterID}'}

    def route_search(
            self, search_term=None, custom_headers=None, raw=False, **operation_config):
        """Searches for meters and places to add to a route. Qualifying meters are
        those with a channel and not on a route.
        Meter properties that are searched include: meter code, meter info and
        meter serial number.
        Places must have qualifying meters. Place properties that are searched
        include place code and place info.

        :param search_term: The search term
        :type search_term: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: RouteLogicalDeviceSearchDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.RouteLogicalDeviceSearchDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.route_search.metadata['url']

        # Construct parameters
        query_parameters = {}
        if search_term is not None:
            query_parameters['searchTerm'] = self._serialize.query("search_term", search_term, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('RouteLogicalDeviceSearchDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    route_search.metadata = {'url': '/api/v3/route/search'}

    def get_savings_meter_bill_list(
            self, meter_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Savings Details of usage and cost avoidance for a meter's bills.

        :param meter_id: The meter to retrieve data for
        :type meter_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 5
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.SavingsSavingsMeterBillResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_savings_meter_bill_list.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[SavingsSavingsMeterBillResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_savings_meter_bill_list.metadata = {'url': '/api/v3/savings/meter/{meterId}/bill'}

    def get_savings_meter_bill(
            self, meter_id, bill_id, custom_headers=None, raw=False, **operation_config):
        """Savings Details of usage and cost avoidance for a meter and bill.

        :param meter_id: The meter to retrieve data for
        :type meter_id: int
        :param bill_id: The bill to retrieve data for
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsSavingsMeterBillResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsSavingsMeterBillResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_savings_meter_bill.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsSavingsMeterBillResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_savings_meter_bill.metadata = {'url': '/api/v3/savings/meter/{meterId}/bill/{billId}'}

    def get_savings_meter_bill_daily(
            self, meter_id, bill_id, custom_headers=None, raw=False, **operation_config):
        """Savings by day for a meter and bill.

        :param meter_id: The meter to retrieve data for
        :type meter_id: int
        :param bill_id: The bill to retrieve data for
        :type bill_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsSavingsMeterBillDailyResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.SavingsSavingsMeterBillDailyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_savings_meter_bill_daily.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsSavingsMeterBillDailyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_savings_meter_bill_daily.metadata = {'url': '/api/v3/savings/meter/{meterId}/bill/{billId}/daily'}

    def get_special_adjustment_types(
            self, custom_headers=None, raw=False, **operation_config):
        """Returns Special Adjustment Types.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.CostAvoidanceSpecialAdjustmentTypeDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_special_adjustment_types.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CostAvoidanceSpecialAdjustmentTypeDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_special_adjustment_types.metadata = {'url': '/api/v3/savings/adjustment/special/type'}

    def get_special_adjustment_methods(
            self, custom_headers=None, raw=False, **operation_config):
        """Returns list of Special Adjustment Methods.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.SavingsSpecialAdjustmentMethodDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_special_adjustment_methods.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[SavingsSpecialAdjustmentMethodDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_special_adjustment_methods.metadata = {'url': '/api/v3/savings/adjustment/special/method'}

    def get_savings_adjusted_cost_type_list(
            self, custom_headers=None, raw=False, **operation_config):
        """Gets a list of the Adjusted Cost Types.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.CostAvoidanceAdjustedCostTypeResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_savings_adjusted_cost_type_list.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CostAvoidanceAdjustedCostTypeResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_savings_adjusted_cost_type_list.metadata = {'url': '/api/v3/savings/adjustedCostType'}

    def generate_baseline_get(
            self, meter_id, save=None, custom_headers=None, raw=False, **operation_config):
        """

        :param meter_id:
        :type meter_id: int
        :param save:
        :type save: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsBaselineProcessorResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsBaselineProcessorResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.generate_baseline_get.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if save is not None:
            query_parameters['save'] = self._serialize.query("save", save, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsBaselineProcessorResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    generate_baseline_get.metadata = {'url': '/api/v3/savings/meter/{meterId}/baseline'}

    def generate_baseline(
            self, meter_id, baseline_settings=None, save=None, custom_headers=None, raw=False, **operation_config):
        """

        :param meter_id:
        :type meter_id: int
        :param baseline_settings:
        :type baseline_settings:
         ~energycap.sdk.models.SavingsBaselineProcessorRequestDTO
        :param save:
        :type save: bool
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsBaselineProcessorResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsBaselineProcessorResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.generate_baseline.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if save is not None:
            query_parameters['save'] = self._serialize.query("save", save, 'bool')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if baseline_settings is not None:
            body_content = self._serialize.body(baseline_settings, 'SavingsBaselineProcessorRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsBaselineProcessorResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    generate_baseline.metadata = {'url': '/api/v3/savings/meter/{meterId}/baseline'}

    def manual_adjustment(
            self, meter_id, bill_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates and removes manual adjustments.

        :param meter_id: The meter for which the manual adjustment is being
         made
        :type meter_id: int
        :param bill_id: The bill for which the manual adjustment is being made
        :type bill_id: int
        :param dto:
        :type dto: ~energycap.sdk.models.SavingsManualAdjustmentRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.manual_adjustment.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'billId': self._serialize.url("bill_id", bill_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'SavingsManualAdjustmentRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    manual_adjustment.metadata = {'url': '/api/v3/savings/meter/{meterId}/bill/{billId}/manualadjustment'}

    def get_special_adjustments(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Returns all the special adjustments for the meter.

        :param meter_id: The meter to return special adjustment for
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.SavingsSpecialAdjustmentResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_special_adjustments.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[SavingsSpecialAdjustmentResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_special_adjustments.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/special'}

    def create_special_adjustment(
            self, meter_id, special_adjustment_create_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a new special adjustment.

        :param meter_id: The meter to create a special adjustment on
        :type meter_id: int
        :param special_adjustment_create_dto: Data representing the special
         adjustment being created
        :type special_adjustment_create_dto:
         ~energycap.sdk.models.SavingsSpecialAdjustmentCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsSpecialAdjustmentResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsSpecialAdjustmentResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_special_adjustment.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if special_adjustment_create_dto is not None:
            body_content = self._serialize.body(special_adjustment_create_dto, 'SavingsSpecialAdjustmentCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsSpecialAdjustmentResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_special_adjustment.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/special'}

    def get_special_adjustment(
            self, meter_id, special_adjustment_id, custom_headers=None, raw=False, **operation_config):
        """Returns special adjustments by id.

        :param meter_id: The meter to return special adjustment for
        :type meter_id: int
        :param special_adjustment_id: The special adjustment to return
        :type special_adjustment_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsSpecialAdjustmentResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsSpecialAdjustmentResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_special_adjustment.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'specialAdjustmentId': self._serialize.url("special_adjustment_id", special_adjustment_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsSpecialAdjustmentResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_special_adjustment.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/special/{specialAdjustmentId}'}

    def edit_special_adjustment(
            self, meter_id, special_adjustment_id, special_adjustment_edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit a special adjustment.

        :param meter_id: The meter with the special adjustment to edit
        :type meter_id: int
        :param special_adjustment_id: The special adjustment to edit
        :type special_adjustment_id: int
        :param special_adjustment_edit_dto: Data representing the special
         adjustment being modified
        :type special_adjustment_edit_dto:
         ~energycap.sdk.models.SavingsSpecialAdjustmentEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsSpecialAdjustmentResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsSpecialAdjustmentResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_special_adjustment.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'specialAdjustmentId': self._serialize.url("special_adjustment_id", special_adjustment_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if special_adjustment_edit_dto is not None:
            body_content = self._serialize.body(special_adjustment_edit_dto, 'SavingsSpecialAdjustmentEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsSpecialAdjustmentResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_special_adjustment.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/special/{specialAdjustmentId}'}

    def delete_special_adjustment_by_id(
            self, meter_id, special_adjustment_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a special adjustment by id.

        :param meter_id: The meter to delete the special adjustment from
        :type meter_id: int
        :param special_adjustment_id: The special adjustment to delete
        :type special_adjustment_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_special_adjustment_by_id.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'specialAdjustmentId': self._serialize.url("special_adjustment_id", special_adjustment_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_special_adjustment_by_id.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/special/{specialAdjustmentId}'}

    def get_all_other_savings(
            self, meter_id, custom_headers=None, raw=False, **operation_config):
        """Get a list of other savings on given meter.

        :param meter_id: The meter for which the other savings are being
         retrieved
        :type meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.SavingsOtherSavingsResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_all_other_savings.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[SavingsOtherSavingsResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_all_other_savings.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/othersavings'}

    def create_other_savings(
            self, meter_id, other_savings_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create an other savings adjustment.

        :param meter_id: The meter for which the other savings is being
         created
        :type meter_id: int
        :param other_savings_dto: Data representing the other savings
         adjustment being created
        :type other_savings_dto:
         ~energycap.sdk.models.SavingsOtherSavingsRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsOtherSavingsResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsOtherSavingsResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_other_savings.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if other_savings_dto is not None:
            body_content = self._serialize.body(other_savings_dto, 'SavingsOtherSavingsRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsOtherSavingsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_other_savings.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/othersavings'}

    def get_one_other_savings(
            self, meter_id, other_savings_id, custom_headers=None, raw=False, **operation_config):
        """Get one other savings record.

        :param meter_id: The meter for which the other savings are being
         retrieved
        :type meter_id: int
        :param other_savings_id: The identifier for the other savings record
         which is being retrieved
        :type other_savings_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsOtherSavingsResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsOtherSavingsResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_one_other_savings.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'otherSavingsId': self._serialize.url("other_savings_id", other_savings_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsOtherSavingsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_one_other_savings.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/othersavings/{otherSavingsId}'}

    def edit_other_savings(
            self, meter_id, other_savings_id, other_savings_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit an other savings adjustment.

        :param meter_id: The meter for which the other savings is being edited
        :type meter_id: int
        :param other_savings_id:
        :type other_savings_id: int
        :param other_savings_dto: Data representing the other savings
         adjustment being edited
        :type other_savings_dto:
         ~energycap.sdk.models.SavingsOtherSavingsRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsOtherSavingsResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsOtherSavingsResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_other_savings.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'otherSavingsId': self._serialize.url("other_savings_id", other_savings_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if other_savings_dto is not None:
            body_content = self._serialize.body(other_savings_dto, 'SavingsOtherSavingsRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsOtherSavingsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_other_savings.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/othersavings/{otherSavingsId}'}

    def delete_other_savings(
            self, meter_id, other_savings_id, custom_headers=None, raw=False, **operation_config):
        """Delete an other savings adjustment.

        :param meter_id: The meter for which the other savings is being
         deleted
        :type meter_id: int
        :param other_savings_id: The other saving identifier which is being
         deleted
        :type other_savings_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_other_savings.metadata['url']
        path_format_arguments = {
            'meterId': self._serialize.url("meter_id", meter_id, 'int'),
            'otherSavingsId': self._serialize.url("other_savings_id", other_savings_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_other_savings.metadata = {'url': '/api/v3/savings/meter/{meterId}/adjustment/othersavings/{otherSavingsId}'}

    def get_other_savings_categories(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.SavingsOtherSavingsCategoryDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_other_savings_categories.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[SavingsOtherSavingsCategoryDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_other_savings_categories.metadata = {'url': '/api/v3/savings/adjustment/othersavings/category'}

    def create_other_savings_category(
            self, other_savings_category_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """Create other savings catergory.

        :param other_savings_category_request_dto: The request dto of the
         other saving category which needs to be saved
        :type other_savings_category_request_dto:
         ~energycap.sdk.models.SavingsOtherSavingsCategoryRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SavingsOtherSavingsCategoryDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.SavingsOtherSavingsCategoryDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_other_savings_category.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if other_savings_category_request_dto is not None:
            body_content = self._serialize.body(other_savings_category_request_dto, 'SavingsOtherSavingsCategoryRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SavingsOtherSavingsCategoryDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_other_savings_category.metadata = {'url': '/api/v3/savings/adjustment/othersavings/category'}

    def get_search_result(
            self, query=None, limit=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves search result.

        :param query:
        :type query: str
        :param limit:
        :type limit: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SearchSearchResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.SearchSearchResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_search_result.metadata['url']

        # Construct parameters
        query_parameters = {}
        if query is not None:
            query_parameters['query'] = self._serialize.query("query", query, 'str')
        if limit is not None:
            query_parameters['limit'] = self._serialize.query("limit", limit, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SearchSearchResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_search_result.metadata = {'url': '/api/v3/search'}

    def get_template(
            self, type=None, filter=None, custom_headers=None, raw=False, **operation_config):
        """Returns an Excel file used within SetupSheets.

        :param type:
        :type type: int
        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_template.metadata['url']

        # Construct parameters
        query_parameters = {}
        if type is not None:
            query_parameters['type'] = self._serialize.query("type", type, 'int')
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    get_template.metadata = {'url': '/api/v3/setup'}

    def post_template(
            self, stream=None, custom_headers=None, raw=False, **operation_config):
        """Processes a SetupSheet Excel file.

        :param stream:
        :type stream: ~energycap.sdk.models.SystemIOStream
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SetupSetupDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.SetupSetupDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.post_template.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if stream is not None:
            body_content = self._serialize.body(stream, 'SystemIOStream')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SetupSetupDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    post_template.metadata = {'url': '/api/v3/setup'}

    def get_filters(
            self, type=None, custom_headers=None, raw=False, **operation_config):
        """Returns the applicable filters for a given type to filter down
        SetupSheet Templates.

        :param type:
        :type type: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CommonFilterResponse] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_filters.metadata['url']

        # Construct parameters
        query_parameters = {}
        if type is not None:
            query_parameters['type'] = self._serialize.query("type", type, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CommonFilterResponse]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_filters.metadata = {'url': '/api/v3/setup/filter'}

    def get_users(
            self, custom_headers=None, raw=False, **operation_config):
        """Return a list of users and their properties. Requires View permission
        for Users &amp; Roles.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UserUserResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_users.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UserUserResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_users.metadata = {'url': '/api/v3/user'}

    def create_user(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Create a new user. Requires Manage permission for Users &amp; Roles.

        :param dto: The object defining the properties for the new user
        :type dto: ~energycap.sdk.models.UserUserCreateRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: UserUserResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.UserUserResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_user.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'UserUserCreateRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('UserUserResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_user.metadata = {'url': '/api/v3/user'}

    def get_user(
            self, user_id, custom_headers=None, raw=False, **operation_config):
        """Return a specific user and its properties. Requires View permission for
        Users &amp; Roles.

        :param user_id: The ID of the user to return
        :type user_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: UserUserResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.UserUserResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_user.metadata['url']
        path_format_arguments = {
            'userId': self._serialize.url("user_id", user_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('UserUserResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_user.metadata = {'url': '/api/v3/user/{userId}'}

    def edit_user(
            self, user_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edit an existing user. Requires Manage permission for Users &amp;
        Roles. Please note that password is optional; all other items are
        required. MaxApprovalAmount may be null; if there is an existing value
        and null is passed, the existing value will be overwritten.

        :param user_id: The ID of the user to edit
        :type user_id: int
        :param dto: The object defining the updated properties of the existing
         user.
        :type dto: ~energycap.sdk.models.UserUserEditRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: UserUserResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.UserUserResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_user.metadata['url']
        path_format_arguments = {
            'userId': self._serialize.url("user_id", user_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'UserUserEditRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('UserUserResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_user.metadata = {'url': '/api/v3/user/{userId}'}

    def get_tags(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Get a list of tags within a tag category.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.TagTagResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_tags.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[TagTagResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_tags.metadata = {'url': '/api/v3/tag'}

    def get_tasks(
            self, task_type_code=None, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves a list of tasks.

        :param task_type_code: Task type code
        :type task_type_code: str
        :param filter: Filters to apply
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.TaskTaskResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_tasks.metadata['url']

        # Construct parameters
        query_parameters = {}
        if task_type_code is not None:
            query_parameters['taskTypeCode'] = self._serialize.query("task_type_code", task_type_code, 'str')
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[TaskTaskResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_tasks.metadata = {'url': '/api/v3/task'}

    def get_task(
            self, task_guid, custom_headers=None, raw=False, **operation_config):
        """Retrieves a task by its GUID. The output property could potentially
        contain a large amount of data. If response time or size becomes an
        issue, use field reduction to eliminate the output field.

        :param task_guid: The GUID of the task
        :type task_guid: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: TaskTaskResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.TaskTaskResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_task.metadata['url']
        path_format_arguments = {
            'taskGUID': self._serialize.url("task_guid", task_guid, 'str')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('TaskTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_task.metadata = {'url': '/api/v3/task/{taskGUID}'}

    def get_task_output_file(
            self, task_guid, custom_headers=None, raw=False, **operation_config):
        """Retrieves a task's error log as an EXCEL file by its GUID.
        At this time error logs can be downloaded for "readingimport" task type
        only.

        :param task_guid: The GUID of the task
        :type task_guid: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_task_output_file.metadata['url']
        path_format_arguments = {
            'taskGUID': self._serialize.url("task_guid", task_guid, 'str')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    get_task_output_file.metadata = {'url': '/api/v3/task/{taskGUID}/errorOutput'}

    def edit_task(
            self, task_id, task_edit_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits the note/comment associated with a task.

        :param task_id: The identifier for the task being edited
        :type task_id: int
        :param task_edit_dto: Data representing the note
        :type task_edit_dto: ~energycap.sdk.models.TaskTaskEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: TaskTaskResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.TaskTaskResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_task.metadata['url']
        path_format_arguments = {
            'taskId': self._serialize.url("task_id", task_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if task_edit_dto is not None:
            body_content = self._serialize.body(task_edit_dto, 'TaskTaskEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('TaskTaskResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_task.metadata = {'url': '/api/v3/task/{taskId}'}

    def get_templates(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all templates.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.TemplateTemplateResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_templates.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[TemplateTemplateResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_templates.metadata = {'url': '/api/v3/template'}

    def get_template1(
            self, template_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one template.

        :param template_id: The template identifier for the template being
         retrieved
        :type template_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: TemplateTemplateResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.TemplateTemplateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_template1.metadata['url']
        path_format_arguments = {
            'templateID': self._serialize.url("template_id", template_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('TemplateTemplateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_template1.metadata = {'url': '/api/v3/template/{templateID}'}

    def get_template_versions(
            self, template_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all template versions for a given template.

        :param template_id: The template identifier for the template version
         being retrieved
        :type template_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.TemplateTemplateVersionDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_template_versions.metadata['url']
        path_format_arguments = {
            'templateId': self._serialize.url("template_id", template_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[TemplateTemplateVersionDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_template_versions.metadata = {'url': '/api/v3/template/{templateId}/version'}

    def get_template_version(
            self, template_id, version_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one template version for a given template.

        :param template_id: The template identifier for the template version
         being retrieved
        :type template_id: int
        :param version_id: The identifier for the template version being
         retrieved
        :type version_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: TemplateTemplateVersionDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.TemplateTemplateVersionDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_template_version.metadata['url']
        path_format_arguments = {
            'templateId': self._serialize.url("template_id", template_id, 'int'),
            'versionId': self._serialize.url("version_id", version_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('TemplateTemplateVersionDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_template_version.metadata = {'url': '/api/v3/template/{templateId}/version/{versionId}'}

    def create_template_from_bill(
            self, bill_id, meter_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a new Template from an existing Bill's line items belonging to
        the provided Meter.

        :param bill_id: Id of Bill from which to create the Template
        :type bill_id: int
        :param meter_id: Id of Meter whose bodylines from source bill will be
         used to create the Template
        :type meter_id: int
        :param dto: Additional information about new Template
        :type dto: ~energycap.sdk.models.TemplatesCreateTemplateFromBillDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: TemplateTemplateResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.TemplateTemplateResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_template_from_bill.metadata['url']
        path_format_arguments = {
            'billId': self._serialize.url("bill_id", bill_id, 'int'),
            'meterId': self._serialize.url("meter_id", meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'TemplatesCreateTemplateFromBillDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('TemplateTemplateResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_template_from_bill.metadata = {'url': '/api/v3/bill/{billId}/meter/{meterId}/template'}

    def get_time_zones(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves the list of Time Zones.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.TimeZoneTimeZoneResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_time_zones.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[TimeZoneTimeZoneResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_time_zones.metadata = {'url': '/api/v3/timezone'}

    def get_ud_fs(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs. The array of UDFs can be filtered by UDF type.
        Possible values for this filter include: "account", "meter", "place"
        and "vendor". See the filter property for more details.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_ud_fs.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_ud_fs.metadata = {'url': '/api/v3/udf'}

    def create_udf(
            self, dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a UDF.

        :param dto: Data representing the UDF being created
        :type dto: ~energycap.sdk.models.UDFUDFCreateDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: UDFUDFResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.UDFUDFResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_udf.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'UDFUDFCreateDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('UDFUDFResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_udf.metadata = {'url': '/api/v3/udf'}

    def get_udf(
            self, udf_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one UDF.

        :param udf_id: The UDF identifier
        :type udf_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: UDFUDFResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.UDFUDFResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_udf.metadata['url']
        path_format_arguments = {
            'udfId': self._serialize.url("udf_id", udf_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('UDFUDFResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_udf.metadata = {'url': '/api/v3/udf/{udfId}'}

    def edit_udf(
            self, udf_id, dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a UDF.

        :param udf_id:
        :type udf_id: int
        :param dto: Data representing the UDF being edited
        :type dto: ~energycap.sdk.models.UDFUDFEditDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: UDFUDFResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.UDFUDFResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_udf.metadata['url']
        path_format_arguments = {
            'udfId': self._serialize.url("udf_id", udf_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if dto is not None:
            body_content = self._serialize.body(dto, 'UDFUDFEditDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('UDFUDFResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_udf.metadata = {'url': '/api/v3/udf/{udfId}'}

    def delete_udf(
            self, udf_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a UDF. Currently NO confirmation is required.

        :param udf_id: The identifier for the UDF being deleted
        :type udf_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_udf.metadata['url']
        path_format_arguments = {
            'udfId': self._serialize.url("udf_id", udf_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_udf.metadata = {'url': '/api/v3/udf/{udfId}'}

    def edit_custom_fields(
            self, udf_type, udf_update_dt_os=None, custom_headers=None, raw=False, **operation_config):
        """

        :param udf_type:
        :type udf_type: str
        :param udf_update_dt_os:
        :type udf_update_dt_os: list[~energycap.sdk.models.UDFUDFUpdateDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_custom_fields.metadata['url']
        path_format_arguments = {
            'udfType': self._serialize.url("udf_type", udf_type, 'str')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if udf_update_dt_os is not None:
            body_content = self._serialize.body(udf_update_dt_os, '[UDFUDFUpdateDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_custom_fields.metadata = {'url': '/api/v3/udf/list/{udfType}'}

    def get_units(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all units.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UnitUnitDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_units.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UnitUnitDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_units.metadata = {'url': '/api/v3/unit'}

    def get_bill_template_unit_mismatches(
            self, custom_headers=None, raw=False, **operation_config):
        """Detects account-meters having bill history (from BAM) containing use,
        actual or billed demand units that do not match the current template.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.UnitSystemBillTemplateUnitMismatchResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_template_unit_mismatches.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UnitSystemBillTemplateUnitMismatchResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_template_unit_mismatches.metadata = {'url': '/api/v3/unitsystem/billtemplateunitmismatch'}

    def unit_change_processor(
            self, account_meter_id, custom_headers=None, raw=False, **operation_config):
        """Modifies historical bills to use the units in the most current template
        assigned.

        :param account_meter_id: AccountMeterId to process
        :type account_meter_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.unit_change_processor.metadata['url']
        path_format_arguments = {
            'accountMeterId': self._serialize.url("account_meter_id", account_meter_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    unit_change_processor.metadata = {'url': '/api/v3/unitsystem/billtemplateunitmismatch/accountmeter/{accountMeterId}'}

    def get_user_roles(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.SystemUserRoleSystemUserRoleResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_user_roles.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[SystemUserRoleSystemUserRoleResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_user_roles.metadata = {'url': '/api/v3/userrole'}

    def create_user_role(
            self, system_user_role_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param system_user_role_request_dto:
        :type system_user_role_request_dto:
         ~energycap.sdk.models.SystemUserRoleSystemUserRoleRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SystemUserRoleSystemUserRoleResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.SystemUserRoleSystemUserRoleResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_user_role.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if system_user_role_request_dto is not None:
            body_content = self._serialize.body(system_user_role_request_dto, 'SystemUserRoleSystemUserRoleRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SystemUserRoleSystemUserRoleResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_user_role.metadata = {'url': '/api/v3/userrole'}

    def get_user_role(
            self, user_role_id, custom_headers=None, raw=False, **operation_config):
        """

        :param user_role_id:
        :type user_role_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SystemUserRoleSystemUserRoleResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.SystemUserRoleSystemUserRoleResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_user_role.metadata['url']
        path_format_arguments = {
            'userRoleId': self._serialize.url("user_role_id", user_role_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SystemUserRoleSystemUserRoleResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_user_role.metadata = {'url': '/api/v3/userrole/{userRoleId}'}

    def edit_user_role(
            self, user_role_id, system_user_role_request_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param user_role_id:
        :type user_role_id: int
        :param system_user_role_request_dto:
        :type system_user_role_request_dto:
         ~energycap.sdk.models.SystemUserRoleSystemUserRoleRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: SystemUserRoleSystemUserRoleResponseDTO or ClientRawResponse
         if raw=true
        :rtype: ~energycap.sdk.models.SystemUserRoleSystemUserRoleResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_user_role.metadata['url']
        path_format_arguments = {
            'userRoleId': self._serialize.url("user_role_id", user_role_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if system_user_role_request_dto is not None:
            body_content = self._serialize.body(system_user_role_request_dto, 'SystemUserRoleSystemUserRoleRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('SystemUserRoleSystemUserRoleResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_user_role.metadata = {'url': '/api/v3/userrole/{userRoleId}'}

    def delete_user_role(
            self, user_role_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a user role.

        :param user_role_id: The identifier for the role being deleted
        :type user_role_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_user_role.metadata['url']
        path_format_arguments = {
            'userRoleId': self._serialize.url("user_role_id", user_role_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_user_role.metadata = {'url': '/api/v3/userrole/{userRoleId}'}

    def get_vendor(
            self, vendor_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves one vendor.

        :param vendor_id: The identifier for the vendor being retrieved
        :type vendor_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: VendorsVendorResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.VendorsVendorResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_vendor.metadata['url']
        path_format_arguments = {
            'vendorID': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('VendorsVendorResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_vendor.metadata = {'url': '/api/v3/vendor/{vendorID}'}

    def delete_vendor(
            self, vendor_id, custom_headers=None, raw=False, **operation_config):
        """Deletes a vendor.

        :param vendor_id: The identifier for the vendor being deleted
        :type vendor_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.delete_vendor.metadata['url']
        path_format_arguments = {
            'vendorID': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.delete(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    delete_vendor.metadata = {'url': '/api/v3/vendor/{vendorID}'}

    def get_vendors(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves an array of vendors. The array of vendors can be filtered by
        a number of criteria. See the filter property for more details.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.VendorsVendorResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_vendors.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[VendorsVendorResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_vendors.metadata = {'url': '/api/v3/vendor'}

    def create_vendor(
            self, vendor_dto=None, custom_headers=None, raw=False, **operation_config):
        """Creates a vendor.

        :param vendor_dto: Data representing the vendor being created
        :type vendor_dto: ~energycap.sdk.models.VendorsVendorRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: VendorsVendorResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.VendorsVendorResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.create_vendor.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if vendor_dto is not None:
            body_content = self._serialize.body(vendor_dto, 'VendorsVendorRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('VendorsVendorResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    create_vendor.metadata = {'url': '/api/v3/vendor'}

    def edit_vendor(
            self, vendor_id, vendor_dto=None, custom_headers=None, raw=False, **operation_config):
        """Edits a vendor.

        :param vendor_id: The identifier for the vendor being edited
        :type vendor_id: int
        :param vendor_dto: Data representing the vendor being edited
        :type vendor_dto: ~energycap.sdk.models.VendorsVendorRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: VendorsVendorResponseDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.VendorsVendorResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_vendor.metadata['url']
        path_format_arguments = {
            'vendorId': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if vendor_dto is not None:
            body_content = self._serialize.body(vendor_dto, 'VendorsVendorRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('VendorsVendorResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_vendor.metadata = {'url': '/api/v3/vendor/{vendorId}'}

    def get_vendor_udfs(
            self, vendor_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for a particular vendor.

        :param vendor_id: The vendor identifier for which UDF values are being
         retrieved
        :type vendor_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_vendor_udfs.metadata['url']
        path_format_arguments = {
            'vendorId': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_vendor_udfs.metadata = {'url': '/api/v3/vendor/{vendorId}/udf'}

    def edit_vendor_ud_fs(
            self, vendor_id, udfs=None, custom_headers=None, raw=False, **operation_config):
        """Edits a vendor UDF values.

        :param vendor_id: The identifier for the vendor being edited
        :type vendor_id: int
        :param udfs: Data representing the vendor UDFs being edited
        :type udfs: list[~energycap.sdk.models.UDFUDFValueDTO]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFUDFFieldChildDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_vendor_ud_fs.metadata['url']
        path_format_arguments = {
            'vendorId': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if udfs is not None:
            body_content = self._serialize.body(udfs, '[UDFUDFValueDTO]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFUDFFieldChildDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_vendor_ud_fs.metadata = {'url': '/api/v3/vendor/{vendorId}/udf'}

    def get_vendors_udfs(
            self, filter=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves all UDFs for all vendors.

        :param filter:
        :type filter: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.UDFVendorUDFResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_vendors_udfs.metadata['url']

        # Construct parameters
        query_parameters = {}
        if filter is not None:
            query_parameters['filter'] = self._serialize.query("filter", filter, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[UDFVendorUDFResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_vendors_udfs.metadata = {'url': '/api/v3/vendor/udf'}

    def get_vendor_contracts(
            self, vendor_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves all Contracts for a particular vendor.

        :param vendor_id: The vendor identifier for which Contracts are being
         retrieved
        :type vendor_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.VendorsVendorContractResponseDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_vendor_contracts.metadata['url']
        path_format_arguments = {
            'vendorId': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[VendorsVendorContractResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_vendor_contracts.metadata = {'url': '/api/v3/vendor/{vendorId}/contract'}

    def add_accounts(
            self, vendor_id, account_ids=None, custom_headers=None, raw=False, **operation_config):
        """Add accounts to a vendor.

        :param vendor_id: The identifier for the vendor to which accounts are
         being added
        :type vendor_id: int
        :param account_ids: Data representing the accounts being added
        :type account_ids: list[int]
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: None or ClientRawResponse if raw=true
        :rtype: None or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.add_accounts.metadata['url']
        path_format_arguments = {
            'vendorId': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if account_ids is not None:
            body_content = self._serialize.body(account_ids, '[int]')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
    add_accounts.metadata = {'url': '/api/v3/vendor/{vendorId}/account'}

    def get_vendor_actual_yearly(
            self, vendor_id, number_of_years=None, custom_headers=None, raw=False, **operation_config):
        """Retrieves rolled up actual billing data by vendor for a given number of
        years- Result includes cost, use, unit cost on an yearly basis. It also
        includes commodity level break up.

        :param vendor_id: The vendor to retrieve data for
        :type vendor_id: int
        :param number_of_years: Number of years of data to return includes the
         current year - Defaults to 4
        :type number_of_years: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: VendorDigestVendorDigestActualYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.VendorDigestVendorDigestActualYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_vendor_actual_yearly.metadata['url']
        path_format_arguments = {
            'vendorId': self._serialize.url("vendor_id", vendor_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}
        if number_of_years is not None:
            query_parameters['numberOfYears'] = self._serialize.query("number_of_years", number_of_years, 'int')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('VendorDigestVendorDigestActualYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_vendor_actual_yearly.metadata = {'url': '/api/v3/vendor/{vendorId}/digest/actual/yearly'}

    def get_widgets(
            self, custom_headers=None, raw=False, **operation_config):
        """Retrieves an array of widgets.

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.DashboardWidgetResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_widgets.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[DashboardWidgetResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_widgets.metadata = {'url': '/api/v3/widget'}

    def get_eui_chart_data1(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves energy use intensity data for a specific widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.PlaceDigestPlaceDigestEnergyUseIntensityDTO]
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_eui_chart_data1.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[PlaceDigestPlaceDigestEnergyUseIntensityDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_eui_chart_data1.metadata = {'url': '/api/v3/widget/eui/{specificWidgetId}'}

    def get_missing_bills(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Returns gaps in billing data for a date range specified by the  Bill
        Date Range filter.
        A gap is defined as at least one day that falls within the account's
        service dates and
        not included in any bill for that account meter.
        The Allowable Gap Days filter specifies the number of days that must be
        in the gap
        before it will be included in the results.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.WidgetMissingBillWidgetDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_missing_bills.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[WidgetMissingBillWidgetDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_missing_bills.metadata = {'url': '/api/v3/widget/missingbills/{specificWidgetId}'}

    def get_bills_overdue_from_vendor(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves data about bills that have not yet been received from
        vendors.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype:
         list[~energycap.sdk.models.WidgetBillsOverdueFromVendorResponseDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bills_overdue_from_vendor.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[WidgetBillsOverdueFromVendorResponseDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bills_overdue_from_vendor.metadata = {'url': '/api/v3/widget/billsoverduefromvendor/{specificWidgetId}'}

    def get_interval_data(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves interval data for a specific widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.ReadingReadingDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_interval_data.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[ReadingReadingDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_interval_data.metadata = {'url': '/api/v3/widget/intervaldata/{specificWidgetId}'}

    def get_map_places(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves map data for a specific map widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.WidgetMapPlaceDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_map_places.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[WidgetMapPlaceDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_map_places.metadata = {'url': '/api/v3/widget/map/{specificWidgetId}'}

    def get_most_expensive_buildings(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves the most expensive building data for a specific widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.WidgetPlaceCostDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_most_expensive_buildings.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[WidgetPlaceCostDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_most_expensive_buildings.metadata = {'url': '/api/v3/widget/mostexpensivebuildings/{specificWidgetId}'}

    def get_monthly_use_trend(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves monthly global use trend for a dashboard widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_monthly_use_trend.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_monthly_use_trend.metadata = {'url': '/api/v3/widget/monthlyusetrend/{specificWidgetId}'}

    def get_monthly_cost_trend(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves monthly cost trend for a dashboard widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_monthly_cost_trend.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_monthly_cost_trend.metadata = {'url': '/api/v3/widget/monthlycosttrend/{specificWidgetId}'}

    def get_carbon_footprint(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves the carbon footprint data for a specific widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestGHGYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestGHGYearlyResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_carbon_footprint.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestGHGYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_carbon_footprint.metadata = {'url': '/api/v3/widget/carbonfootprint/{specificWidgetId}'}

    def get_cost_avoiadance_program_savings(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Returns Cost Avoidance Savings Program-to-date. The user can set the
        topmost place and/or commodity filters.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestSavingsYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestSavingsYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_cost_avoiadance_program_savings.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestSavingsYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_cost_avoiadance_program_savings.metadata = {'url': '/api/v3/widget/programsavings/{specificWidgetId}'}

    def monthly_cap_trend(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """

        :param specific_widget_id:
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.CostAvoidanceMonthlyCAPTrendDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.monthly_cap_trend.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[CostAvoidanceMonthlyCAPTrendDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    monthly_cap_trend.metadata = {'url': '/api/v3/widget/monthlysavingstrend/{specificWidgetId}'}

    def get_most_expensive_meters(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves the most expensive meters data for a specific widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: list or ClientRawResponse if raw=true
        :rtype: list[~energycap.sdk.models.WidgetMeterCostDTO] or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_most_expensive_meters.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('[WidgetMeterCostDTO]', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_most_expensive_meters.metadata = {'url': '/api/v3/widget/mostexpensivemeters/{specificWidgetId}'}

    def get_commodity_cost(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves data for commodity cost dashboard widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_commodity_cost.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestActualAndCalendarizedMonthlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_commodity_cost.metadata = {'url': '/api/v3/widget/commoditycost/{specificWidgetId}'}

    def cost_avoidance_by_commodity(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves data for cost avoidance by commodity dashboard widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: PlaceDigestPlaceDigestSavingsYearlyResponseDTO or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.PlaceDigestPlaceDigestSavingsYearlyResponseDTO
         or ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.cost_avoidance_by_commodity.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('PlaceDigestPlaceDigestSavingsYearlyResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    cost_avoidance_by_commodity.metadata = {'url': '/api/v3/widget/costavoidancebycommodity/{specificWidgetId}'}

    def get_bill_count(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """Retrieves a count of bills for the specific widget.

        :param specific_widget_id: Specific widget identifier to get the data
         for
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: CommonSingleValueDTO or ClientRawResponse if raw=true
        :rtype: ~energycap.sdk.models.CommonSingleValueDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_bill_count.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('CommonSingleValueDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_bill_count.metadata = {'url': '/api/v3/widget/billcount/{specificWidgetId}'}

    def execute_sproc_widget(
            self, specific_widget_id, custom_headers=None, raw=False, **operation_config):
        """

        :param specific_widget_id:
        :type specific_widget_id: int
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: WidgetPerformanceComparisonDataResponseDto or
         ClientRawResponse if raw=true
        :rtype:
         ~energycap.sdk.models.WidgetPerformanceComparisonDataResponseDto or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.execute_sproc_widget.metadata['url']
        path_format_arguments = {
            'specificWidgetId': self._serialize.url("specific_widget_id", specific_widget_id, 'int')
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('WidgetPerformanceComparisonDataResponseDto', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    execute_sproc_widget.metadata = {'url': '/api/v3/widget/exec/{specificWidgetId}'}

    def get_batch_workflow(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: WorkflowBatchWorkflowSettingsDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.WorkflowBatchWorkflowSettingsDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_batch_workflow.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('WorkflowBatchWorkflowSettingsDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_batch_workflow.metadata = {'url': '/api/v3/workflow/batch'}

    def get_export_workflow(
            self, export_mode=None, custom_headers=None, raw=False, **operation_config):
        """

        :param export_mode:
        :type export_mode: str
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: WorkflowExportWorkflowSettingsDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.WorkflowExportWorkflowSettingsDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_export_workflow.metadata['url']

        # Construct parameters
        query_parameters = {}
        if export_mode is not None:
            query_parameters['exportMode'] = self._serialize.query("export_mode", export_mode, 'str')

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('WorkflowExportWorkflowSettingsDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_export_workflow.metadata = {'url': '/api/v3/workflow/export'}

    def get_approval_workflow(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: WorkflowApprovalWorkflowSettingsDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.WorkflowApprovalWorkflowSettingsDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_approval_workflow.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('WorkflowApprovalWorkflowSettingsDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_approval_workflow.metadata = {'url': '/api/v3/workflow/approval'}

    def get_workflow(
            self, custom_headers=None, raw=False, **operation_config):
        """

        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: WorkflowWorkflowSettingsResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.WorkflowWorkflowSettingsResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.get_workflow.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.get(url, query_parameters)
        response = self._client.send(request, header_parameters, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('WorkflowWorkflowSettingsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    get_workflow.metadata = {'url': '/api/v3/workflow'}

    def edit_workflow(
            self, request_dto=None, custom_headers=None, raw=False, **operation_config):
        """

        :param request_dto:
        :type request_dto:
         ~energycap.sdk.models.WorkflowWorkflowSettingsRequestDTO
        :param dict custom_headers: headers that will be added to the request
        :param bool raw: returns the direct response alongside the
         deserialized response
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :return: WorkflowWorkflowSettingsResponseDTO or ClientRawResponse if
         raw=true
        :rtype: ~energycap.sdk.models.WorkflowWorkflowSettingsResponseDTO or
         ~msrest.pipeline.ClientRawResponse
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        # Construct URL
        url = self.edit_workflow.metadata['url']

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json-patch+json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct body
        if request_dto is not None:
            body_content = self._serialize.body(request_dto, 'WorkflowWorkflowSettingsRequestDTO')
        else:
            body_content = None

        # Construct and send request
        request = self._client.put(url, query_parameters)
        response = self._client.send(
            request, header_parameters, body_content, stream=False, **operation_config)

        if response.status_code not in [200]:
            raise HttpOperationError(self._deserialize, response)

        deserialized = None

        if response.status_code == 200:
            deserialized = self._deserialize('WorkflowWorkflowSettingsResponseDTO', response)

        if raw:
            client_raw_response = ClientRawResponse(deserialized, response)
            return client_raw_response

        return deserialized
    edit_workflow.metadata = {'url': '/api/v3/workflow'}
